#import "DiscoverView.h"
#import "ePOS-Print.h"
#import "ShowMsg.h"

#define DISCOVERY_INTERVAL  0.5

typedef enum : NSInteger {
    SelectedInterfaceIndexWiFi = 0,
    SelectedInterfaceIndexBluetooth = 1,
} SelectedInterfaceIndex;

@interface DiscoverView ()
- (void)reloadPrinterView;
- (void)timerFindPrinter:(NSTimer*)timer;
- (void)findStart;
@end

@implementation DiscoverView

- (id)initWithConnectionType:(int)connectionType {
    self = [super init];
    if (self) {
        printerList_ = [[NSArray alloc] init];
        initConnectionType_ = connectionType;
    }
    return self;
}

- (void)dealloc
{
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    if ([self respondsToSelector:@selector(edgesForExtendedLayout)]) {
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    
    //init device type
    switch(initConnectionType_){
        case EPOS_OC_DEVTYPE_TCP:
            _deviceType.selectedSegmentIndex = 0;
            break;
        default:
            _deviceType.selectedSegmentIndex = 1;
            break;
    }
    
    //find start
    [self findStart];
}

- (void)viewDidUnload
{
    //stop old finder
    if(timer_ != nil){
        [timer_ invalidate];
        timer_ = nil;
    }
    [EpsonIoFinder stop];
    
    [super viewDidUnload];
}

//get table section number(NSTableView)
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

//get table row number(NSTableView)
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
    return [printerList_ count];
}

//get table row(cell) item(NSTableView)
- (UITableViewCell*)tableView:(UITableView*)tableView
        cellForRowAtIndexPath:(NSIndexPath*)indexPath
{
    //TableViewCellの生成
    static NSString* identifier = @"basis-cell";
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:identifier];
    if(nil == cell){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:identifier] ;
    }
    if(indexPath.row >= 0 && indexPath.row < [printerList_ count]){
        cell.textLabel.text = [[printerList_ objectAtIndex:indexPath.row] printerName];
        EpsonIoDeviceInfo* devInfo = (EpsonIoDeviceInfo*)[printerList_ objectAtIndex:indexPath.row];
        int connectionType = [self connectionTypeFromSegmentedControlIndex:_deviceType.selectedSegmentIndex];
        cell.detailTextLabel.text = [self macAddress:devInfo connectionType:connectionType];
    }
    return cell;
}

//select table(row) item(NSTableView)
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //return settings
    if(indexPath.row >= 0 && indexPath.row < [printerList_ count]){
        [tableView deselectRowAtIndexPath:indexPath animated:NO];
        
        if(self.delegate != nil){
            EpsonIoDeviceInfo* devInfo = (EpsonIoDeviceInfo*)[printerList_ objectAtIndex:indexPath.row];
            int connectionType = [self connectionTypeFromSegmentedControlIndex:_deviceType.selectedSegmentIndex];
            NSString* macAddress = [self macAddress:devInfo connectionType:connectionType];
            [self.delegate onSelectPrinter:connectionType macAddress:macAddress address:devInfo.deviceName];
        }
    }
    
    [self.navigationController popViewControllerAnimated:YES];
}

//update printe view(call find thread)
- (void)reloadPrinterView
{
    [_printerView reloadData];
}

//change connection type
- (IBAction)changeConnectionType:(id)sender {
    [self findStart];
}

//find printer timer
- (void)timerFindPrinter:(NSTimer*)timer
{
    int result = 0;
    NSArray *items = [EpsonIoFinder getDeviceInfoList:&result FilterOption:EPSONIO_OC_PARAM_DEFAULT];
    if(items != nil && result ==EPSONIO_OC_SUCCESS){
        bool change = NO;
        if([printerList_ count] != [items count]){
            printerList_ = items;
            change = YES;
        }
        if(change){
            [self performSelectorOnMainThread:@selector(reloadPrinterView) withObject:nil waitUntilDone:YES];
        }
    }
}

//find start/restart
- (void)findStart
{
    //stop old finder
    [EpsonIoFinder stop];
    
    //stop find timer
    if(timer_ != nil){
        [timer_ invalidate];
        timer_ = nil;
    }

    //clear list
    printerList_ = [[NSArray alloc] init];
    if(_printerView != nil){
        [_printerView reloadData];
    }
    
    //find device type and find option
    int connectionType;
    NSString *option = nil;
    switch(_deviceType.selectedSegmentIndex){
        case 0:
            connectionType = EPSONIO_OC_DEVTYPE_TCP;
            option = @"255.255.255.255";
            break;
        default:
            connectionType = EPSONIO_OC_DEVTYPE_BLUETOOTH;
            break;
    }

    //find start
    int result = [EpsonIoFinder start:connectionType FindOption:option];
    if(result != EPSONIO_OC_SUCCESS){
        [ShowMsg showExceptionEpsonIo:result method:@"start"];
        return ;
    }
    
    //start timer
    timer_ = [NSTimer scheduledTimerWithTimeInterval:DISCOVERY_INTERVAL
                                              target:self
                                            selector:@selector(timerFindPrinter:)
                                            userInfo:nil
                                             repeats:YES];
}

//----------------------------------------------------------------------------//
#pragma mark - Utiltiy Method
//----------------------------------------------------------------------------//
- (NSString *)macAddress:(EpsonIoDeviceInfo*)deviceInfo connectionType:(int)connectionType
{
    NSString* macAddress = nil;
    if(connectionType == EPOS_OC_DEVTYPE_TCP){
        macAddress = [NSString stringWithString:deviceInfo.macAddress];
    } else if(connectionType == EPOS_OC_DEVTYPE_BLUETOOTH){
        macAddress = [NSString stringWithString:deviceInfo.deviceName];
    }
    return macAddress;
}

- (int)connectionTypeFromSegmentedControlIndex:(NSInteger)index
{
    switch(index){
        case SelectedInterfaceIndexWiFi:
            return EPOS_OC_DEVTYPE_TCP;
        case SelectedInterfaceIndexBluetooth:
            return EPOS_OC_DEVTYPE_BLUETOOTH;
        default:
            return EPOS_OC_DEVTYPE_TCP;
    }
}

@end
