#import "PrintQrViewController.h"
#import "ePOS-Print.h"
#import "PrintQrCodeTask.h"
#import "IndicatorView.h"
#import "DiscoverView.h"
#import "PrinterInfoController.h"
#import "Utility.h"
#import "ShowMsg.h"

static const long PRINT_QRCODE_SIZE             = 5;

@interface PrintQrViewController () <SelectPrinterDelegate>
{
    NSString* printerModel_;
    NSString* address_;
    NSString* MACAddress_;
    int connectionType_;
    int lang_;
    IndicatorView* pqrIndicatorView_;
}
@end

@implementation PrintQrViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    if ([self respondsToSelector:@selector(edgesForExtendedLayout)]){
        // For iOS7
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    
    self.title = NSLocalizedString(@"SAMPLE_Title_PrintQRCode", @"");
    
    printerModel_ = @"";
    address_ = @"";
    MACAddress_ = @"";
    connectionType_ = EPOS_OC_DEVTYPE_TCP;
    lang_ = EPOS_OC_MODEL_ANK;
    [Utility UIButtonDisableWithColor:self.printUIButton];
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



//----------------------------------------------------------------------------//
#pragma mark - press findUIButton Method (Find Printer)
//----------------------------------------------------------------------------//
- (IBAction)pressFindUIButton:(id)sender {
    // segue find view controller
    DiscoverView *vc = [[DiscoverView alloc] initWithConnectionType:connectionType_];
    vc.delegate = self;
    [self.navigationController pushViewController:vc animated:YES];
    
}

//----------------------------------------------------------------------------//
#pragma mark - press pressPrintUIButton Method (Print QR Code )
//----------------------------------------------------------------------------//
- (IBAction)pressPrintUIButton:(id)sender
{
    [self showIndicator];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
        
        PrintQrCodeTask* printQrCodeTask = [[PrintQrCodeTask alloc] initWithPrinterModel:printerModel_
                                                                                 address:address_
                                                                              MACAddress:MACAddress_
                                                                          connectionType:connectionType_
                                                                                    lang:lang_];
        
        BOOL isSuccessPrint = [printQrCodeTask printQr:PRINT_QRCODE_SIZE];
        dispatch_async(dispatch_get_main_queue(),  ^{
            
            [self hideIndicator];
            if(!isSuccessPrint){
                [self showPrintQrErrorMsg];
            }
        });
    });
}

//----------------------------------------------------------------------------//
#pragma mark - Indicator Method
//----------------------------------------------------------------------------//
- (void)showIndicator
{
    pqrIndicatorView_ = [[IndicatorView alloc] init];
    [pqrIndicatorView_ show:[[UIApplication sharedApplication] keyWindow]];
}

- (void)hideIndicator
{
    [pqrIndicatorView_ hide];
    pqrIndicatorView_ = nil;
}

//----------------------------------------------------------------------------//
#pragma mark - Error Message Method
//----------------------------------------------------------------------------//
- (void)showPrintQrErrorMsg
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"dialog_title_error", @"")
                                                    message:NSLocalizedString(@"PQR_msg_PrintError", @"")
                                                   delegate:nil
                                          cancelButtonTitle:nil
                                          otherButtonTitles:NSLocalizedString(@"dialog_button_ok", @""), nil];
    [alert show];
}



//----------------------------------------------------------------------------//
#pragma mark - SelectPrinterDelegate
//----------------------------------------------------------------------------//
- (void)onSelectPrinter:(int)connectionType macAddress:(NSString*)macAddress address:(NSString*)address
{
    NSString *interfaceName = @"";
    // Use PrinterInfoController to set printerModel and lang
    PrinterInfoController *printerInfo = [[PrinterInfoController alloc] init];
    BOOL rc = [printerInfo getPrinterNameWithDeviceName:address connectionType:connectionType];
    if (rc == YES) {
        printerModel_ = [printerInfo printerModel];
        lang_ = [printerInfo language];
        connectionType_ = connectionType;
        MACAddress_ = macAddress;
        address_ = address;
        interfaceName = [Utility interfaceString:connectionType_];
        [Utility UIButtonEnableWithColor:self.printUIButton];
    }else{
        [self showPrintQrErrorMsg];
        printerModel_ = @"";
        lang_ = EPOS_OC_MODEL_ANK;
        connectionType_ = connectionType;
        MACAddress_ = @"";
        address_ = @"";
        
        [Utility UIButtonDisableWithColor:self.printUIButton];
        
    }

    _targetInterfaceUILabel.text = interfaceName;
    _targetDeviceNameUILabel.text = printerModel_;
    _targetMACAddressUILabel.text = MACAddress_;
    
}

@end
