#import "ePOS-Print.h"
#import "PrintSampleTask.h"

static const long       PRINT_SAMPLE_TIMEOUT  = 5000;

static NSString* const  PRINT_LINE                      = @"--------------------";
static NSString* const  PRINT_TITLE                     = @"Sample Print";
static NSString* const  PRINT_WIFI_ADDRESS              = @"Network Address:";
static NSString* const  PRINT_BLUETOOTH_ADDRESS         = @"Bluetooth Address:";
static NSString* const  PRINT_MESSAGE                   = @"Print successfully!!";

@interface PrintSampleTask ()
{
    NSString* printerModel_;
    NSString* MACAddress_;
    int connectionType_;
}
@end

@implementation PrintSampleTask

- (instancetype)init
{
    @throw [NSException exceptionWithName:@"Don't use init"
                                   reason:@"Please use designated initializer"
                                 userInfo:nil];
    return nil;
}

// Designated initializer
- (instancetype)initWithPrinterModel:(NSString*)printerModel macAddress:(NSString *)macAddress connectionType:(int)connectionType
{
    self = [super init];
    if(self){
        printerModel_ = printerModel;
        MACAddress_ = macAddress;
        connectionType_ = connectionType;
    }
    return self;
}

//----------------------------------------------------------------------------//
#pragma mark - Print Method
//----------------------------------------------------------------------------//
- (BOOL)printSample
{
    EposPrint *eposPrint = [[EposPrint alloc] init];
    if(!eposPrint){
        return NO;
    }

    // Open printer
    int result = [eposPrint openPrinter:connectionType_ DeviceName:MACAddress_];
    if(result != EPOS_OC_SUCCESS){
        return NO;
    }
    
    // Create builder
    EposBuilder *builder = [[EposBuilder alloc] initWithPrinterModel:printerModel_ Lang:EPOS_OC_MODEL_ANK];
    if(!builder){
        return NO;
    }
    
    // Make print data
    BOOL isMakeTextSuccess = [self makePrintText:builder];
    if(!isMakeTextSuccess){
        return NO;
    }
    
    // Send print data
    unsigned long status = 0;
    result = [eposPrint sendData:builder
                             Timeout:PRINT_SAMPLE_TIMEOUT
                              Status:&status];
    if(result != EPOS_OC_SUCCESS){
        return NO;
    }
    
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Make Print Text
//----------------------------------------------------------------------------//
- (BOOL)makePrintText:(EposBuilder*)builder
{
    if(!builder){
        return NO;
    }
    
    BOOL isSuccess = [self makeHeaderText:builder];
    if(!isSuccess){
        return NO;
    }
    
    isSuccess = [self makeBodyText:builder];
    if(!isSuccess){
        return NO;
    }
    
    int result = [builder addCut:EPOS_OC_CUT_FEED];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Make Header Text
//----------------------------------------------------------------------------//
- (BOOL)makeHeaderText:(EposBuilder*)builder
{
    if(!builder){
        return NO;
    }
    
    int result = EPOS_OC_ERR_FAILURE;
    result = [builder addText:PRINT_LINE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    [builder addText:PRINT_TITLE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addText:PRINT_LINE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:2];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    return YES;
}


//----------------------------------------------------------------------------//
#pragma mark - Make Body Text
//----------------------------------------------------------------------------//
- (BOOL)makeBodyText:(EposBuilder*)builder
{
    if(!builder){
        return NO;
    }
    
    int result = [builder addText:printerModel_];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    NSString* interfaceText = [self interfaceText];
    result = [builder addText:interfaceText];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addText:MACAddress_];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:5];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addTextAlign:EPOS_OC_ALIGN_CENTER];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addText:PRINT_MESSAGE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:2];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Make Interface Text
//----------------------------------------------------------------------------//
- (NSString*)interfaceText
{
    switch (connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:  return PRINT_WIFI_ADDRESS;
        case EPOS_OC_DEVTYPE_BLUETOOTH: return PRINT_BLUETOOTH_ADDRESS;
        default:                   return @"";
    }
}

@end
