﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class BarcodePage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;
        const int HRI_NONE_INDEX = 0;
        const int HRI_ABOVE_INDEX = 1;
        const int HRI_BELOW_INDEX = 2;
        const int HRI_BOTH_INDEX = 3;
        const int FONT_A_INDEX = 0;
        const int FONT_B_INDEX = 1;
        const int FONT_C_INDEX = 2;
        const int FONT_D_INDEX = 3;
        const int FONT_E_INDEX = 4;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;
        private string defaultBarcodeData = "";

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public BarcodePage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if (printer == null)
            {
                return;
            }
            else
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //init type list
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_upca"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_upce"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_ean13"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_jan13"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_ean8"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_jan8"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_code39"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_itf"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_codabar"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_code93"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_code128"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_gs1128"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_gs1om"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_gs1tr"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_gs1li"));
            this.BarcodeType_ComboBox.Items.Add(loader.GetString("bartype_gs1ex"));

            //init HRI list
            this.BarcodeHRI_ComboBox.Items.Add(loader.GetString("hri_none"));
            this.BarcodeHRI_ComboBox.Items.Add(loader.GetString("hri_above"));
            this.BarcodeHRI_ComboBox.Items.Add(loader.GetString("hri_below"));
            this.BarcodeHRI_ComboBox.Items.Add(loader.GetString("hri_both"));

            //init font list
            this.BarcodeFont_ComboBox.Items.Add(loader.GetString("font_a"));
            this.BarcodeFont_ComboBox.Items.Add(loader.GetString("font_b"));
            this.BarcodeFont_ComboBox.Items.Add(loader.GetString("font_c"));
            this.BarcodeFont_ComboBox.Items.Add(loader.GetString("font_d"));
            this.BarcodeFont_ComboBox.Items.Add(loader.GetString("font_e"));

            //default setting
            this.BarcodeType_ComboBox.SelectedIndex = 0;
            this.BarcodeHRI_ComboBox.SelectedIndex = HRI_BELOW_INDEX;
            this.BarcodeFont_ComboBox.SelectedIndex = FONT_A_INDEX;

            this.ModuleSizeWidth_TxtBox.Text = "3";
            this.ModuleSizeHeight_TxtBox.Text = "162";

            setDefaultBarcodeData();
        }

        private void BarcodeType_Selected(object sender, SelectionChangedEventArgs e)
        {
            bool isChangedBarcodeData = false;

            if (defaultBarcodeData == BarcodeData_TxtBox.Text)
            {
                isChangedBarcodeData = true;
            }

            if (isChangedBarcodeData)
            {
                // update barcode data
                setDefaultBarcodeData();
            }
            else
            {
                // not update
                return;
            }
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await printBarcode();
            this.IsEnabled = true;
        }

        private async Task printBarcode()
        {
            if (string.IsNullOrWhiteSpace(BarcodeData_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_notext"));
                return;
            }

            Builder builder = null;
            Print printer = null;
            PrinterStatus sendStatus;
            sendStatus.printerStatus = 0;
            sendStatus.batteryStatus = 0;
            string method = "";

            try
            {
                method = "Builder";
                builder = new Builder(MainPage.getPrinterName(), MainPage.getLanguage());

                //add command
                method = "AddBarcode";
                builder.AddBarcode(getBuilderData(),
                                   getBuilderType(),
                                   getBuilderHri(),
                                   getBuilderFont(),
                                   getBuilderWidth(),
                                   getBuilderHeight());

                //send builder data
                try
                {
                    printer = MainPage.getPrinter();
                    sendStatus = await printer.SendDataAsync(builder, SEND_TIMEOUT);
                    showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus);
                }
                catch (Exception ex)
                {
                    PrinterStatus status = printer.GetPrinterStatus(ex.HResult);
                    showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus);
                }
            }
            catch (Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }

            //remove builder
            if(builder !=null)
            {
                try
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
                catch (Exception)
                {
                    builder = null;
                }
            }
        }

        private string getBuilderData()
        {
            return BarcodeData_TxtBox.Text;
        }

        private BarcodeType getBuilderType()
        {
            BarcodeType[] typelist = {
                                 BarcodeType.BARCODE_UPC_A,
                                 BarcodeType.BARCODE_UPC_E,
                                 BarcodeType.BARCODE_EAN13,
                                 BarcodeType.BARCODE_JAN13,
                                 BarcodeType.BARCODE_EAN8,
                                 BarcodeType.BARCODE_JAN8,
                                 BarcodeType.BARCODE_CODE39,
                                 BarcodeType.BARCODE_ITF,
                                 BarcodeType.BARCODE_CODABAR,
                                 BarcodeType.BARCODE_CODE93,
                                 BarcodeType.BARCODE_CODE128,
                                 BarcodeType.BARCODE_GS1_128,
                                 BarcodeType.BARCODE_GS1_DATABAR_OMNIDIRECTIONAL,
                                 BarcodeType.BARCODE_GS1_DATABAR_TRUNCATED,
                                 BarcodeType.BARCODE_GS1_DATABAR_LIMITED,
                                 BarcodeType.BARCODE_GS1_DATABAR_EXPANDED
                             };

            return typelist[BarcodeType_ComboBox.SelectedIndex];
        }

        private Hri getBuilderHri()
        {
            switch(BarcodeHRI_ComboBox.SelectedIndex)
            {
                case HRI_ABOVE_INDEX:
                    return Hri.HRI_ABOVE;
                case HRI_BELOW_INDEX:
                    return Hri.HRI_BELOW;
                case HRI_BOTH_INDEX:
                    return Hri.HRI_BOTH;
                case HRI_NONE_INDEX:
                default:
                    return Hri.HRI_NONE;
            }
        }

        private Font getBuilderFont()
        {
            switch(BarcodeFont_ComboBox.SelectedIndex)
            {
                case FONT_B_INDEX:
                    return Font.FONT_B;
                case FONT_C_INDEX:
                    return Font.FONT_C;
                case FONT_D_INDEX:
                    return Font.FONT_D;
                case FONT_E_INDEX:
                    return Font.FONT_E;
                case FONT_A_INDEX:
                default:
                    return Font.FONT_A;
            }
        }

        private int getBuilderWidth()
        {
            try
            {
                return int.Parse(ModuleSizeWidth_TxtBox.Text);
            }
            catch(Exception)
            {
                return 0;
            }
        }

        private int getBuilderHeight()
        {
            try
            {
                return int.Parse(ModuleSizeHeight_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private void setDefaultBarcodeData()
        {
            //update default data
            defaultBarcodeData = getDefaultBarcodeData();

            //update ui
            BarcodeData_TxtBox.Text = defaultBarcodeData;
        }

        private string getDefaultBarcodeData()
        {
            string[] barcodeDataList = {
                                        loader.GetString("bardata_upca"),
                                        loader.GetString("bardata_upce"),
                                        loader.GetString("bardata_ean13"),
                                        loader.GetString("bardata_jan13"),
                                        loader.GetString("bardata_ean8"),
                                        loader.GetString("bardata_jan8"),
                                        loader.GetString("bardata_code39"),
                                        loader.GetString("bardata_itf"),
                                        loader.GetString("bardata_codabar"),
                                        loader.GetString("bardata_code93"),
                                        loader.GetString("bardata_code128"),
                                        loader.GetString("bardata_gs1128"),
                                        loader.GetString("bardata_gs1om"),
                                        loader.GetString("bardata_gs1tr"),
                                        loader.GetString("bardata_gs1li"),
                                        loader.GetString("bardata_gs1ex")
                                       };

            return barcodeDataList[BarcodeType_ComboBox.SelectedIndex];
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }
}
