﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class GetPrinterNamePage : Page
    {
        static byte[] CMD_ESC_ENABLE_PRINTER = {
                                                   0x1b, 0x3d, 0x01
                                               };

        static byte[] CMD_GS_I_PRINTER_NAME = {
                                                  0x1d, 0x49, 0x43
                                              };

        static byte[] CMD_GS_I_ADDITIONAL_FONTS ={
                                                     0x1d, 0x49, 0x45
                                                 };

        const int RESPONSE_HEADER = 0x5f;
        const int RESPONSE_TERMINAL = 0x00;
        const int RESPONSE_TIMEOUT = 1000;
        const int RESPONSE_MAXBYTE = 128;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;
        private string fontName = null;
        private string printerName = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public GetPrinterNamePage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            //select default radiobutton
            switch (MainPage.getDeviceType())
            {
                case DevType.DEVTYPE_BLUETOOTH:
                    this.BT_RadioButton.IsChecked = true;
                    break;
                case DevType.DEVTYPE_TCP:
                default:
                    this.TCP_RadioButton.IsChecked = true;
                    break;
            }

            //init edit
            this.DeviceName_TxtBox.Text = MainPage.getDeviceName();
        }

        private async void GetPrinterName_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await getPrinterName();
            this.IsEnabled = true;
        }

        private async Task getPrinterName()
        {
            if (string.IsNullOrWhiteSpace(DeviceName_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_noaddress"));
                return;
            }

            IoDevType deviceType = IoDevType.TCP;

            if (BT_RadioButton.IsChecked == true)
            {
                deviceType = IoDevType.BLUETOOTH;
            }
            else
            {
                deviceType = IoDevType.TCP;
            }

            fontName = null;
            printerName = null;

            EpsonIo port = null;
            string method = "";

            try
            {
                byte[] receiveBuffer = new byte[RESPONSE_MAXBYTE];
                bool ret = false;

                //open
                port = new EpsonIo();
                method = "OpenAsync";
                await port.OpenAsync(deviceType, DeviceName_TxtBox.Text, "");

                await clearReceiveBuffer(port);

                //send command(esc/pose)
                //enable printer
                method = "Write";
                port.WriteBytes(CMD_ESC_ENABLE_PRINTER, 0, CMD_ESC_ENABLE_PRINTER.Length);
                await port.WriteAsync(RESPONSE_TIMEOUT);

                //printer name
                port.WriteBytes(CMD_GS_I_PRINTER_NAME, 0, CMD_GS_I_PRINTER_NAME.Length);
                await port.WriteAsync(RESPONSE_TIMEOUT);

                method = "Read";
                ret = await receiveResponse(port, receiveBuffer);

                if (ret)
                {
                    //additonal fonts
                    port.WriteBytes(CMD_GS_I_ADDITIONAL_FONTS, 0, CMD_GS_I_ADDITIONAL_FONTS.Length);
                    await port.WriteAsync(RESPONSE_TIMEOUT);

                    method = "Read";
                    ret = await receiveResponse(port, receiveBuffer);
                }

                //close
                method = "closeAsync";
                await port.CloseAsync();
            }
            catch (Exception ex)
            {
                showMsg.showIoException(ex.HResult, method);
                goto ERR;
            }

            if (printerName == null || fontName == null)
            {
                showMsg.showError(loader.GetString("errmsg_getname"));
            }
            else
            {
                showMsg.showPrinterName(printerName, getLanguageSpec(fontName));
            }

            return;

            ERR:
                if (port != null)
                {
                    try
                    {
                        await port.CloseAsync();
                        port = null;
                    }
                    catch
                    {
                        port = null;
                    }
                }
            }
        
        private async Task clearReceiveBuffer(EpsonIo port)
        {
            try
            {
                byte[] receiveData = new byte[RESPONSE_MAXBYTE];
                await port.ReadAsync(RESPONSE_MAXBYTE, RESPONSE_TIMEOUT);
                port.ReadBytes(out receiveData, receiveData, 0, RESPONSE_MAXBYTE);
            }
            catch(Exception ex)
            {
                throw ex;
            }
        }

        private async Task<bool> receiveResponse(EpsonIo port, byte[] receiveBuffer)
        {
            if( (null == receiveBuffer) || (0 >= receiveBuffer.Length))
            {
                return false;
            }

            int readSize = 0;

            //receive
            try
            {
                await port.ReadAsync(receiveBuffer.Length, RESPONSE_TIMEOUT);
                readSize = port.ReadBytes(out receiveBuffer, receiveBuffer, 0, receiveBuffer.Length);
            }
            catch(Exception ex)
            {
                throw ex;
            }

            if ((0 < readSize))
            {
                byte[] response = new byte[receiveBuffer.Length];
                receiveBuffer.CopyTo(response, 0);

                if(!analyzeResponse(response))
                {
                    return false;
                }
            }

            return true;
        }

        private Boolean analyzeResponse(byte[] response)
        {
            int currentPos = 0;

            //search 5f header
            for(currentPos = 0; currentPos < response.Length; currentPos++)
            {
                if(response[currentPos] == RESPONSE_HEADER)
                {
                    currentPos++;
                    break;
                }
            }

            if( currentPos >= response.Length)
            {
                return false;
            }

            // terminate check
            int endPos = 0;
            for(endPos = currentPos; endPos < response.Length; endPos++)
            {
                if(response[endPos] == RESPONSE_TERMINAL)
                {
                    break;
                }
            }

            //get Response string
            string responseString = null;
            try
            {
                responseString = System.Text.Encoding.GetEncoding("US-ASCII").GetString(response, currentPos, endPos - currentPos);
            }
            catch (Exception)
            {
                return false;
            }

            if (null == printerName)
            {
                printerName = responseString.TrimEnd(); 
            }
            else
            {
                fontName = responseString.TrimEnd(); 
            }

            return true;

        }

        private string getLanguageSpec(string font)
        {
            if( "" == font)
            {
                return loader.GetString("language_ank");
            }
            else if("KANJI JAPANESE" == font)
            {
                return loader.GetString("language_japanese");
            }
            else
            {
                return loader.GetString("language_other");
            }
        }
    }
}
