﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class LogSettingsPage : Page
    {
        const int LOG_DISABLE = 0;
        const int LOG_STORAGE = 1;
        const int LOG_TCP = 2;
        const int LOG_LEVEL_LOW = 0;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public LogSettingsPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            //Initialize ComboBox
            this.LogEnabled_ComboBox.Items.Add(loader.GetString("log_disable"));
            this.LogEnabled_ComboBox.Items.Add(loader.GetString("log_storage"));
            this.LogEnabled_ComboBox.Items.Add(loader.GetString("log_tcp"));

            this.LogLevel_ComboBox.Items.Add(loader.GetString("log_low"));

            //default setting
            this.LogEnabled_ComboBox.SelectedIndex = LOG_DISABLE;
            this.LogLevel_ComboBox.SelectedIndex = LOG_LEVEL_LOW;

            this.address_TxtBox.SelectedText = loader.GetString("logsettings_dummy_ip");
            this.port_TxtBox.SelectedText = loader.GetString("logsettings_dummy_port");
            this.logsize_TxtBox.SelectedText = loader.GetString("logsettings_dummy_logsize");
        }

        private void Setting_Button_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Log.SetLogSettings(getPeriod(), getEnabled(), getIpAddress(), getPort(), getLogSize(), getLogLevel());
            }
            catch(Exception ex)
            {
                showMsg.showEposException(ex.HResult, "SetLogSettings");
            }
        }

        private LogEnabled getEnabled()
        {
            switch (LogEnabled_ComboBox.SelectedIndex)
            {
                case LOG_STORAGE:
                    return LogEnabled.LOG_STORAGE;
                case LOG_TCP:
                    return LogEnabled.LOG_TCP;
                case LOG_DISABLE:
                default:
                    return LogEnabled.LOG_DISABLE;
            }
        }

        private String getIpAddress()
        {
            return address_TxtBox.Text;
        }

        private int getPort()
        {
            string port = port_TxtBox.Text;
            try
            {
                return int.Parse(port);
            }
            catch(Exception)
            {
                return 0;
            }
        }

        private int getLogSize()
        {
            string logsize = logsize_TxtBox.Text;
            try
            {
                return int.Parse(logsize);
            }
            catch (Exception)
            {
                return 0;
            }
            
        }

        private LogLevel getLogLevel()
        {
            switch (LogLevel_ComboBox.SelectedIndex)
            {
                case LOG_LEVEL_LOW:
                default:
                    return LogLevel.LOG_LOW;
            }
        }

        private LogPeriod getPeriod()
        {
            if (logsetting_ToggleButton.IsOn)
            {
                return LogPeriod.LOG_PERMANENT;
            }
            else
            {
                return LogPeriod.LOG_TEMPORARY;
            }
        }
    }
}