﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class MainPage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;
        const int CLOSE_TIMEOUT = 10 * 1000;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;

        private static Print printer = null;
        private static ModelLang lang = ModelLang.MODEL_LANG_ANK;
        private static string printerName = "TM-T88V";

        private static DevType deviceType = DevType.DEVTYPE_TCP;
        private static string deviceName = "192.168.192.168";

        public static void setBuilderOptions(Print setPrint, ModelLang selectLang, string selectPrinterName)
        {
            printer = setPrint;
            lang = selectLang;
            printerName = selectPrinterName;
        }

        public static void setSelectedPrinterDiscovery(DevType selectDevicetype, string selectDeviceName)
        {
            deviceType = selectDevicetype;
            deviceName = selectDeviceName;
        }

        public static Print getPrinter()
        {
            return printer;
        }
        public static ModelLang getLanguage()
        {
            return lang;
        }

        public static string getPrinterName()
        {
            return printerName;
        }

        public static DevType getDeviceType()
        {
            return deviceType;
        }

        public static string getDeviceName()
        {
            return deviceName;
        }

        public MainPage()
        {
            this.InitializeComponent();

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            if (printer != null)
            { 
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //update button state
            updateButtonStatus();
        }

        private void PrinterDiscovery_Button_Click(object sender, RoutedEventArgs e)
        {
            //Printer Discovery Page
            this.Frame.Navigate(typeof(PrinterDiscoveryPage), null);
        }

        private void Open_Button_Click(object sender, RoutedEventArgs e)
        {
            //Open Page
            this.Frame.Navigate(typeof(OpenPage), null);
        }

        private void Text_Button_Click(object sender, RoutedEventArgs e)
        {
            //Text Page
            this.Frame.Navigate(typeof(TextPage), null);
        }

        private void Image_Button_Click(object sender, RoutedEventArgs e)
        {
            //Image Page
            this.Frame.Navigate(typeof(ImagePage), null);
        }

        private void Barcode_Button_Click(object sender, RoutedEventArgs e)
        {
            //Barcode Page
            this.Frame.Navigate(typeof(BarcodePage), null);
        }

        private void TwoDCode_Button_Click(object sender, RoutedEventArgs e)
        {
            //2D Code Page
            this.Frame.Navigate(typeof(TwoDCodePage), null);
        }

        private void PageMode_Button_Click(object sender, RoutedEventArgs e)
        {
            //Page Mode Page
            this.Frame.Navigate(typeof(PageModePage), null);
        }

        private void Cut_Button_Click(object sender, RoutedEventArgs e)
        {
            //Cut Page
            this.Frame.Navigate(typeof(CutPage), null);
        }

        private void GetPrinterName_Button_Click(object sender, RoutedEventArgs e)
        {
            //Get Printer Name  Page
            this.Frame.Navigate(typeof(GetPrinterNamePage), null);
        }

        private void LogSettings_Button_Click(object sender, RoutedEventArgs e)
        {
            //Log settings Page
            this.Frame.Navigate(typeof(LogSettingsPage), null);
        }

        private async void GetStatus_Button_Click(object sender, RoutedEventArgs e)
        {
            //Get Printer Status
            this.IsEnabled = false;
            await showPrinterStatus();
            this.IsEnabled = true;
        }

        private async void Close_Button_Click(object sender, RoutedEventArgs e)
        {
            // Close
            await closePrinter();
            updateButtonStatus();
        }

        private async Task closePrinter()
        {
            if(printer != null)
            {
                try
                {
                    await printer.ClosePrinterAsync();
                    printer = null;
                }
                catch (Exception ex)
                {
                    showMsg.showEposException(ex.HResult, "ClosePrinterAsync");
                }
            }
        }

        private void updateButtonStatus()
        {
            Button[] groupControl1 = { Open_Button, 
                                       GetPrinterName_Button };

            Button[] groupControl2 = { Close_Button, Text_Button,
                                       Image_Button, Barcode_Button, 
                                       TwoDCode_Button, PageMode_Button,
                                       Cut_Button, GetStatus_Button };

            Button[] enableControl = null;
            Button[] disableControl = null;

            if (printer == null)
            {
                enableControl = groupControl1;
                disableControl = groupControl2;
            }
            else
            {
                enableControl = groupControl2;
                disableControl = groupControl1;
            }

            for (int i = 0; i < enableControl.Length; i++)
            {
                enableControl[i].IsEnabled = true;
            }

            for (int i = 0; i < disableControl.Length; i++)
            {
                disableControl[i].IsEnabled = false;
            }
        }

        private async Task showPrinterStatus()
        {
            PrinterStatus status;
            status.printerStatus = 0;
            status.batteryStatus = 0;
            string method = "";
            try
            {
                //Get printer status and battery status
                method = "GetStatusAsync";
                status = await printer.GetStatusAsync();
                showMsg.showStatus(EposHResult.HR_S_OK, status.printerStatus, status.batteryStatus);
            }
            catch(Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }
}
