﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class OpenPage : Page
    {
        const int LANG_ANK = 0;
        const int LANG_JAPANESE = 1;
        const int LANG_CHINESE = 2;
        const int LANG_TAIWAN = 3;
        const int LANG_KOREAN = 4;
        const int LANG_THAI = 5;
        const int LANG_SOUTHASIA = 6;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public OpenPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            //init printer list 
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_m10"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_m30"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p20"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p60"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p60ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p80"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t20"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t20ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t70"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t70ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t81ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t82"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t82ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t88v"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t90ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_u220"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_u330"));

            //init language list
            this.Language_ComboBox.Items.Add(loader.GetString("model_ank"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_japanese"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_chinese"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_taiwan"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_korean"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_thai"));
            this.Language_ComboBox.Items.Add(loader.GetString("model_southasia"));

            //default setting
            this.PrinterName_ComboBox.SelectedIndex = 0;
            this.Language_ComboBox.SelectedIndex = LANG_ANK;

            this.Interval_TxtBox.Text = "1000";
            //select default radiobutton
            switch(MainPage.getDeviceType())
            {
                case DevType.DEVTYPE_BLUETOOTH:
                    this.BT_RadioButton.IsChecked = true;
                    break;
                case DevType.DEVTYPE_TCP:
                default:
                    this.TCP_RadioButton.IsChecked = true;
                    break;
            }
            //init edit
            this.DeviceName_TxtBox.Text = MainPage.getDeviceName();
        }

        private async void Open_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await openPrinter();
            this.IsEnabled = true;
        }

        private async Task openPrinter()
        {
            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            if (string.IsNullOrWhiteSpace(DeviceName_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_noaddress"));
                return;
            }

            DevType deviceType = getDeviceType();
            Monitoring enabled = Monitoring.MONITORING_FALSE;

            if(getStatusMonitorEnable())
            {
                enabled = Monitoring.MONITORING_TRUE;
            }
            else
            {
                enabled = Monitoring.MONITORING_FALSE;
            }

            int interval = getStatusMonitorInterval();
            Print printer = null;
            try
            {
                //open
                printer = new Print();
                if(printer != null)
                {
                    printer.SetStatusChangeEventCallback(statusChangeEvent);
                    printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
                }
                await printer.OpenPrinterAsync(deviceType, getDeviceName(), enabled, interval);
            }
            catch(Exception ex)
            {
                printer = null;
                showMsg.showEposException(ex.HResult, "OpenPrinterAsync");
                return;
            }

            //set for builder class
            MainPage.setBuilderOptions(printer, getSelectedLanguge(), getSelectedPrinterName());

            if(this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
        }

        private DevType getDeviceType()
        {
            if(BT_RadioButton.IsChecked == true)
            {
                return DevType.DEVTYPE_BLUETOOTH;
            }
            else
            {
                return DevType.DEVTYPE_TCP;
            }
        }

        private string getDeviceName()
        {
            return DeviceName_TxtBox.Text;
        }

        private bool getStatusMonitorEnable()
        {
            return statusMonitor_ToggleButton.IsOn;
        }

        private int getStatusMonitorInterval()
        {
            try
            {
                return int.Parse(Interval_TxtBox.Text);
            }
            catch(Exception)
            {
                return 0;
            }
        }

        private string getSelectedPrinterName()
        {
            return PrinterName_ComboBox.SelectedItem.ToString();
        }

        private ModelLang getSelectedLanguge()
        {
            ModelLang language = ModelLang.MODEL_LANG_ANK;
            switch (Language_ComboBox.SelectedIndex)
            {
                case LANG_JAPANESE:
                    language = ModelLang.MODEL_LANG_JAPANESE;
                    break;
                case LANG_CHINESE:
                    language = ModelLang.MODEL_LANG_CHINESE;  
                    break;
                case LANG_TAIWAN:
                    language = ModelLang.MODEL_LANG_TAIWAN;       
                    break;
                case LANG_KOREAN:
                    language = ModelLang.MODEL_LANG_KOREAN;
                    break;
                case LANG_THAI:
                    language = ModelLang.MODEL_LANG_THAI;
                    break;
                case LANG_SOUTHASIA:
                    language = ModelLang.MODEL_LANG_SOUTHASIA;
                    break;
                case LANG_ANK:
                default:
                    language = ModelLang.MODEL_LANG_ANK;
                    break;
            }
            return language;
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }
}
