﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class PageModePage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public PageModePage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if (printer == null)
            {
                return;
            }
            else
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //default setting
            this.PrintCharacters_TxtBox.Text = loader.GetString("pagemode_edit_text");
            this.PrintAreaX_TxtBox.Text = "100";
            this.PrintAreaY_TxtBox.Text = "100";
            this.PrintAreaWidth_TxtBox.Text = "300";
            this.PrintAreaHeight_TxtBox.Text = "300";
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await printText();
            this.IsEnabled = true;
        }
        
        private async Task printText()
        {
            if (string.IsNullOrWhiteSpace(PrintCharacters_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_notext"));
                return;
            }

            Builder builder = null;
            Print printer = null;
            PrinterStatus sendStatus;
            sendStatus.printerStatus = 0;
            sendStatus.batteryStatus = 0;
            string method = "";

            try
            {
                //create builder
                method = "Builder";
                builder = new Builder(MainPage.getPrinterName(), MainPage.getLanguage());

                //add command
                method = "AddPageBegin";
                builder.AddPageBegin();

                method = "AddPageArea";
                builder.AddPageArea(getBuilderAreaX(), getBuilderAreaY(), getBuilderAreaWidth(), getBuilderAreaHeight());

                method = "AddText";
                builder.AddText(getBuilderText());

                method = "AddPageEnd";
                builder.AddPageEnd();

                //send builder data
                try
                {
                    printer = MainPage.getPrinter();
                    sendStatus = await printer.SendDataAsync(builder, SEND_TIMEOUT);
                    showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus);
                }
                catch (Exception ex)
                {
                    PrinterStatus status = printer.GetPrinterStatus(ex.HResult);
                    showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus);
                }
            }
            catch (Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }

            //remove builder
            if (builder != null)
            {
                try
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
                catch (Exception)
                {
                    builder = null;
                }
            }
        }

        private int getBuilderAreaX()
        {
            try
            {
                return int.Parse(PrintAreaX_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private int getBuilderAreaY()
        {
            try
            {
                return int.Parse(PrintAreaY_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private int getBuilderAreaWidth()
        {
            try
            {
                return int.Parse(PrintAreaWidth_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private int getBuilderAreaHeight()
        {
            try
            {
                return int.Parse(PrintAreaHeight_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private string getBuilderText()
        {
            return PrintCharacters_TxtBox.Text;
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }

    }
}