﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using Windows.UI.Core;
using System.Threading;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class PrinterDiscoveryPage : Page
    {
        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;
        private Timer updateListThread = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public PrinterDiscoveryPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if(printer != null)
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            DevType deviceType = MainPage.getDeviceType();
            switch(deviceType)
            {
                case DevType.DEVTYPE_BLUETOOTH:
                    BT_RadioButton.IsChecked = true;
                    break;
                case DevType.DEVTYPE_TCP:
                default:
                    TCP_RadioButton.IsChecked = true;
                    break;
            }
        }

        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            await findStart();
        }

        private void run(object sender)
        {
            updateList();
        }
        private async Task findStart()
        {
            //stop old finder
            while(true)
            {
                try
                {
                    await Finder.StopAsync();
                    break;
                }
                catch(Exception ex)
                {
                    if(ex.HResult != EposHResult.HR_E_PENDING)
                    {
                        break;
                    }
                }
            }

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }

            PrinterDiscovery_ListView.Items.Clear();

            //get device type and find
            try
            {
                if (BT_RadioButton.IsChecked == true)
                {
                    await Finder.StartAsync(IoDevType.BLUETOOTH, "");
                }
                else
                {
                    await Finder.StartAsync(IoDevType.TCP, "255.255.255.255");
                }
                
            }
            catch(Exception ex)
            {
                showMsg.showIoException(ex.HResult, "StartAsync");
                return;
            }

            updateListThread = new Timer(run, null, 0, 500);
        }

        private async void updateList()
        {
            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                DeviceInfo[] deviceList = null;

                try
                {
                    deviceList = Finder.GetDeviceInfoList(IoFilterOption.DEFAULT);

                    if (deviceList == null)
                    {
                        if (PrinterDiscovery_ListView.Items.Count() > 0)
                        {
                            PrinterDiscovery_ListView.Items.Clear();
                        }
                    }
                    else if (deviceList.Length != PrinterDiscovery_ListView.Items.Count())
                    {
                        PrinterDiscovery_ListView.Items.Clear();
                        for (int i = 0; i < deviceList.Length; i++)
                        {
                            PrinterDiscovery_ListView.Items.Add(new displayDeviceList(deviceList[i].PrinterName, deviceList[i].DeviceName));
                        }
                    }

                }
                catch (Exception)
                {
                    return;
                }

            });
        }
        private DevType getSelectedDevicetype()
        {
            if (BT_RadioButton.IsChecked == true)
            {
                return DevType.DEVTYPE_BLUETOOTH;
            }
            else
            {
                return DevType.DEVTYPE_TCP;
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override async void OnNavigatedFrom(NavigationEventArgs e)
        {
            while (true)
            {
                try
                {
                    await Finder.StopAsync();
                    break;
                }
                catch (Exception ex)
                {
                    if (ex.HResult != EposHResult.HR_E_PENDING)
                    {
                        break;
                    }
                }
            }

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }
            navigationHelper.OnNavigatedFrom(e);
        }

        private void ListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            MainPage.setSelectedPrinterDiscovery(getSelectedDevicetype(), (String)((displayDeviceList)PrinterDiscovery_ListView.SelectedItem).Device);

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }
            if (this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
        }

        private async void RadioButton_Clicked(object sender, RoutedEventArgs e)
        {
            await findStart();
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }

    public class displayDeviceList
    {
        public String DeviceName { get; set; }
        public String Device { get; set; }

        public displayDeviceList(String devicename, String device)
        {
            DeviceName = devicename;
            Device = device;
        }
    }
}
