﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


using System.Threading;
using Windows.UI.Popups;
using Windows.ApplicationModel.Resources;
using Windows.UI.Notifications;
using Windows.Data.Xml.Dom;

using LibEposPrint;

namespace ePOS_Print_CS
{
    class ShowMsg
    {
        private ResourceLoader loader = new ResourceLoader();

        const int BITCNT_INT = 32;

        public void showEposException(int errStatus, string method)
        {
            string msg = "";

            msg = string.Format("{0}\n  {1}\n{2}\n  {3}",
                                 loader.GetString("methoderr_errcode"),
                                 getEposExceptionText(errStatus),
                                 loader.GetString("methoderr_method"),
                                 method);

            show(msg);
        }

        public void showIoException(int errStatus, string method)
        {
            string msg = "";

            msg = string.Format("{0}\n  {1}\n{2}\n  {3}",
                                 loader.GetString("methoderr_errcode"),
                                 getIoExceptionText(errStatus),
                                 loader.GetString("methoderr_method"),
                                 method);

            show(msg);
        }

        public void showError(string errMsg)
        {
            show(errMsg);
        }

        public void showStatus(int result, int status, int battery)
        {
            string msg = "";

            msg = string.Format("{0}\n  {1}\n{2}\n  {3}\n{4}\n  0x{5:x04}",
                                 loader.GetString("statusmsg_result"),
                                 getEposExceptionText(result),
                                 loader.GetString("statusmsg_status"),
                                 getEposStatusText(status),
                                 loader.GetString("statusmsg_batterystatus"),
                                 battery);

            show(msg);
        }
        public void showPrinterName(string printerName, string LanguageName)
        {
            string msg = "";

            msg = string.Format("{0}\n  {1}\n{2}\n  {3}", 
                                  loader.GetString("namemsg_name"), printerName, 
                                  loader.GetString("namemsg_language"), LanguageName);
            show(msg);
        }

        public void showStatusChangeEvent(string deviceName, int status)
        {
            string msg = "";
            msg = string.Format("{0} : {1}\n{2} : {3}",
                                        loader.GetString("getname_label_ip"),
                                        deviceName,
                                        loader.GetString("statusmsg_status"),
                                        getEposStatusText(status));

            showToast(msg);
        }

        public void showBatteryStatusChangeEvent(string deviceName, int battery)
        {
            string msg = "";

            msg = string.Format("{0} : {1}\n{2} : 0x{3:x04}",
                                loader.GetString("getname_label_ip"),
                                deviceName,
                                loader.GetString("statusmsg_batterystatus"),
                                battery);

            showToast(msg);
        }

        private async void show(string msg)
        {
            MessageDialog msgDialog = new MessageDialog("");
            msgDialog.Content = msg;
            msgDialog.Commands.Add(new UICommand("OK"));
            await msgDialog.ShowAsync();
        }

        private void showToast(string msg)
        {
            toast(msg);
        }

        private void toast(string msg)
        {
            string toastMsg = msg.Replace("\n  ", " : ");
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.LoadXml("<toast><visual version='1'><binding template='ToastText01'><text id='1'>" + toastMsg + "</text></binding></visual></toast>");
                ToastNotification toast = new ToastNotification(xmlDoc);
                ToastNotificationManager.CreateToastNotifier().Show(toast);
            }
        }

        private String getIoExceptionText(int state)
        {
            string resultTxt = "";

            if (IoHResult.HR_S_OK == state)
            {
                resultTxt = "HR_S_OK";
            }
            else if (IoHResult.HR_E_INVALIDARG == state)
            {
                resultTxt = "HR_E_INVALIDARG";
            }
            else if (IoHResult.HR_E_ACCESSDENIED == state)
            {
                resultTxt = "HR_E_ACCESSDENIED";
            }
            else if (IoHResult.HR_E_OUTOFMEMORY == state)
            {
                resultTxt = "HR_E_OUTOFMEMORY";
            }
            else if (IoHResult.HR_E_PENDING == state)
            {
                resultTxt = "HR_E_PENDING";
            }
            else if (IoHResult.HR_E_FAIL == state)
            {
                resultTxt = "HR_E_FAIL";
            }
            else
            {
                resultTxt = string.Format("{0}", state);
            }
            return resultTxt;
        }

        private string getEposExceptionText(int state)
        {
            string resultTxt = "";

            if(EposHResult.HR_S_OK == state)
            {
                resultTxt = "HR_S_OK";
            }
            else if (EposHResult.HR_E_INVALIDARG == state)
            {
                resultTxt = "HR_E_INVALIDARG";
            }
            else if (EposHResult.HR_E_ACCESSDENIED == state)
            {
                resultTxt = "HR_E_ACCESSDENIED";
            }
            else if (EposHResult.HR_E_ABORT == state)
            {
                resultTxt = "HR_E_ABORT";
            }
            else if (EposHResult.HR_E_PENDING == state)
            {
                resultTxt = "HR_E_PENDING";
            }
            else if (EposHResult.HR_E_OUTOFMEMORY == state)
            {
                resultTxt = "HR_E_OUTOFMEMORY";
            }
            else if(EposHResult.HR_E_FAIL == state)
            {
                resultTxt = "HR_E_FAIL";
            }
            else
            {
                resultTxt = string.Format("{0}", state);
            }
            
            return resultTxt;
        }

        private string getEposStatusText(int status)
        {
            string resultTxt = "";

            for (int bit = 0; bit < BITCNT_INT; bit++)
            {
                int value = 1 << bit;
                if((value & status) !=0)
                {
                    string msg = "";
                    if(value == Print.ST_NO_RESPONSE){
                        msg = "NO_RESPONSE";
                    }
                    else if(value == Print.ST_PRINT_SUCCESS){
                        msg = "PRINT_SUCCESS";
                    }
                    else if(value == Print.ST_DRAWER_KICK){
                        msg = "DRAWER_KICK";
                    }
                    else if(value == Print.ST_OFF_LINE){
                        msg = "OFF_LINE";
                    }
                    else if(value == Print.ST_COVER_OPEN){
                        msg = "COVER_OPEN";
                    }
                    else if(value == Print.ST_PAPER_FEED){
                        msg = "PAPER_FEED";
                    }
                    else if(value == Print.ST_WAIT_ON_LINE){
                        msg = "WAIT_ON_LINE";
                    }
                    else if(value == Print.ST_PANEL_SWITCH){
                        msg = "PANEL_SWITCH";
                    }
                    else if(value == Print.ST_MECHANICAL_ERR){
                        msg = "MECHANICAL_ERR";
                    }
                    else if(value == Print.ST_AUTOCUTTER_ERR){
                        msg = "AUTOCUTTER_ERR";
                    }
                    else if(value == Print.ST_UNRECOVER_ERR){
                        msg = "UNRECOVER_ERR";
                    }
                    else if(value == Print.ST_AUTORECOVER_ERR){
                        msg = "AUTORECOVER_ERR";
                    }
                    else if(value == Print.ST_RECEIPT_NEAR_END){
                        msg = "RECEIPT_NEAR_END";
                    }
                    else if(value == Print.ST_RECEIPT_END){
                        msg = "RECEIPT_END";
                    }
                    else if(value == Print.ST_BUZZER){
                    }
                    else if(value == Print.ST_HEAD_OVERHEAT){
                        msg = "HEAD_OVERHEAT";
                    }
                    else if (value == Print.ST_MOTOR_OVERHEAT){
                        msg = "MOTOR_OVERHEAT";
                    }
                    else if (value == Print.ST_BATTERY_OVERHEAT){
                        msg = "BATTERY_OVERHEAT";
                    }
                    else if (value == Print.ST_WRONG_PAPER){
                        msg = "WRONG_PAPER";
                    }
                    else{
                        msg = string.Format("{0}", value);
                    }

                    if(!string.IsNullOrEmpty(msg))
                    {
                        if (!string.IsNullOrEmpty(resultTxt))
                        {
                            resultTxt += "\n  ";
                        }
                        resultTxt += msg;
                    } 
                }
            }
            return resultTxt;
        }
    }
}