﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class TextPage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;
        const int SIZEWIDTH_MAX = 7;
        const int SIZEWIDTH_MIN = 1;
        const int SIZEHEIGHT_MAX = 7;
        const int SIZEHEIGHT_MIN = 1;
        const int FONT_A = 0;
        const int FONT_B = 1;
        const int FONT_C = 2;
        const int FONT_D = 3;
        const int FONT_E = 4;
        const int ALIGN_LEFT = 0;
        const int ALIGN_CENTER = 1;
        const int ALIGN_RIGHT = 2;
        const int LANG_ANK = 0;
        const int LANG_JAPANESE = 1;
        const int LANG_CHINESE = 2;
        const int LANG_TAIWAN = 3;
        const int LANG_KOREAN = 4;
        const int LANG_THAI = 5;
        const int LANG_SOUTHASIA = 6;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public TextPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if (printer == null)
            {
                return;
            }
            else
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //init font list
            this.Font_ComboBox.Items.Add(loader.GetString("font_a"));
            this.Font_ComboBox.Items.Add(loader.GetString("font_b"));
            this.Font_ComboBox.Items.Add(loader.GetString("font_c"));
            this.Font_ComboBox.Items.Add(loader.GetString("font_d"));
            this.Font_ComboBox.Items.Add(loader.GetString("font_e"));

            //init align list
            this.Align_ComboBox.Items.Add(loader.GetString("align_left"));
            this.Align_ComboBox.Items.Add(loader.GetString("align_center"));
            this.Align_ComboBox.Items.Add(loader.GetString("align_right"));

            //init language list
            this.Language_ComboBox.Items.Add(loader.GetString("language_ank"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_japanese"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_simplified_chinese"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_traditional_chinese"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_korean"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_thai"));
            this.Language_ComboBox.Items.Add(loader.GetString("language_vietnamese"));

            //init size list
            for (int i = SIZEWIDTH_MIN; i <= SIZEWIDTH_MAX; i++)
            {
                this.SizeWidth_ComboBox.Items.Add(i.ToString());
            }

            for (int i = SIZEHEIGHT_MIN; i <= SIZEHEIGHT_MAX; i++)
            {
                this.SizeHeight_ComboBox.Items.Add(i.ToString());
            }

            //default setting
            this.Font_ComboBox.SelectedIndex = FONT_A;
            this.Align_ComboBox.SelectedIndex = ALIGN_LEFT;
            this.Language_ComboBox.SelectedIndex = LANG_ANK;
            this.SizeWidth_ComboBox.SelectedIndex = 0;
            this.SizeHeight_ComboBox.SelectedIndex = 0;

            this.PrintCharacters_TxtBox.Text = loader.GetString("text_edit_text");
            this.LineSpacing_TxtBox.Text = "30";
            this.XPosition_TxtBox.Text = "0";
            this.FeedUnit_TxtBox.Text = "30";
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await printText();
            this.IsEnabled = true;
        }

        private async Task printText()
        {
            if (string.IsNullOrWhiteSpace(PrintCharacters_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_notext"));
                return;
            }

            Builder builder = null;
            Print printer = null;
            PrinterStatus sendStatus;
            sendStatus.printerStatus = 0;
            sendStatus.batteryStatus = 0;
            string method = "";

            try
            {
                //create builder
                method = "Builder";
                builder = new Builder(MainPage.getPrinterName(), MainPage.getLanguage());

                //add command
                method = "AddTextFont";
                builder.AddTextFont(getBuilderFont());

                method = "AddTextAlign";
                builder.AddTextAlign(getBuilderAlign());

                method = "AddTextLineSpace";
                builder.AddTextLineSpace(getBuilderLineSpace());

                method = "AddTextLang";
                builder.AddTextLang(getBuilderLnaguage());

                method = "AddTextSize";
                builder.AddTextSize(getBuilderSizeW(), getBuilderSizeH());

                method = "AddTextStyle";
                builder.AddTextStyle(Reverse.REVERSE_FALSE, getBuilderUnderline(), getBuilderStyleBold(), Color.COLOR_1);

                method = "AddTextPosition";
                builder.AddTextPosition(getBuilderXPostion());

                method = "AddText";
                builder.AddText(getBuilderText());

                method = "AddFeedUnit";
                builder.AddFeedUnit(getBuilderFeedUnit());

                //send builder data
                try
                {
                    printer = MainPage.getPrinter();
                    sendStatus = await printer.SendDataAsync(builder, SEND_TIMEOUT);
                    showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus);
                }
                catch (Exception ex)
                {
                    PrinterStatus status = printer.GetPrinterStatus(ex.HResult);
                    showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus);
                }
            }
            catch (Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }

            //remove builder
            if (builder != null)
            {
                try
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
                catch (Exception)
                {
                    builder = null;
                }
            }
        }

        private Font getBuilderFont()
        {
            switch(Font_ComboBox.SelectedIndex)
            {
                case FONT_B:
                    return Font.FONT_B;
                case FONT_C:
                    return Font.FONT_C;
                case FONT_D:
                    return Font.FONT_D;
                case FONT_E:
                    return Font.FONT_E;
                case FONT_A:
                default:
                    return Font.FONT_A;
            }
        }

        private Align getBuilderAlign()
        {
            switch(Align_ComboBox.SelectedIndex)
            {
                case ALIGN_CENTER:
                    return Align.ALIGN_CENTER;
                case ALIGN_RIGHT:
                    return Align.ALIGN_RIGHT;
                case ALIGN_LEFT:
                default:
                    return Align.ALIGN_LEFT;
            }
        }

        private int getBuilderLineSpace()
        {
            try
            {
                return int.Parse(LineSpacing_TxtBox.Text);
            }
            catch(Exception)
            {
                return 0;
            }
        }

        private Lang getBuilderLnaguage()
        {
            switch(Language_ComboBox.SelectedIndex)
            {
                case LANG_JAPANESE:
                    return Lang.LANG_JA;
                case LANG_CHINESE:
                    return Lang.LANG_ZH_CN;
                case LANG_TAIWAN:
                    return Lang.LANG_ZH_TW;
                case LANG_KOREAN:
                    return Lang.LANG_KO;
                case LANG_THAI:
                    return Lang.LANG_TH;
                case LANG_SOUTHASIA:
                    return Lang.LANG_VI;
                case LANG_ANK:
                default:
                    return Lang.LANG_EN;
            }
        }

        private int getBuilderSizeW()
        {
            return SizeWidth_ComboBox.SelectedIndex + 1;
        }

        private int getBuilderSizeH()
        {
            return SizeHeight_ComboBox.SelectedIndex + 1;
        }

        private Emphasis getBuilderStyleBold()
        {
            if (style_ToggleButton.IsOn)
            {
                return Emphasis.EMPHASIS_TRUE;
            }
            else
            {
                return Emphasis.EMPHASIS_FALSE;
            }
        }

        private Underline getBuilderUnderline()
        {
            if (underline_ToggleButton.IsOn)
            {
                return Underline.UNDERLINE_TRUE;
            }
            else
            {
                return Underline.UNDERLINE_FALSE;
            }
        }

        private int getBuilderXPostion()
        {
            try
            {
                return int.Parse(XPosition_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private int getBuilderFeedUnit()
        {
            try
            {
                return int.Parse(FeedUnit_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private string getBuilderText()
        {
            return PrintCharacters_TxtBox.Text;
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }

    }
}
