﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class TwoDCodePage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;

        const int SYMBOL_PDF417_STANDARD = 0;
        const int SYMBOL_PDF417_TRUNCATED = 1;
        const int SYMBOL_QRCODE_MODEL1 = 2;
        const int SYMBOL_QRCODE_MODEL2 = 3;
        const int SYMBOL_MAXICODE_MODE2 = 4;
        const int SYMBOL_MAXICODE_MODE3 = 5;
        const int SYMBOL_MAXICODE_MODE4 = 6;
        const int SYMBOL_MAXICODE_MODE5 = 7;
        const int SYMBOL_MAXICODE_MODE6 = 8;
        const int SYMBOL_GS1_DATABAR_STACKED = 9;
        const int SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL = 10;
        const int SYMBOL_GS1_DATABAR_EXPANDED_STACKED = 11;
        const int SYMBOL_AZTECCODE_FULLRANGE = 12;
        const int SYMBOL_AZTECCODE_COMPACT = 13;
        const int SYMBOL_DATAMATRIX_SQUARE = 14;
        const int SYMBOL_DATAMATRIX_RECTANGLE8 = 15;
        const int SYMBOL_DATAMATRIX_RECTANGLE12 = 16;
        const int SYMBOL_DATAMATRIX_RECTANGLE16 = 17;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;
        private string defaultTwoDcodeData = "";

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public TwoDCodePage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if (printer == null)
            {
                return;
            }
            else
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //init type list
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_pdf417s"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_pdf417t"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_qr1"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_qr2"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_maxcode2"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_maxcode3"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_maxcode4"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_maxcode5"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_maxcode6"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_gr1st"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_gr1so"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_gr1es"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_aztecfull"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_azteccompact"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_dmsquare"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_dmrect_8rows"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_dmrect_12rows"));
            this.TwoDCodeType_ComboBox.Items.Add(loader.GetString("code2type_dmrect_16rows"));

            //default setting
            this.TwoDCodeType_ComboBox.SelectedIndex = SYMBOL_PDF417_STANDARD;
            setDefaultTwoDcodeErrorLevel();
            setDefaultTwoDcodeData();
            setDefaultTwoDcodeModuleSize();
            setDefaultTwoDcodeMaxSizeTxtBox();

            this.ModuleSizeWidth_TxtBox.Text = "3";
            this.ModuleSizeHeight_TxtBox.Text = "3";
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await printBarcode();
            this.IsEnabled = true;
        }

        private async Task printBarcode()
        {
            if (string.IsNullOrWhiteSpace(TwoDcodeData_TxtBox.Text))
            {
                showMsg.showError(loader.GetString("errmsg_notext"));
                return;
            }
           
            Builder builder = null;
            Print printer = null;
            PrinterStatus sendStatus;
            sendStatus.printerStatus = 0;
            sendStatus.batteryStatus = 0;
            string method = "";

            try
            {
                //create builder
                method = "Builder";
                builder = new Builder(MainPage.getPrinterName(), MainPage.getLanguage());

                //add command
                method = "AddSymbol";
                builder.AddSymbol(getBuilderData(), getBuilderType(), getBuilderLevel1(), getBuilderLevel2(), getBuilderWidth(), getBuilderHeight(), getBuilderMaxSize());

                //send builder data
                try
                {
                    printer = MainPage.getPrinter();
                    sendStatus = await printer.SendDataAsync(builder, SEND_TIMEOUT);
                    showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus);
                }
                catch (Exception ex)
                {
                    PrinterStatus status = printer.GetPrinterStatus(ex.HResult);
                    showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus);
                }
            }
            catch (Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }

            //remove builder
            if (builder != null)
            {
                try
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
                catch (Exception)
                {
                    builder = null;
                }
            }
        }

        private string getBuilderData()
        {
            return TwoDcodeData_TxtBox.Text;
        }

        private SymbolType getBuilderType()
        {
            LibEposPrint.SymbolType[] typelist = {
                             SymbolType.SYMBOL_PDF417_STANDARD,
                             SymbolType.SYMBOL_PDF417_TRUNCATED,
                             SymbolType.SYMBOL_QRCODE_MODEL_1,
                             SymbolType.SYMBOL_QRCODE_MODEL_2,
                             SymbolType.SYMBOL_MAXICODE_MODE_2,
                             SymbolType.SYMBOL_MAXICODE_MODE_3,
                             SymbolType.SYMBOL_MAXICODE_MODE_4,
                             SymbolType.SYMBOL_MAXICODE_MODE_5,
                             SymbolType.SYMBOL_MAXICODE_MODE_6,
                             SymbolType.SYMBOL_GS1_DATABAR_STACKED,
                             SymbolType.SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL,
                             SymbolType.SYMBOL_GS1_DATABAR_EXPANDED_STACKED,
                             SymbolType.SYMBOL_AZTECCODE_FULLRANGE,
                             SymbolType.SYMBOL_AZTECCODE_COMPACT,
                             SymbolType.SYMBOL_DATAMATRIX_SQUARE,
                             SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_8,
                             SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_12,
                             SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_16};

            return typelist[TwoDCodeType_ComboBox.SelectedIndex];
        }

        private Level getBuilderLevel1()
        {
            Level[] levellist = null;
            Level[] pdf417levellist = {
                                       Level.LEVEL_0,
                                       Level.LEVEL_1,
                                       Level.LEVEL_2,
                                       Level.LEVEL_3,
                                       Level.LEVEL_4,
                                       Level.LEVEL_5,
                                       Level.LEVEL_6,
                                       Level.LEVEL_7,
                                       Level.LEVEL_8};

            Level[] qrlevellist = {
                                   Level.LEVEL_L,
                                   Level.LEVEL_M,
                                   Level.LEVEL_Q,
                                   Level.LEVEL_H};

            switch(TwoDCodeType_ComboBox.SelectedIndex)
            {
                case SYMBOL_PDF417_STANDARD:
                case SYMBOL_PDF417_TRUNCATED:
                    levellist = pdf417levellist;
                    break;
                case SYMBOL_QRCODE_MODEL1:
                case SYMBOL_QRCODE_MODEL2:
                    levellist = qrlevellist;
                    break;
                default:
                    break;
            }
            
            if(levellist == null)
            {
                return Level.LEVEL_DEFAULT; 
            }
            else
            {
                return levellist[ErrorCorrectionLevel_ComboBox.SelectedIndex];
            }
        }

        private int getBuilderLevel2()
        {
            try
            {
                return int.Parse(ErrorCorrectionLevel_TxtBox.Text);
            }
            catch (Exception)
            {
                return 0;
            }
        }

        private int getBuilderWidth()
        {
            if (ModuleSizeWidth_TxtBox.IsEnabled == true)
            {
                try
                {
                    return int.Parse(ModuleSizeWidth_TxtBox.Text);
                }
                catch (Exception)
                {
                    return 0;
                }
            }
            else
            {
                return Builder.PARAM_UNSPECIFIED;
            }
        }

        private int getBuilderHeight()
        {
            if(ModuleSizeHeight_TxtBox.IsEnabled == true)
            {
                try
                {
                    return int.Parse(ModuleSizeHeight_TxtBox.Text);
                }
                catch (Exception)
                {
                    return 0;
                }
            }
            else
            {
                return Builder.PARAM_UNSPECIFIED;
            }
        }

        private int getBuilderMaxSize()
        {
            if(MaxSize_TxtBox.IsEnabled == true)
            {
                try
                {
                    return int.Parse(MaxSize_TxtBox.Text);
                }
                catch(Exception)
                {
                    return 0;
                }
            }
            else
            {
                return Builder.PARAM_UNSPECIFIED;
            }
        }

        private void setDefaultTwoDcodeData()
        {
            defaultTwoDcodeData = getDefaultTwoDcodeData();

            TwoDcodeData_TxtBox.Text = defaultTwoDcodeData;
        }

        private string getDefaultTwoDcodeData()
        {
            string[] TowDcodeDataList = {
                                         loader.GetString("code2data_pdf417"),
                                         loader.GetString("code2data_pdf417t"),
                                         loader.GetString("code2data_qr1"),
                                         loader.GetString("code2data_qr2"),
                                         loader.GetString("code2data_maxcode2"), 
                                         loader.GetString("code2data_maxcode3"),
                                         loader.GetString("code2data_maxcode4"),
                                         loader.GetString("code2data_maxcode5"),
                                         loader.GetString("code2data_maxcode6"),
                                         loader.GetString("code2data_gr1so"),
                                         loader.GetString("code2data_gr1st"),
                                         loader.GetString("code2data_gr1es"),
                                         loader.GetString("code2data_azteccompact"),
                                         loader.GetString("code2data_aztecfull"),
                                         loader.GetString("code2data_dmsquare"),
                                         loader.GetString("code2data_dmrect_8rows"),
                                         loader.GetString("code2data_dmrect_12rows"),
                                         loader.GetString("code2data_dmrect_16rows")};

            return TowDcodeDataList[TwoDCodeType_ComboBox.SelectedIndex];
        }

        private void setDefaultTwoDcodeErrorLevel()
        {
            int iTwoDCodeType = TwoDCodeType_ComboBox.SelectedIndex;
            setDefaultTwoDcodeErrorLevelComboBox(iTwoDCodeType);
            setDefaultTwoDcodeErrorLevelTxtBox(iTwoDCodeType);
        }

        private void setDefaultTwoDcodeErrorLevelComboBox(int iTwoDCodeType)
        {
            bool isEnableCombo = true;
            string[] ErrorLevelCombo = null;
            string[] pdf417ErrorLevel = {
                                         loader.GetString("level_pdf417_0"),
                                         loader.GetString("level_pdf417_1"),
                                         loader.GetString("level_pdf417_2"),
                                         loader.GetString("level_pdf417_3"),
                                         loader.GetString("level_pdf417_4"),
                                         loader.GetString("level_pdf417_5"),
                                         loader.GetString("level_pdf417_6"),
                                         loader.GetString("level_pdf417_7"),
                                         loader.GetString("level_pdf417_8")};

            string[] qrErrorLevel ={
                                    loader.GetString("level_qr_l"),
                                    loader.GetString("level_qr_m"),
                                    loader.GetString("level_qr_q"),
                                    loader.GetString("level_qr_h")};

            string[] noErrorLevel = { loader.GetString("empty")};

            switch (iTwoDCodeType)
            {
                case SYMBOL_PDF417_STANDARD:
                case SYMBOL_PDF417_TRUNCATED:
                    ErrorLevelCombo = pdf417ErrorLevel;
                    break;
                case SYMBOL_QRCODE_MODEL1:
                case SYMBOL_QRCODE_MODEL2:
                    ErrorLevelCombo = qrErrorLevel;
                    break;
                default:
                    ErrorLevelCombo = noErrorLevel;
                    isEnableCombo = false;
                    break;
            }

            ErrorCorrectionLevel_ComboBox.Items.Clear();

            for (int i = 0; i < ErrorLevelCombo.Length; i++)
            {
                ErrorCorrectionLevel_ComboBox.Items.Add(ErrorLevelCombo[i]);
            }

            ErrorCorrectionLevel_ComboBox.SelectedIndex = 0;

            ErrorCorrectionLevel_ComboBox.IsEnabled = isEnableCombo;

        }

        private void setDefaultTwoDcodeErrorLevelTxtBox(int iTwoDCodeType)
        {
            bool isEnableTxtBox = false; ;

            string ErrorLevelTxtBox = "";

            switch (iTwoDCodeType)
            {
                case SYMBOL_AZTECCODE_FULLRANGE:
                case SYMBOL_AZTECCODE_COMPACT:
                    isEnableTxtBox = true;
                    ErrorLevelTxtBox = "23";
                    break;
                default:
                    break;
            }
            ErrorCorrectionLevel_TxtBox.IsEnabled = isEnableTxtBox;
            ErrorCorrectionLevel_TxtBox.Text = ErrorLevelTxtBox;
        }

        private void setDefaultTwoDcodeMaxSizeTxtBox()
        {
            bool isEnableTxtBox = false; ;

            string maxSizeTxtBox = "";

            int iTwoDCodeType = TwoDCodeType_ComboBox.SelectedIndex;

            switch (iTwoDCodeType)
            {
                case SYMBOL_PDF417_STANDARD:
                case SYMBOL_PDF417_TRUNCATED:
                    isEnableTxtBox = true;
                    maxSizeTxtBox = "0";
                    break;
                case SYMBOL_GS1_DATABAR_STACKED:
                case SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL:
                case SYMBOL_GS1_DATABAR_EXPANDED_STACKED:
                    isEnableTxtBox = true;
                    maxSizeTxtBox = "114";
                    break;
                default:
                    break;
            }
            MaxSize_TxtBox.IsEnabled = isEnableTxtBox;
            MaxSize_TxtBox.Text = maxSizeTxtBox;
        }

        private void setDefaultTwoDcodeModuleSize()
        {
            bool isEnableTxtBoxWidth = false;
            bool isEnableTxtBoxHeight = false;

            string moduleSizeWidth = "";
            string moduleSizeHeight = "";

            int iTwoDCodeType = TwoDCodeType_ComboBox.SelectedIndex;

            switch (iTwoDCodeType)
            {
                case SYMBOL_PDF417_STANDARD:
                case SYMBOL_PDF417_TRUNCATED:
                    isEnableTxtBoxWidth = true;
                    isEnableTxtBoxHeight = true;
                    moduleSizeWidth = "3";
                    moduleSizeHeight = "3";
                    break;
                case SYMBOL_QRCODE_MODEL1:
                case SYMBOL_QRCODE_MODEL2:
                case SYMBOL_AZTECCODE_FULLRANGE:
                case SYMBOL_AZTECCODE_COMPACT:
                case SYMBOL_DATAMATRIX_SQUARE:
                case SYMBOL_DATAMATRIX_RECTANGLE8:
                case SYMBOL_DATAMATRIX_RECTANGLE12:
                case SYMBOL_DATAMATRIX_RECTANGLE16:
                    isEnableTxtBoxWidth = true;
                    isEnableTxtBoxHeight = false;
                    moduleSizeWidth = "3";
                    moduleSizeHeight = "";
                    break;
                case SYMBOL_MAXICODE_MODE2:
                case SYMBOL_MAXICODE_MODE3:
                case SYMBOL_MAXICODE_MODE4:
                case SYMBOL_MAXICODE_MODE5:
                case SYMBOL_MAXICODE_MODE6:
                    isEnableTxtBoxWidth = false;
                    isEnableTxtBoxHeight = false;
                    moduleSizeWidth = "";
                    moduleSizeHeight = "";
                    break;
                case SYMBOL_GS1_DATABAR_STACKED:
                case SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL:
                case SYMBOL_GS1_DATABAR_EXPANDED_STACKED:
                    isEnableTxtBoxWidth = true;
                    isEnableTxtBoxHeight = false;
                    moduleSizeWidth = "2";
                    moduleSizeHeight = "";
                    break;
                default:
                    break;
            }

            ModuleSizeWidth_TxtBox.IsEnabled = isEnableTxtBoxWidth;
            ModuleSizeWidth_TxtBox.Text = moduleSizeWidth;

            ModuleSizeHeight_TxtBox.IsEnabled = isEnableTxtBoxHeight;
            ModuleSizeHeight_TxtBox.Text = moduleSizeHeight;

        }

        private void TwoDcodeType_Selected(object sender, SelectionChangedEventArgs e)
        {
            bool isChangedTwoDcodeData = false;

            setDefaultTwoDcodeErrorLevel();
            setDefaultTwoDcodeMaxSizeTxtBox();
            setDefaultTwoDcodeModuleSize();

            if (defaultTwoDcodeData == TwoDcodeData_TxtBox.Text)
            {
                isChangedTwoDcodeData = true;
            }

            if (isChangedTwoDcodeData)
            {
                setDefaultTwoDcodeData();
            }
            else
            {
                // not update
                return;
            }
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }
}