﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class BarcodePage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000
    Const HRI_NONE_INDEX = 0
    Const HRI_ABOVE_INDEX = 1
    Const HRI_BELOW_INDEX = 2
    Const HRI_BOTH_INDEX = 3
    Const FONT_A_INDEX = 0
    Const FONT_B_INDEX = 1
    Const FONT_C_INDEX = 2
    Const FONT_D_INDEX = 3
    Const FONT_E_INDEX = 4

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing
    Private _defaultBarcodeData As String = String.Empty

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' init type list
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_upca"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_upce"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_ean13"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_jan13"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_ean8"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_jan8"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_code39"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_itf"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_codabar"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_code93"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_code128"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_gs1128"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_gs1om"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_gs1tr"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_gs1li"))
        Me.BarcodeType_ComboBox.Items.Add(_loader.GetString("bartype_gs1ex"))

        ' init HRI list
        Me.BarcodeHRI_ComboBox.Items.Add(_loader.GetString("hri_none"))
        Me.BarcodeHRI_ComboBox.Items.Add(_loader.GetString("hri_above"))
        Me.BarcodeHRI_ComboBox.Items.Add(_loader.GetString("hri_below"))
        Me.BarcodeHRI_ComboBox.Items.Add(_loader.GetString("hri_both"))

        ' init font list
        Me.BarcodeFont_ComboBox.Items.Add(_loader.GetString("font_a"))
        Me.BarcodeFont_ComboBox.Items.Add(_loader.GetString("font_b"))
        Me.BarcodeFont_ComboBox.Items.Add(_loader.GetString("font_c"))
        Me.BarcodeFont_ComboBox.Items.Add(_loader.GetString("font_d"))
        Me.BarcodeFont_ComboBox.Items.Add(_loader.GetString("font_e"))

        ' default setting
        Me.BarcodeType_ComboBox.SelectedIndex = HRI_NONE_INDEX
        Me.BarcodeHRI_ComboBox.SelectedIndex = HRI_BELOW_INDEX
        Me.BarcodeFont_ComboBox.SelectedIndex = FONT_A_INDEX

        Me.ModuleSizeWidth_TxtBox.Text = "3"
        Me.ModuleSizeHeight_TxtBox.Text = "162"

        SetDefaultBarcodeData()

    End Sub

    Private Sub BarcodeType_Selected(sender As Object, e As SelectionChangedEventArgs)

        Dim isChangedBarcodeData As Boolean = False

        If Equals(_defaultBarcodeData, BarcodeData_TxtBox.Text) Then
            isChangedBarcodeData = True
        End If

        If isChangedBarcodeData Then
            ' update barcode data
            SetDefaultBarcodeData()
        Else
            ' not update
            Return
        End If

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await PrintBarcode()
        Me.IsEnabled = True
    End Sub

    Private Async Function PrintBarcode() As Task

        If String.IsNullOrWhiteSpace(BarcodeData_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddBarcode"
            builder.AddBarcode(GetBuilderData(),
                               GetBuilderType(),
                               GetBuilderHri(),
                               GetBuilderFont(),
                               GetBuilderWidth(),
                               GetBuilderHeight())

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If

    End Function

    Private Function GetBuilderData() As String
        Return BarcodeData_TxtBox.Text
    End Function

    Private Function GetBuilderType() As BarcodeType

        Dim typelist As BarcodeType() = {
                                 BarcodeType.BARCODE_UPC_A,
                                 BarcodeType.BARCODE_UPC_E,
                                 BarcodeType.BARCODE_EAN13,
                                 BarcodeType.BARCODE_JAN13,
                                 BarcodeType.BARCODE_EAN8,
                                 BarcodeType.BARCODE_JAN8,
                                 BarcodeType.BARCODE_CODE39,
                                 BarcodeType.BARCODE_ITF,
                                 BarcodeType.BARCODE_CODABAR,
                                 BarcodeType.BARCODE_CODE93,
                                 BarcodeType.BARCODE_CODE128,
                                 BarcodeType.BARCODE_GS1_128,
                                 BarcodeType.BARCODE_GS1_DATABAR_OMNIDIRECTIONAL,
                                 BarcodeType.BARCODE_GS1_DATABAR_TRUNCATED,
                                 BarcodeType.BARCODE_GS1_DATABAR_LIMITED,
                                 BarcodeType.BARCODE_GS1_DATABAR_EXPANDED
                         }

        Return typelist(BarcodeType_ComboBox.SelectedIndex)

    End Function

    Private Function GetBuilderHri() As Hri

        Select Case (BarcodeHRI_ComboBox.SelectedIndex)
            Case HRI_ABOVE_INDEX
                Return Hri.HRI_ABOVE
            Case HRI_BELOW_INDEX
                Return Hri.HRI_BELOW
            Case HRI_BOTH_INDEX
                Return Hri.HRI_BOTH
            Case Else
                Return Hri.HRI_NONE
        End Select

    End Function

    Private Function GetBuilderFont() As Font

        Select Case (BarcodeFont_ComboBox.SelectedIndex)
            Case FONT_B_INDEX
                Return Font.FONT_B
            Case FONT_C_INDEX
                Return Font.FONT_C
            Case FONT_D_INDEX
                Return Font.FONT_D
            Case FONT_E_INDEX
                Return Font.FONT_E
            Case Else
                Return Font.FONT_A
        End Select

    End Function

    Private Function GetBuilderWidth() As Integer

        Try
            Return Integer.Parse(ModuleSizeWidth_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderHeight() As Integer

        Try
            Return Integer.Parse(ModuleSizeHeight_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Sub SetDefaultBarcodeData()

        ' update default data
        _defaultBarcodeData = GetDefaultBarcodeData()

        ' update ui
        BarcodeData_TxtBox.Text = _defaultBarcodeData

    End Sub

    Private Function GetDefaultBarcodeData() As String

        Dim barcodeDataList() As String = {
                                            _loader.GetString("bardata_upca"),
                                            _loader.GetString("bardata_upce"),
                                            _loader.GetString("bardata_ean13"),
                                            _loader.GetString("bardata_jan13"),
                                            _loader.GetString("bardata_ean8"),
                                            _loader.GetString("bardata_jan8"),
                                            _loader.GetString("bardata_code39"),
                                            _loader.GetString("bardata_itf"),
                                            _loader.GetString("bardata_codabar"),
                                            _loader.GetString("bardata_code93"),
                                            _loader.GetString("bardata_code128"),
                                            _loader.GetString("bardata_gs1128"),
                                            _loader.GetString("bardata_gs1om"),
                                            _loader.GetString("bardata_gs1tr"),
                                            _loader.GetString("bardata_gs1li"),
                                            _loader.GetString("bardata_gs1ex")
                                        }

        Return barcodeDataList(BarcodeType_ComboBox.SelectedIndex)

    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
