﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class CutPage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000
    Const CUT_NO_FEED = 0
    Const CUT_FEED = 1

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' init type list
        Me.CutType_ComboBox.Items.Add(_loader.GetString("cuttype_nofeed"))
        Me.CutType_ComboBox.Items.Add(_loader.GetString("cuttype_feed"))

        ' default setting
        Me.CutType_ComboBox.SelectedIndex = CUT_FEED

    End Sub

    Private Async Sub Cut_Button_Click(sender As Object, e As RoutedEventArgs) Handles Cut_Button.Click
        Me.IsEnabled = False
        Await CutPrint()
        Me.IsEnabled = True
    End Sub

    Private Async Function CutPrint() As Task

        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            ' create builder
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddCut"
            builder.AddCut(GetBuilderType())

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If

    End Function

    Private Function GetBuilderType() As Cut

        Select Case (CutType_ComboBox.SelectedIndex)
            Case CUT_FEED
                Return Cut.CUT_FEED
            Case Else
                Return Cut.CUT_NO_FEED
        End Select

    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
