﻿Imports Windows.UI.Core
Imports Windows.ApplicationModel.Resources

Imports LibEposPrint
Public NotInheritable Class LogSettingsPage
    Inherits Page

    Const LOG_DISABLE = 0
    Const LOG_STORAGE = 1
    Const LOG_TCP = 2

    Const LOG_LEVEL_LOW As Integer = 0

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        ' init enabled list
        Me.LogEnabled_ComboBox.Items.Add(_loader.GetString("log_disable"))
        Me.LogEnabled_ComboBox.Items.Add(_loader.GetString("log_storage"))
        Me.LogEnabled_ComboBox.Items.Add(_loader.GetString("log_tcp"))

        ' init log level
        Me.LogLevel_ComboBox.Items.Add(_loader.GetString("log_low"))

        'Default setting
        Me.LogEnabled_ComboBox.SelectedIndex = LOG_DISABLE
        Me.LogLevel_ComboBox.SelectedIndex = LOG_LEVEL_LOW

        Me.address_TxtBox.SelectedText = _loader.GetString("logsettings_dummy_ip")
        Me.port_TxtBox.SelectedText = _loader.GetString("logsettings_dummy_port")
        Me.logsize_TxtBox.SelectedText = _loader.GetString("logsettings_dummy_logsize")

    End Sub

    Private Sub Setting_Button_Click(sender As Object, e As RoutedEventArgs)

        Try
            Log.SetLogSettings(getPeriod(), GetEnabled(), GetIpAddress(), GetPort(), GetLogSize(), GetLogLevel())
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, "SetLogSettings")
        End Try

    End Sub

    Private Function GetEnabled() As LogEnabled

        Select Case LogEnabled_ComboBox.SelectedIndex
            Case LOG_STORAGE
                Return LogEnabled.LOG_STORAGE
            Case LOG_TCP
                Return LogEnabled.LOG_TCP
            Case Else
                Return LogEnabled.LOG_DISABLE
        End Select

    End Function

    Private Function GetIpAddress() As String

        Return address_TxtBox.Text

    End Function

    Private Function GetPort() As Integer

        Dim port As Integer = port_TxtBox.Text
        Try
            Return Integer.Parse(port)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetLogSize() As Integer

        Dim logsize As String = logsize_TxtBox.Text
        Try
            Return Integer.Parse(logsize)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetLogLevel() As LogLevel

        Select Case (LogLevel_ComboBox.SelectedIndex)
            Case Else
                Return LogLevel.LOG_LOW
        End Select

    End Function

    Private Function getPeriod() As LogPeriod

        If logsetting_ToggleButton.IsOn Then
            Return LogPeriod.LOG_PERMANENT
        Else
            Return LogPeriod.LOG_TEMPORARY
        End If

    End Function
End Class
