﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint


Public NotInheritable Class MainPage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Shared _printer As Print = Nothing
    Shared _lang As ModelLang = ModelLang.MODEL_LANG_ANK
    Shared _printerName As String = "TM-T88V"
    Shared _deviceType As DevType = DevType.DEVTYPE_TCP
    Shared _deviceName As String = "192.168.192.168"

    Shared Sub SetBuilderOptions(selectPrinter As Print, selectLang As ModelLang, selectPrinterName As String)

        _printer = selectPrinter
        _lang = selectLang
        _printerName = selectPrinterName

    End Sub

    Shared Sub SetSelectedPrinterDiscovery(selectDevicetype As DevType, selectDeviceName As String)

        _deviceType = selectDevicetype
        _deviceName = selectDeviceName

    End Sub

    Shared Function GetPrinter()
        Return _printer
    End Function

    Shared Function GetLanguage()
        Return _lang
    End Function

    Shared Function GetPrinterName()
        Return _printerName
    End Function

    Shared Function GetDeviceType()
        Return _deviceType
    End Function

    Shared Function GetDeviceName()
        Return _deviceName
    End Function

    Public Sub New()

        InitializeComponent()

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        If _printer IsNot Nothing Then
            _printer.SetStatusChangeEventCallback(statusChangeEvent)
            _printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' update button state
        updateButtonStatus()

    End Sub

    Private Sub PrinterDiscovery_Button_Click(sender As Object, e As RoutedEventArgs)
        'Printer Discovery Page
        Me.Frame.Navigate(GetType(PrinterDiscoveryPage), Me)
    End Sub

    Private Sub Open_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Open Page
        Me.Frame.Navigate(GetType(OpenPage), Me)
    End Sub

    Private Sub Text_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Text Page
        Me.Frame.Navigate(GetType(TextPage), Me)
    End Sub

    Private Sub Image_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Image Page
        Me.Frame.Navigate(GetType(ImagePage), Me)
    End Sub

    Private Sub Barcode_Button_Click(sender As Object, e As RoutedEventArgs)
        'Barcode Page
        Me.Frame.Navigate(GetType(BarcodePage), Me)
    End Sub

    Private Sub TwoDCode_Button_Click(sender As Object, e As RoutedEventArgs)
        ' 2D Code Page
        Me.Frame.Navigate(GetType(TwoDCodePage), Me)
    End Sub

    Private Sub PageMode_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Page Mode Page
        Me.Frame.Navigate(GetType(PageModePage), Me)
    End Sub

    Private Sub Cut_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Cut Page
        Me.Frame.Navigate(GetType(CutPage), Me)
    End Sub

    Private Sub GetPrinterName_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Get Printer Name  Page
        Me.Frame.Navigate(GetType(GetPrinterNamePage), Me)
    End Sub

    Private Sub LogSettings_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Log settings Page
        Me.Frame.Navigate(GetType(LogSettingsPage), Me)
    End Sub

    Private Async Sub GetStatus_Button_Click(sender As Object, e As RoutedEventArgs)
        ' Get Printer Status
        Me.IsEnabled = False
        Await showPrinterStatus()
        Me.IsEnabled = True
    End Sub

    Private Async Sub Close_Button_Click(sender As Object, e As RoutedEventArgs)
        Await closePrinter()
        updateButtonStatus()
    End Sub

    Private Async Function closePrinter() As Task

        If _printer IsNot Nothing Then
            Try
                Await _printer.ClosePrinterAsync()
                _printer = Nothing
            Catch ex As Exception
                _showMsg.showEposException(ex.HResult, "ClosePrinterAsync")
            End Try
        End If

    End Function

    Private Sub updateButtonStatus()

        Dim groupControl1 As Button() = {Open_Button,
                                         GetPrinterName_Button}

        Dim groupControl2 As Button() = {Close_Button, Text_Button,
                                         Image_Button, Barcode_Button,
                                         TwoDCode_Button, PageMode_Button,
                                         Cut_Button, GetStatus_Button}

        Dim enableControl As Button() = Nothing
        Dim disableControl As Button() = Nothing

        If _printer Is Nothing Then
            enableControl = groupControl1
            disableControl = groupControl2
        Else
            enableControl = groupControl2
            disableControl = groupControl1
        End If

        Dim i As Integer

        For i = 0 To enableControl.Length - 1
            enableControl(i).IsEnabled = True
        Next

        For i = 0 To disableControl.Length - 1
            disableControl(i).IsEnabled = False
        Next

    End Sub

    Private Async Function showPrinterStatus() As Task
        Dim status As PrinterStatus = Nothing
        status.printerStatus = 0
        status.batteryStatus = 0
        Dim method As String = String.Empty
        Try
            'Get printer status and battery status
            method = "getStatus"
            status = Await _printer.GetStatusAsync()
            _showMsg.showStatus(EposHResult.HR_S_OK, status.printerStatus, status.batteryStatus)
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try
    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub
End Class
