﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class OpenPage
    Inherits Page

    Const LANG_ANK = 0
    Const LANG_JAPANESE = 1
    Const LANG_CHINESE = 2
    Const LANG_TAIWAN = 3
    Const LANG_KOREAN = 4
    Const LANG_THAI = 5
    Const LANG_SOUTHASIA = 6

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        ' init printer list
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_m10"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_m30"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p20"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p60"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p60ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p80"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t20"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t20ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t70"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t70ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t81ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t82"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t82ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t88v"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t90ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_u220"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_u330"))

        ' init language list
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_ank"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_japanese"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_chinese"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_taiwan"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_korean"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_thai"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("model_southasia"))

        Me.PrinterName_ComboBox.SelectedIndex = 0
        Me.Language_ComboBox.SelectedIndex = LANG_ANK
        Me.Interval_TxtBox.Text = "1000"

        ' select default radiobutton
        Select Case MainPage.GetDeviceType()
            Case DevType.DEVTYPE_BLUETOOTH
                Me.BT_RadioButton.IsChecked = True
            Case Else
                Me.TCP_RadioButton.IsChecked = True
        End Select

        ' init edit
        Me.DeviceName_TxtBox.Text = MainPage.GetDeviceName()
    End Sub

    Private Async Sub Open_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await OpenPrinter()
        Me.IsEnabled = True
    End Sub

    Private Async Function OpenPrinter() As Task

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        If String.IsNullOrWhiteSpace(DeviceName_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim deviceType As DevType = GetDeviceType()
        Dim enabled As Monitoring = Monitoring.MONITORING_FALSE

        If GetStatusMonitorEnable() Then
            enabled = Monitoring.MONITORING_TRUE
        Else
            enabled = Monitoring.MONITORING_FALSE
        End If

        Dim interval As Integer = GetStatusMonitorInterval()
        Dim printer As Print = Nothing
        Try
            ' open
            printer = New Print()
            If printer IsNot Nothing Then
                printer.SetStatusChangeEventCallback(statusChangeEvent)
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
            End If
            Await printer.OpenPrinterAsync(deviceType, GetDeviceName(), enabled, interval)
        Catch ex As Exception
            printer = Nothing
            _showMsg.showEposException(ex.HResult, "OpenPrinterAsync")
            Return
        End Try

        ' set for builder class
        MainPage.SetBuilderOptions(printer, GetSelectedLanguge(), GetSelectedPrinterName())

        If Me.Frame.CanGoBack Then
            Me.Frame.GoBack()
        End If

    End Function

    Private Function GetDeviceType() As DevType

        If BT_RadioButton.IsChecked Then
            Return DevType.DEVTYPE_BLUETOOTH
        Else
            Return DevType.DEVTYPE_TCP
        End If

    End Function

    Private Function GetDeviceName() As String
        Return DeviceName_TxtBox.Text
    End Function

    Private Function GetStatusMonitorEnable() As Boolean
        Return statusMonitor_ToggleButton.IsOn
    End Function

    Private Function GetStatusMonitorInterval() As Integer

        Try
            Return Integer.Parse(Interval_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetSelectedPrinterName()
        Return PrinterName_ComboBox.SelectedItem.ToString()
    End Function

    Private Function GetSelectedLanguge() As ModelLang

        Dim language As ModelLang = ModelLang.MODEL_LANG_ANK
        Select Case Language_ComboBox.SelectedIndex
            Case LANG_JAPANESE
                language = ModelLang.MODEL_LANG_JAPANESE
            Case LANG_CHINESE
                language = ModelLang.MODEL_LANG_CHINESE
            Case LANG_TAIWAN
                language = ModelLang.MODEL_LANG_TAIWAN
            Case LANG_KOREAN
                language = ModelLang.MODEL_LANG_KOREAN
            Case LANG_THAI
                language = ModelLang.MODEL_LANG_THAI
            Case LANG_SOUTHASIA
                language = ModelLang.MODEL_LANG_SOUTHASIA
            Case Else
                language = ModelLang.MODEL_LANG_ANK
        End Select
        Return language

    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
