﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class PageModePage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' Default Option
        Me.PrintCharacters_TxtBox.Text = _loader.GetString("pagemode_edit_text")
        Me.PrintAreaX_TxtBox.Text = "100"
        Me.PrintAreaY_TxtBox.Text = "100"
        Me.PrintAreaWidth_TxtBox.Text = "300"
        Me.PrintAreaHeight_TxtBox.Text = "300"

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await PrintText()
        Me.IsEnabled = True
    End Sub

    Private Async Function PrintText() As Task

        If String.IsNullOrWhiteSpace(PrintCharacters_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            ' create builder
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddPageBegin"
            builder.AddPageBegin()

            method = "addPageArea"
            builder.AddPageArea(GetBuilderAreaX(), GetBuilderAreaY(), GetBuilderAreaWidth(), GetBuilderAreaHeight())

            method = "addText"
            builder.AddText(GetBuilderText())

            method = "addPageEnd"
            builder.AddPageEnd()

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If
    End Function

    Private Function GetBuilderAreaX() As Integer

        Try
            Return Integer.Parse(PrintAreaX_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderAreaY() As Integer

        Try
            Return Integer.Parse(PrintAreaY_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderAreaWidth() As Integer

        Try
            Return Integer.Parse(PrintAreaWidth_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderAreaHeight() As Integer

        Try
            Return Integer.Parse(PrintAreaHeight_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderText() As String
        Return PrintCharacters_TxtBox.Text
    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
