﻿Imports Windows.UI.Core
Imports System.Threading
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class PrinterDiscoveryPage
    Inherits Page

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing
    Private _updateListThread As Timer = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)
        AddHandler Me._navigationHelper.LoadState, AddressOf NavigationHelper_LoadState

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer IsNot Nothing Then
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        Dim deviceType As DevType = MainPage.GetDeviceType()
        Select Case (deviceType)
            Case DevType.DEVTYPE_BLUETOOTH
                BT_RadioButton.IsChecked = True
            Case Else
                TCP_RadioButton.IsChecked = True
        End Select

    End Sub

    Private Async Sub NavigationHelper_LoadState(sender As Object, e As Common.LoadStateEventArgs)
        Await findStart()
    End Sub

    Private Async Sub Run(sender As Object)
        Await updateList()
    End Sub

    Private Async Function findStart() As Task
        ' stop old finder
        While True
            Try
                Await Finder.StopAsync()
                Exit While
            Catch ex As Exception
                If (ex.HResult <> EposHResult.HR_E_PENDING) Then
                    Exit While
                End If
            End Try
        End While

        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If

        PrinterDiscovery_ListView.Items.Clear()

        ' get device type and find
        Try

            If BT_RadioButton.IsChecked Then
                Await Finder.StartAsync(IoDevType.BLUETOOTH, "")
            Else
                Await Finder.StartAsync(IoDevType.TCP, "255.255.255.255")
            End If
        Catch ex As Exception
            _showMsg.showIoException(ex.HResult, "StartAsync")
            Return
        End Try

        _updateListThread = New Timer((AddressOf Run), Nothing, 0, 1000)
    End Function

    Private Async Function updateList() As Task

        Await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, _
                                 Sub()
                                     Try
                                         Dim deviceList As DeviceInfo() = Nothing
                                         deviceList = Finder.GetDeviceInfoList(IoFilterOption.DEFAULT)
                                         If (deviceList Is Nothing) Then
                                             If (PrinterDiscovery_ListView.Items.Count() > 0) Then
                                                 PrinterDiscovery_ListView.Items.Clear()
                                             End If
                                         ElseIf (deviceList.Length <> PrinterDiscovery_ListView.Items.Count()) Then
                                             PrinterDiscovery_ListView.Items.Clear()
                                             Dim i As Integer = 0
                                             For i = 0 To deviceList.Length - 1
                                                 PrinterDiscovery_ListView.Items.Add(New displayDeviceList(deviceList(i).PrinterName, deviceList(i).DeviceName))
                                             Next
                                         End If

                                     Catch ex As Exception
                                         Return
                                     End Try
                                 End Sub)
    End Function

    Private Function GetSelectedDevicetype()
        If BT_RadioButton.IsChecked Then
            Return DevType.DEVTYPE_BLUETOOTH
        Else
            Return DevType.DEVTYPE_TCP
        End If
    End Function

    Protected Overrides Sub OnNavigatedTo(e As NavigationEventArgs)
        _navigationHelper.OnNavigatedTo(e)
    End Sub


    Protected Overrides Async Sub OnNavigatedFrom(e As NavigationEventArgs)
        While True
            Try
                Await Finder.StopAsync()
                Exit While
            Catch ex As Exception
                If (ex.HResult <> EposHResult.HR_E_PENDING) Then
                    Exit While
                End If
            End Try
        End While

        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If
        _navigationHelper.OnNavigatedFrom(e)
    End Sub

    Private Sub ListView_SelectionChanged(sender As Object, e As SelectionChangedEventArgs)

        MainPage.SetSelectedPrinterDiscovery(GetSelectedDevicetype(), (PrinterDiscovery_ListView.SelectedItem).Device)
        
        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If

        If Me.Frame.CanGoBack Then
            Me.Frame.GoBack()
        End If
    End Sub

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

    Private Async Sub RadioButton_Clicked(sender As Object, e As RoutedEventArgs) Handles TCP_RadioButton.Click, BT_RadioButton.Click
        Await findStart()
    End Sub
End Class

Public Class displayDeviceList

    Public Property DeviceName As String
    Public Property Device As String

    Public Sub New(ByVal deviceName As String, ByVal device As String)
        Me.DeviceName = deviceName
        Me.Device = device
    End Sub

End Class