﻿Imports Windows.UI.Core
Imports Windows.UI.Popups
Imports Windows.ApplicationModel.Resources
Imports Windows.UI.Notifications
Imports Windows.Data.Xml.Dom
Imports LibEposPrint

Public Class ShowMsg
    Private loader As ResourceLoader = New ResourceLoader()

    Const BITCNT_INT = 31

    Public Sub showEposException(errStatus As Integer, method As String)
        Dim msg As String = String.Empty

        msg = String.Format("{0}  {1}{2}  {3}",
                            loader.GetString("methoderr_errcode") + vbCrLf,
                            getEposExceptionText(errStatus) + vbCrLf,
                            loader.GetString("methoderr_method") + vbCrLf,
                            method)
        show(msg)
    End Sub

    Public Sub showIoException(errStatus As Integer, method As String)
        Dim msg As String = String.Empty

        msg = String.Format("{0}  {1}{2}  {3}",
                            loader.GetString("methoderr_errcode") + vbCrLf,
                            getIoExceptionText(errStatus) + vbCrLf,
                            loader.GetString("methoderr_method") + vbCrLf,
                            method)
        show(msg)
    End Sub

    Public Sub showError(errMsg As String)
        show(errMsg)
    End Sub

    Public Sub showStatus(result As Integer, status As Integer, battery As Integer)
        Dim msg As String = String.Empty

        msg = String.Format("{0}  {1}{2}  {3}{4}  0x{5:x04}",
                                 loader.GetString("statusmsg_result") + vbCrLf,
                                 getEposExceptionText(result) + vbCrLf,
                                 loader.GetString("statusmsg_status") + vbCrLf,
                                 getEposStatusText(status) + vbCrLf,
                                 loader.GetString("statusmsg_batterystatus") + vbCrLf,
                                 battery)

        show(msg)
    End Sub

    Public Sub showPrinterName(printerName As String, languageName As String)
        Dim msg As String = String.Empty

        msg = String.Format("{0}  {1}{2}  {3}",
                                  loader.GetString("namemsg_name") + vbCrLf, printerName + vbCrLf,
                                  loader.GetString("namemsg_language") + vbCrLf, languageName)
        show(msg)
    End Sub

    Public Sub showStatusChangeEvent(deviceName As String, status As Integer)
        Dim msg As String = String.Empty

        msg = String.Format("{0} : {1}{2} : {3}",
                            loader.GetString("getname_label_ip"),
                            deviceName + vbCrLf,
                            loader.GetString("statusmsg_status"),
                            getEposStatusText(status))
        showToast(msg)
    End Sub

    Public Sub showBatteryStatusChangeEvent(deviceName As String, battery As Integer)
        Dim msg As String = String.Empty

        msg = String.Format("{0} : {1}{2} : 0x{3:x04}",
                            loader.GetString("getname_label_ip"),
                            deviceName + vbCrLf,
                            loader.GetString("statusmsg_batterystatus"),
                            battery)
        showToast(msg)
    End Sub

    Private Async Sub show(msg As String)
        Dim msgDialog As MessageDialog = New MessageDialog("")
        msgDialog.Content = msg
        msgDialog.Commands.Add(New UICommand("OK"))
        Await msgDialog.ShowAsync()
    End Sub

    Private Sub showToast(msg As String)
        toast(msg)
    End Sub

    Private Sub toast(msg As String)

        Dim toastMsg As String = msg.Replace(vbCrLf + "  ", " : ")
        Dim xmlDoc As XmlDocument = New XmlDocument()
        xmlDoc.LoadXml("<toast><visual version='1'><binding template='ToastText01'><text id='1'>" + toastMsg + "</text></binding></visual></toast>")
        Dim toast As ToastNotification = New ToastNotification(xmlDoc)
        ToastNotificationManager.CreateToastNotifier().Show(toast)
    End Sub

    Private Function getIoExceptionText(state As Integer)
        Dim resultTxt = String.Empty

        If IoHResult.HR_S_OK = state Then
            resultTxt = "HR_S_OK"
        ElseIf IoHResult.HR_E_INVALIDARG = state Then
            resultTxt = "HR_E_INVALIDARG"
        ElseIf IoHResult.HR_E_ACCESSDENIED = state Then
            resultTxt = "HR_E_ACCESSDENIED"
        ElseIf IoHResult.HR_E_OUTOFMEMORY = state Then
            resultTxt = "HR_E_OUTOFMEMORY"
        ElseIf IoHResult.HR_E_PENDING = state Then
            resultTxt = "HR_E_PENDING"
        ElseIf IoHResult.HR_E_FAIL = state Then
            resultTxt = "HR_E_FAIL"
        Else
            resultTxt = String.Format("{0}", state)
        End If

        Return resultTxt
    End Function

    Private Function getEposExceptionText(state As Integer) As String
        Dim resultTxt = String.Empty

        If EposHResult.HR_S_OK = state Then
            resultTxt = "HR_S_OK"
        ElseIf EposHResult.HR_E_INVALIDARG = state Then
            resultTxt = "HR_E_INVALIDARG"
        ElseIf EposHResult.HR_E_ACCESSDENIED = state Then
            resultTxt = "HR_E_ACCESSDENIED"
        ElseIf EposHResult.HR_E_ABORT = state Then
            resultTxt = "HR_E_ABORT"
        ElseIf EposHResult.HR_E_PENDING = state Then
            resultTxt = "HR_E_PENDING"
        ElseIf EposHResult.HR_E_OUTOFMEMORY = state Then
            resultTxt = "HR_E_OUTOFMEMORY"
        ElseIf EposHResult.HR_E_FAIL = state Then
            resultTxt = "HR_E_FAIL"
        Else
            resultTxt = String.Format("{0}", state)
        End If

        Return resultTxt
    End Function

    Private Function getEposStatusText(status As Integer)
        Dim resultTxt As String = String.Empty
        Dim bit As Integer = 0

        For bit = 0 To BITCNT_INT
            Dim value As Integer = 1 << bit
            If (value And status) <> 0 Then
                Dim msg As String = String.Empty
                If value = Print.ST_NO_RESPONSE Then
                    msg = "NO_RESPONSE"
                ElseIf value = Print.ST_PRINT_SUCCESS Then
                    msg = "PRINT_SUCCESS"
                ElseIf value = Print.ST_DRAWER_KICK Then
                    msg = "DRAWER_KICK"
                ElseIf value = Print.ST_OFF_LINE Then
                    msg = "OFF_LINE"
                ElseIf value = Print.ST_COVER_OPEN Then
                    msg = "COVER_OPEN"
                ElseIf value = Print.ST_PAPER_FEED Then
                    msg = "PAPER_FEED"
                ElseIf value = Print.ST_WAIT_ON_LINE Then
                    msg = "WAIT_ON_LINE"
                ElseIf value = Print.ST_PANEL_SWITCH Then
                    msg = "PANEL_SWITCH"
                ElseIf value = Print.ST_MECHANICAL_ERR Then
                    msg = "MECHANICAL_ERR"
                ElseIf value = Print.ST_AUTOCUTTER_ERR Then
                    msg = "AUTOCUTTER_ERR"
                ElseIf value = Print.ST_UNRECOVER_ERR Then
                    msg = "UNRECOVER_ERR"
                ElseIf value = Print.ST_AUTORECOVER_ERR Then
                    msg = "AUTORECOVER_ERR"
                ElseIf value = Print.ST_RECEIPT_NEAR_END Then
                    msg = "RECEIPT_NEAR_END"
                ElseIf value = Print.ST_RECEIPT_END Then
                    msg = "RECEIPT_END"
                ElseIf value = Print.ST_BUZZER Then
                ElseIf value = Print.ST_HEAD_OVERHEAT Then
                    msg = "HEAD_OVERHEAT"
                ElseIf value = Print.ST_MOTOR_OVERHEAT Then
                    msg = "MOTOR_OVERHEAT"
                ElseIf value = Print.ST_BATTERY_OVERHEAT Then
                    msg = "BATTERY_OVERHEAT"
                ElseIf value = Print.ST_WRONG_PAPER Then
                    msg = "WRONG_PAPER"
                Else
                    msg = String.Format("{0}", value)
                End If

                If (String.IsNullOrEmpty(msg) = False) Then
                    If (String.IsNullOrEmpty(resultTxt) = False) Then
                        resultTxt += vbCrLf + "  "
                    End If
                    resultTxt += msg
                End If
            End If
        Next
        Return resultTxt
    End Function
End Class
