﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class TextPage
    Inherits Page

    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property
    Private _navigationHelper As Common.NavigationHelper

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Const SEND_TIMEOUT = 10 * 1000
    Const SIZEWIDTH_MAX = 8
    Const SIZEWIDTH_MIN = 1
    Const SIZEHEIGHT_MAX = 8
    Const SIZEHEIGHT_MIN = 1
    Const FONT_A = 0
    Const FONT_B = 1
    Const FONT_C = 2
    Const FONT_D = 3
    Const FONT_E = 4
    Const ALIGN_LEFT = 0
    Const ALIGN_CENTER = 1
    Const ALIGN_RIGHT = 2
    Const LANG_ANK = 0
    Const LANG_JAPANESE = 1
    Const LANG_CHINESE = 2
    Const LANG_TAIWAN = 3
    Const LANG_KOREAN = 4
    Const LANG_THAI = 5
    Const LANG_SOUTHASIA = 6

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' init font list
        Me.Font_ComboBox.Items.Add(_loader.GetString("font_a"))
        Me.Font_ComboBox.Items.Add(_loader.GetString("font_b"))
        Me.Font_ComboBox.Items.Add(_loader.GetString("font_c"))
        Me.Font_ComboBox.Items.Add(_loader.GetString("font_d"))
        Me.Font_ComboBox.Items.Add(_loader.GetString("font_e"))

        ' init align list
        Me.Align_ComboBox.Items.Add(_loader.GetString("align_left"))
        Me.Align_ComboBox.Items.Add(_loader.GetString("align_center"))
        Me.Align_ComboBox.Items.Add(_loader.GetString("align_right"))

        ' init language list
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_ank"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_japanese"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_simplified_chinese"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_traditional_chinese"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_korean"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_thai"))
        Me.Language_ComboBox.Items.Add(_loader.GetString("language_vietnamese"))

        ' init size list
        Dim i As Integer
        For i = SIZEWIDTH_MIN To SIZEWIDTH_MAX
            Me.SizeWidth_ComboBox.Items.Add(i.ToString())
        Next

        For i = SIZEHEIGHT_MIN To SIZEHEIGHT_MAX
            Me.SizeHeight_ComboBox.Items.Add(i.ToString())
        Next

        ' default setting
        Me.Font_ComboBox.SelectedIndex = FONT_A
        Me.Align_ComboBox.SelectedIndex = ALIGN_LEFT
        Me.Language_ComboBox.SelectedIndex = LANG_ANK
        Me.SizeWidth_ComboBox.SelectedIndex = 0
        Me.SizeHeight_ComboBox.SelectedIndex = 0

        Me.PrintCharacters_TxtBox.Text = _loader.GetString("text_edit_text")
        Me.LineSpacing_TxtBox.Text = "30"
        Me.XPosition_TxtBox.Text = "0"
        Me.FeedUnit_TxtBox.Text = "30"

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await PrintText()
        Me.IsEnabled = True
    End Sub

    Private Async Function PrintText() As Task

        If String.IsNullOrWhiteSpace(PrintCharacters_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            ' create builder
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddTextFont"
            builder.AddTextFont(GetBuilderFont())

            method = "AddTextAlign"
            builder.AddTextAlign(GetBuilderAlign())

            method = "AddTextLineSpace"
            builder.AddTextLineSpace(GetBuilderLineSpace())

            method = "AddTextLang"
            builder.AddTextLang(GetBuilderLnaguage())

            method = "AddTextSize"
            builder.AddTextSize(GetBuilderSizeW(), GetBuilderSizeH())

            method = "AddTextStyle"
            builder.AddTextStyle(Reverse.REVERSE_FALSE, GetBuilderUnderline(), GetBuilderStyleBold(), Color.COLOR_1)

            method = "AddTextPosition"
            builder.AddTextPosition(GetBuilderXPostion())

            method = "AddText"
            builder.AddText(GetBuilderText())

            method = "AddFeedUnit"
            builder.AddFeedUnit(GetBuilderFeedUnit())

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If

    End Function

    Private Function GetBuilderFont() As Font

        Select Case (Font_ComboBox.SelectedIndex)
            Case FONT_B
                Return Font.FONT_B
            Case FONT_C
                Return Font.FONT_C
            Case FONT_D
                Return Font.FONT_D
            Case FONT_E
                Return Font.FONT_E
            Case Else
                Return Font.FONT_A
        End Select

    End Function

    Private Function GetBuilderAlign() As Align

        Select Case (Align_ComboBox.SelectedIndex)
            Case ALIGN_CENTER
                Return Align.ALIGN_CENTER
            Case ALIGN_RIGHT
                Return Align.ALIGN_RIGHT
            Case Else
                Return Align.ALIGN_LEFT
        End Select

    End Function

    Private Function GetBuilderLineSpace() As Integer

        Try
            Return Integer.Parse(LineSpacing_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderLnaguage() As Lang

        Select Case (Language_ComboBox.SelectedIndex)
            Case LANG_JAPANESE
                Return Lang.LANG_JA
            Case LANG_CHINESE
                Return Lang.LANG_ZH_CN
            Case LANG_TAIWAN
                Return Lang.LANG_ZH_TW
            Case LANG_KOREAN
                Return Lang.LANG_KO
            Case LANG_THAI
                Return Lang.LANG_TH
            Case LANG_SOUTHASIA
                Return Lang.LANG_VI
            Case Else
                Return Lang.LANG_EN
        End Select

    End Function

    Private Function GetBuilderSizeW() As Integer
        Return SizeWidth_ComboBox.SelectedIndex + 1
    End Function

    Private Function GetBuilderSizeH() As Integer
        Return SizeHeight_ComboBox.SelectedIndex + 1
    End Function

    Private Function GetBuilderStyleBold() As Emphasis

        If style_ToggleButton.IsOn Then
            Return Emphasis.EMPHASIS_TRUE
        Else
            Return Emphasis.EMPHASIS_FALSE
        End If

    End Function

    Private Function GetBuilderUnderline() As Underline

        If underline_ToggleButton.IsOn Then
            Return Underline.UNDERLINE_TRUE
        Else
            Return Underline.UNDERLINE_FALSE
        End If

    End Function

    Private Function GetBuilderXPostion() As Integer

        Try
            Return Integer.Parse(XPosition_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderFeedUnit() As Integer

        Try
            Return Integer.Parse(FeedUnit_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderText() As String
        Return PrintCharacters_TxtBox.Text
    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
