﻿Imports System.Threading.Tasks
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class TwoDCodePage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000
    Const SYMBOL_PDF417_STANDARD = 0
    Const SYMBOL_PDF417_TRUNCATED = 1
    Const SYMBOL_QRCODE_MODEL1 = 2
    Const SYMBOL_QRCODE_MODEL2 = 3
    Const SYMBOL_MAXICODE_MODE2 = 4
    Const SYMBOL_MAXICODE_MODE3 = 5
    Const SYMBOL_MAXICODE_MODE4 = 6
    Const SYMBOL_MAXICODE_MODE5 = 7
    Const SYMBOL_MAXICODE_MODE6 = 8
    Const SYMBOL_GS1_DATABAR_STACKED = 9
    Const SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL = 10
    Const SYMBOL_GS1_DATABAR_EXPANDED_STACKED = 11
    Const SYMBOL_AZTECCODE_FULLRANGE = 12
    Const SYMBOL_AZTECCODE_COMPACT = 13
    Const SYMBOL_DATAMATRIX_SQUARE = 14
    Const SYMBOL_DATAMATRIX_RECTANGLE8 = 15
    Const SYMBOL_DATAMATRIX_RECTANGLE12 = 16
    Const SYMBOL_DATAMATRIX_RECTANGLE16 = 17

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing
    Private _defaultTwoDcodeData As String = String.Empty

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()
        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' init type list
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_pdf417s"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_pdf417t"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_qr1"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_qr2"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_maxcode2"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_maxcode3"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_maxcode4"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_maxcode5"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_maxcode6"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_gr1st"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_gr1so"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_gr1es"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_aztecfull"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_azteccompact"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_dmsquare"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_dmrect_8rows"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_dmrect_12rows"))
        Me.TwoDCodeType_ComboBox.Items.Add(_loader.GetString("code2type_dmrect_16rows"))

        ' Default setting
        Me.TwoDCodeType_ComboBox.SelectedIndex = SYMBOL_PDF417_STANDARD
        SetDefaultTwoDcodeErrorLevel()
        SetDefaultTwoDcodeData()
        SetDefaultTwoDcodeModuleSize()
        SetDefaultTwoDcodeMaxSizeTxtBox()

        Me.ModuleSizeWidth_TxtBox.Text = "3"
        Me.ModuleSizeHeight_TxtBox.Text = "3"

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await PrintBarcode()
        Me.IsEnabled = True
    End Sub

    Private Async Function PrintBarcode() As Task

        If String.IsNullOrWhiteSpace(TwoDcodeData_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            ' create builder
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddSymbol"
            builder.AddSymbol(GetBuilderData(), GetBuilderType(), GetBuilderLevel1(), GetBuilderLevel2(), GetBuilderWidth(), GetBuilderHeight(), GetBuilderMaxSize())

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If

    End Function

    Private Function GetBuilderData() As String
        Return TwoDcodeData_TxtBox.Text
    End Function

    Private Function GetBuilderType() As SymbolType

        Dim typelist As SymbolType() = {
                                        SymbolType.SYMBOL_PDF417_STANDARD,
                                        SymbolType.SYMBOL_PDF417_TRUNCATED,
                                        SymbolType.SYMBOL_QRCODE_MODEL_1,
                                        SymbolType.SYMBOL_QRCODE_MODEL_2,
                                        SymbolType.SYMBOL_MAXICODE_MODE_2,
                                        SymbolType.SYMBOL_MAXICODE_MODE_3,
                                        SymbolType.SYMBOL_MAXICODE_MODE_4,
                                        SymbolType.SYMBOL_MAXICODE_MODE_5,
                                        SymbolType.SYMBOL_MAXICODE_MODE_6,
                                        SymbolType.SYMBOL_GS1_DATABAR_STACKED,
                                        SymbolType.SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL,
                                        SymbolType.SYMBOL_GS1_DATABAR_EXPANDED_STACKED,
                                        SymbolType.SYMBOL_AZTECCODE_FULLRANGE,
                                        SymbolType.SYMBOL_AZTECCODE_COMPACT,
                                        SymbolType.SYMBOL_DATAMATRIX_SQUARE,
                                        SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_8,
                                        SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_12,
                                        SymbolType.SYMBOL_DATAMATRIX_RECTANGLE_16}

        Return typelist(TwoDCodeType_ComboBox.SelectedIndex)

    End Function

    Private Function GetBuilderLevel1() As Level

        Dim levellist As Level() = Nothing

        Dim pdf417levellist As Level() = {
                                            Level.LEVEL_0,
                                            Level.LEVEL_1,
                                            Level.LEVEL_2,
                                            Level.LEVEL_3,
                                            Level.LEVEL_4,
                                            Level.LEVEL_5,
                                            Level.LEVEL_6,
                                            Level.LEVEL_7,
                                            Level.LEVEL_8}
        Dim qrlevellist As Level() = {
                                        Level.LEVEL_L,
                                        Level.LEVEL_M,
                                        Level.LEVEL_Q,
                                        Level.LEVEL_H}

        Select Case (TwoDCodeType_ComboBox.SelectedIndex)
            Case SYMBOL_PDF417_STANDARD,
                 SYMBOL_PDF417_TRUNCATED
                levellist = pdf417levellist
            Case SYMBOL_QRCODE_MODEL1,
                 SYMBOL_QRCODE_MODEL2
                levellist = qrlevellist
        End Select

        If levellist Is Nothing Then
            Return Level.LEVEL_DEFAULT
        Else
            Return levellist(ErrorCorrectionLevel_ComboBox.SelectedIndex)
        End If

    End Function

    Private Function GetBuilderLevel2() As Integer

        Try
            Return Integer.Parse(ErrorCorrectionLevel_TxtBox.Text)
        Catch ex As Exception
            Return 0
        End Try

    End Function

    Private Function GetBuilderWidth() As Integer

        If ModuleSizeWidth_TxtBox.IsEnabled Then
            Try
                Return Integer.Parse(ModuleSizeWidth_TxtBox.Text)
            Catch ex As Exception
                Return 0
            End Try
        Else
            Return Builder.PARAM_UNSPECIFIED
        End If

    End Function

    Private Function GetBuilderHeight() As Integer

        If ModuleSizeHeight_TxtBox.IsEnabled Then
            Try
                Return Integer.Parse(ModuleSizeHeight_TxtBox.Text)
            Catch ex As Exception
                Return 0
            End Try
        Else
            Return Builder.PARAM_UNSPECIFIED
        End If


    End Function

    Private Function GetBuilderMaxSize() As Integer

        If MaxSize_TxtBox.IsEnabled Then
            Try
                Return Integer.Parse(MaxSize_TxtBox.Text)
            Catch ex As Exception
                Return 0
            End Try
        Else
            Return Builder.PARAM_UNSPECIFIED
        End If

    End Function

    Private Sub SetDefaultTwoDcodeData()

        _defaultTwoDcodeData = GetDefaultTwoDcodeData()
        TwoDcodeData_TxtBox.Text = _defaultTwoDcodeData

    End Sub

    Private Function GetDefaultTwoDcodeData() As String

        Dim TwoDcodeDataList() As String = {
                                            _loader.GetString("code2data_pdf417"),
                                            _loader.GetString("code2data_pdf417t"),
                                            _loader.GetString("code2data_qr1"),
                                            _loader.GetString("code2data_qr2"),
                                            _loader.GetString("code2data_maxcode2"),
                                            _loader.GetString("code2data_maxcode3"),
                                            _loader.GetString("code2data_maxcode4"),
                                            _loader.GetString("code2data_maxcode5"),
                                            _loader.GetString("code2data_maxcode6"),
                                            _loader.GetString("code2data_gr1so"),
                                            _loader.GetString("code2data_gr1st"),
                                            _loader.GetString("code2data_gr1es"),
                                            _loader.GetString("code2data_azteccompact"),
                                            _loader.GetString("code2data_aztecfull"),
                                            _loader.GetString("code2data_dmsquare"),
                                            _loader.GetString("code2data_dmrect_8rows"),
                                            _loader.GetString("code2data_dmrect_12rows"),
                                            _loader.GetString("code2data_dmrect_16rows")}

        Return TwoDcodeDataList(TwoDCodeType_ComboBox.SelectedIndex)

    End Function

    Private Sub SetDefaultTwoDcodeErrorLevel()

        Dim iTwoDCodeType As Integer = TwoDCodeType_ComboBox.SelectedIndex
        SetDefaultTwoDcodeErrorLevelComboBox(iTwoDCodeType)
        SetDefaultTwoDcodeErrorLevelTxtBox(iTwoDCodeType)

    End Sub

    Private Sub SetDefaultTwoDcodeErrorLevelComboBox(iTwoDCodeType As Integer)

        Dim isEnableCombo As Boolean = True
        Dim errorLevelCombo() As String = Nothing
        Dim pdf417ErrorLevel() As String = {
                                            _loader.GetString("level_pdf417_0"),
                                            _loader.GetString("level_pdf417_1"),
                                            _loader.GetString("level_pdf417_2"),
                                            _loader.GetString("level_pdf417_3"),
                                            _loader.GetString("level_pdf417_4"),
                                            _loader.GetString("level_pdf417_5"),
                                            _loader.GetString("level_pdf417_6"),
                                            _loader.GetString("level_pdf417_7"),
                                            _loader.GetString("level_pdf417_8")}

        Dim qrErrorLevel() As String = {
                                        _loader.GetString("level_qr_l"),
                                        _loader.GetString("level_qr_m"),
                                        _loader.GetString("level_qr_q"),
                                        _loader.GetString("level_qr_h")}

        Dim noErrorLevel() As String = {_loader.GetString("empty")}

        Select Case (iTwoDCodeType)
            Case SYMBOL_PDF417_STANDARD,
                 SYMBOL_PDF417_TRUNCATED
                errorLevelCombo = pdf417ErrorLevel
            Case SYMBOL_QRCODE_MODEL1,
                 SYMBOL_QRCODE_MODEL2
                errorLevelCombo = qrErrorLevel
            Case Else
                errorLevelCombo = noErrorLevel
                isEnableCombo = False
        End Select

        ErrorCorrectionLevel_ComboBox.Items.Clear()

        Dim i As Integer
        For i = 0 To ErrorLevelCombo.Length - 1
            ErrorCorrectionLevel_ComboBox.Items.Add(ErrorLevelCombo(i))
        Next

        ErrorCorrectionLevel_ComboBox.SelectedIndex = 0

        ErrorCorrectionLevel_ComboBox.IsEnabled = isEnableCombo

    End Sub

    Private Sub SetDefaultTwoDcodeErrorLevelTxtBox(iTwoDCodeType As Integer)

        Dim isEnableTxtBox As Boolean = False
        Dim errorLevelTxtBox As String = String.Empty

        Select Case (iTwoDCodeType)
            Case SYMBOL_AZTECCODE_FULLRANGE,
                 SYMBOL_AZTECCODE_COMPACT
                isEnableTxtBox = True
                errorLevelTxtBox = "23"
        End Select

        ErrorCorrectionLevel_TxtBox.IsEnabled = isEnableTxtBox
        ErrorCorrectionLevel_TxtBox.Text = errorLevelTxtBox

    End Sub

    Private Sub SetDefaultTwoDcodeMaxSizeTxtBox()

        Dim isEnableTxtBox As Boolean = False

        Dim maxSizeTxtBox As String = String.Empty

        Dim iTwoDCodeType As String = TwoDCodeType_ComboBox.SelectedIndex

        Select Case (iTwoDCodeType)
            Case SYMBOL_PDF417_STANDARD,
                SYMBOL_PDF417_TRUNCATED
                isEnableTxtBox = True
                maxSizeTxtBox = "0"
            Case SYMBOL_GS1_DATABAR_STACKED,
                SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL,
                SYMBOL_GS1_DATABAR_EXPANDED_STACKED
                isEnableTxtBox = True
                maxSizeTxtBox = "114"
        End Select
        MaxSize_TxtBox.IsEnabled = isEnableTxtBox
        MaxSize_TxtBox.Text = maxSizeTxtBox

    End Sub

    Private Sub SetDefaultTwoDcodeModuleSize()

        Dim isEnableTxtBoxWidth As Boolean = False
        Dim isEnableTxtBoxHeight As Boolean = False

        Dim moduleSizeWidth As String = String.Empty
        Dim moduleSizeHeight As String = String.Empty

        Dim iTwoDCodeType As Integer = TwoDCodeType_ComboBox.SelectedIndex

        Select Case (iTwoDCodeType)
            Case SYMBOL_PDF417_STANDARD,
                 SYMBOL_PDF417_TRUNCATED
                isEnableTxtBoxWidth = True
                isEnableTxtBoxHeight = True
                moduleSizeWidth = "3"
                moduleSizeHeight = "3"
            Case SYMBOL_QRCODE_MODEL1,
                 SYMBOL_QRCODE_MODEL2,
                 SYMBOL_AZTECCODE_FULLRANGE,
                 SYMBOL_AZTECCODE_COMPACT,
                 SYMBOL_DATAMATRIX_SQUARE,
                 SYMBOL_DATAMATRIX_RECTANGLE8,
                 SYMBOL_DATAMATRIX_RECTANGLE12,
                 SYMBOL_DATAMATRIX_RECTANGLE16
                isEnableTxtBoxWidth = True
                isEnableTxtBoxHeight = False
                moduleSizeWidth = "3"
                moduleSizeHeight = String.Empty
            Case SYMBOL_MAXICODE_MODE2,
                 SYMBOL_MAXICODE_MODE3,
                 SYMBOL_MAXICODE_MODE4,
                 SYMBOL_MAXICODE_MODE5,
                 SYMBOL_MAXICODE_MODE6
                isEnableTxtBoxWidth = False
                isEnableTxtBoxHeight = False
                moduleSizeWidth = String.Empty
                moduleSizeHeight = String.Empty
            Case SYMBOL_GS1_DATABAR_STACKED,
                 SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL,
                 SYMBOL_GS1_DATABAR_EXPANDED_STACKED
                isEnableTxtBoxWidth = True
                isEnableTxtBoxHeight = False
                moduleSizeWidth = "2"
                moduleSizeHeight = String.Empty
        End Select

        ModuleSizeWidth_TxtBox.IsEnabled = isEnableTxtBoxWidth
        ModuleSizeWidth_TxtBox.Text = moduleSizeWidth

        ModuleSizeHeight_TxtBox.IsEnabled = isEnableTxtBoxHeight
        ModuleSizeHeight_TxtBox.Text = moduleSizeHeight

    End Sub

    Private Sub TwoDcodeType_Selected(sender As Object, e As SelectionChangedEventArgs)

        Dim isChangedTwoDcodeData As Boolean = False
        setDefaultTwoDcodeErrorLevel()
        setDefaultTwoDcodeMaxSizeTxtBox()
        setDefaultTwoDcodeModuleSize()

        If Equals(_defaultTwoDcodeData, TwoDcodeData_TxtBox.Text) Then
            isChangedTwoDcodeData = True
        End If

        If isChangedTwoDcodeData = True Then
            SetDefaultTwoDcodeData()
        Else
            ' not update
            Return
        End If

    End Sub

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
