#import "BarcodeView.h"
#import <QuartzCore/QuartzCore.h>
#import "ePOS-Print.h"
#import "ShowMsg.h"

#define SEND_TIMEOUT    10 * 1000
#define HRI_NONE_INDEX  0
#define HRI_ABOVE_INDEX 1
#define HRI_BELOW_INDEX 2
#define HRI_BOTH_INDEX  3
#define FONT_A_INDEX    0
#define FONT_B_INDEX    1
#define FONT_C_INDEX    2
#define FONT_D_INDEX    3
#define FONT_E_INDEX    4

@interface BarcodeView ()
- (void)printBarcode;
- (NSString*)getBuilderData;
- (int)getBuilderType;
- (int)getBuilderHri;
- (int)getBuilderFont;
- (long)getBuilderWidth;
- (long)getBuilderHeight;
- (void)setDefaultBarcodeData;
- (NSString*) getDefaultBarcodeData;
@end

@implementation BarcodeView
@synthesize scrollView_;
@synthesize itemView_;
@synthesize buttonType_;
@synthesize buttonHRI_;
@synthesize buttonFont_;
@synthesize textData_;
@synthesize textModuleSizeW_;
@synthesize textModuleSizeH_;

- (id)initWithPrinter:(EposPrint*)printer
          printername:(NSString*)printername
             language:(int)language
{
    self = [super init];
    if (self) {
        printer_ = [printer retain];
        printername_ = [printername retain];
        language_ = language;
        changeBarcodeData_ = NO;
        defaultBarcodeData_ = @"";
    }
    return self;
}

- (void)dealloc {
    [scrollView_ release];
    [itemView_ release];
    [buttonType_ release];
    [buttonHRI_ release];
    [buttonFont_ release];
    [textData_ release];
    [textModuleSizeW_ release];
    [textModuleSizeH_ release];
    
    [printer_ release];
    [printername_ release];
    [typeList_ release];
    [hriList_ release];
    [fontList_ release];
    [defaultBarcodeData_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //set multiline edit style
    textData_.layer.borderWidth = 1;
    textData_.layer.borderColor = [[UIColor grayColor] CGColor];
    textData_.layer.cornerRadius = 8;
    
    //set multiline edit keyboard accessory
    UIToolbar *doneToolbar = [[[UIToolbar alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 44)] autorelease];
    doneToolbar.barStyle = UIBarStyleBlackTranslucent;
    [doneToolbar sizeToFit];
    UIBarButtonItem *space = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:self action:nil] autorelease];
    UIBarButtonItem *doneButton = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(eventDidPushDone:)] autorelease];
    
    NSMutableArray *items = [NSMutableArray arrayWithObjects:space, doneButton, nil];
    [doneToolbar setItems:items animated:YES];
    textData_.inputAccessoryView = doneToolbar;
    
    //init type list
    typeList_ = [[PickerTableView alloc] init];
    NSString *typelist[] = {
        @"bartype_upca",
        @"bartype_upce",
        @"bartype_ean13",
        @"bartype_jan13",
        @"bartype_ean8",
        @"bartype_jan8",
        @"bartype_code39",
        @"bartype_itf",
        @"bartype_codabar",
        @"bartype_code93",
        @"bartype_code128",
        @"bartype_gs1128",
        @"bartype_gs1om",
        @"bartype_gs1tr",
        @"bartype_gs1li",
        @"bartype_gs1ex",
        nil,
    };
    items = [[[NSMutableArray alloc] init] autorelease];
    for(int i = 0; typelist[i] != nil; i++){
        [items addObject:NSLocalizedString(typelist[i], @"")];
    }
    [buttonType_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [typeList_ setItemList:items];
    typeList_.delegate = self;
    
    //init hri list
    hriList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"hri_none", @"")];
    [items addObject:NSLocalizedString(@"hri_above", @"")];
    [items addObject:NSLocalizedString(@"hri_below", @"")];
    [items addObject:NSLocalizedString(@"hri_both", @"")];
    [buttonHRI_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [hriList_ setItemList:items];
    hriList_.delegate = self;
    
    //init font list
    fontList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"font_a", @"")];
    [items addObject:NSLocalizedString(@"font_b", @"")];
    [items addObject:NSLocalizedString(@"font_c", @"")];
	[items addObject:NSLocalizedString(@"font_d", @"")];
	[items addObject:NSLocalizedString(@"font_e", @"")];
    [buttonFont_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [fontList_ setItemList:items];
    fontList_.delegate = self;
    
    //default setting
    textModuleSizeW_.text = @"3";
    textModuleSizeH_.text = @"162";
    hriList_.selectIndex = 2;
    [buttonHRI_ setTitle:[hriList_ getItem:hriList_.selectIndex] forState:UIControlStateNormal];

    [self setDefaultBarcodeData];
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    scrollBaseView_ = scrollView_;
    defaultActiveViewItem_ = textData_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setButtonType_:nil];
    [self setButtonHRI_:nil];
    [self setButtonFont_:nil];
    [self setTextData_:nil];
    [self setTextModuleSizeW_:nil];
    [self setTextModuleSizeH_:nil];
    [super viewDidUnload];
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj {
    if(obj == typeList_){
        [buttonType_ setTitle:[typeList_ getItem:position] forState:UIControlStateNormal];
        
        //check default change
        if(![defaultBarcodeData_ isEqualToString:textData_.text]){
            changeBarcodeData_ = YES;
        }
        if(changeBarcodeData_){
            //not update
            return ;
        }
        
        //update barcode data
        [self setDefaultBarcodeData];
    }else if(obj == hriList_){
        [buttonHRI_ setTitle:[hriList_ getItem:position] forState:UIControlStateNormal];
    }else if(obj == fontList_){
        [buttonFont_ setTitle:[fontList_ getItem:position] forState:UIControlStateNormal];
    }
}

- (void)eventDidPushDone:(id)sender
{
    [textData_ resignFirstResponder];
}

- (IBAction)eventDidPush:(id)sender {
    switch(((UIView*)sender).tag){
        case 0:
            [self printBarcode];
            break;
        case 1:
            [typeList_ show];
            break;
        case 2:
            [hriList_ show];
            break;
        case 3:
            [fontList_ show];
            break;
        default:
            break;
    }
}

- (void)printBarcode
{
    if(textData_.text.length == 0){
        [ShowMsg showError:@"errmsg_notext"];
        return ;
    }
    
    //create builder
    EposBuilder *builder = [[[EposBuilder alloc] initWithPrinterModel:printername_ Lang:language_] autorelease];
    if(builder == nil){
        return ;
    }

    //add command
    int result = [builder addBarcode:[self getBuilderData]
                                Type:[self getBuilderType]
                                 Hri:[self getBuilderHri]
                                Font:[self getBuilderFont]
                               Width:[self getBuilderWidth]
                              Height:[self getBuilderHeight]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addBarcode"];
        return ;
    }
    
    //send builder data
    unsigned long status = 0;
    unsigned long battery = 0;
    result = [printer_ sendData:builder Timeout:SEND_TIMEOUT Status:&status Battery:&battery];
    [ShowMsg showStatus:result status:status battery:battery];
    
    //remove builder
    [builder clearCommandBuffer];
}

- (NSString*)getBuilderData
{
    return textData_.text;
}

- (int)getBuilderType
{
    int typelist[] = {
        EPOS_OC_BARCODE_UPC_A,
        EPOS_OC_BARCODE_UPC_E,
        EPOS_OC_BARCODE_EAN13,
        EPOS_OC_BARCODE_JAN13,
        EPOS_OC_BARCODE_EAN8,
        EPOS_OC_BARCODE_JAN8,
        EPOS_OC_BARCODE_CODE39,
        EPOS_OC_BARCODE_ITF,
        EPOS_OC_BARCODE_CODABAR,
        EPOS_OC_BARCODE_CODE93,
        EPOS_OC_BARCODE_CODE128,
        EPOS_OC_BARCODE_GS1_128,
        EPOS_OC_BARCODE_GS1_DATABAR_OMNIDIRECTIONAL,
        EPOS_OC_BARCODE_GS1_DATABAR_TRUNCATED,
        EPOS_OC_BARCODE_GS1_DATABAR_LIMITED,
        EPOS_OC_BARCODE_GS1_DATABAR_EXPANDED,
    };
    
    return typelist[typeList_.selectIndex];
}

- (int)getBuilderHri
{
    switch(hriList_.selectIndex){
        case HRI_ABOVE_INDEX:
            return EPOS_OC_HRI_ABOVE;
        case HRI_BELOW_INDEX:
            return EPOS_OC_HRI_BELOW;
        case HRI_BOTH_INDEX:
            return EPOS_OC_HRI_BOTH;
        case HRI_NONE_INDEX:
        default:
            return EPOS_OC_HRI_NONE;
    }
}

- (int)getBuilderFont
{
    switch(fontList_.selectIndex){
        case FONT_B_INDEX:
            return EPOS_OC_FONT_B;
        case FONT_C_INDEX:
            return EPOS_OC_FONT_C;
		case FONT_D_INDEX:
            return EPOS_OC_FONT_D;
		case FONT_E_INDEX:
            return EPOS_OC_FONT_E;
        case FONT_A_INDEX:
        default:
            return EPOS_OC_FONT_A;
    }
}

- (long)getBuilderWidth
{
    return [textModuleSizeW_.text intValue];
}

- (long)getBuilderHeight
{
    return [textModuleSizeH_.text intValue];
}

- (void)setDefaultBarcodeData
{
    //update default data
    [defaultBarcodeData_ release];
    defaultBarcodeData_ = [[self getDefaultBarcodeData] retain];
    
    //update ui
    textData_.text = defaultBarcodeData_;
}

- (NSString*) getDefaultBarcodeData
{
    NSString *dataList[] = {
        @"bardata_upca",
        @"bardata_upce",
        @"bardata_ean13",
        @"bardata_jan13",
        @"bardata_ean8",
        @"bardata_jan8",
        @"bardata_code39",
        @"bardata_itf",
        @"bardata_codabar",
        @"bardata_code93",
        @"bardata_code128",
        @"bardata_gs1128",
        @"bardata_gs1om",
        @"bardata_gs1tr",
        @"bardata_gs1li",
        @"bardata_gs1ex",
    };
    
    return NSLocalizedString(dataList[typeList_.selectIndex], @"");
}

@end
