#import "Code2dView.h"
#import <QuartzCore/QuartzCore.h>
#import "ePOS-Print.h"
#import "ShowMsg.h"

#define SEND_TIMEOUT                                10 * 1000

#define SYMBOL_PDF417_STANDARD_INDEX                0
#define SYMBOL_PDF417_TRUNCATED_INDEX               1
#define SYMBOL_QRCODE_MODEL_1_INDEX                 2
#define SYMBOL_QRCODE_MODEL_2_INDEX                 3
#define SYMBOL_MAXICODE_MODE_2                      4
#define SYMBOL_MAXICODE_MODE_3                      5
#define SYMBOL_MAXICODE_MODE_4                      6
#define SYMBOL_MAXICODE_MODE_5                      7
#define SYMBOL_MAXICODE_MODE_6                      8
#define SYMBOL_GS1_DATABAR_STACKED                  9
#define SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL  10
#define SYMBOL_GS1_DATABAR_EXPANDED_STACKED         11
#define SYMBOL_AZTECCODE_FULLRANGE                  12
#define SYMBOL_AZTECCODE_COMPACT                    13
#define SYMBOL_DATAMATRIX_SQUARE                    14
#define SYMBOL_DATAMATRIX_RECTANGLE_8               15
#define SYMBOL_DATAMATRIX_RECTANGLE_12              16
#define SYMBOL_DATAMATRIX_RECTANGLE_16              17

@interface Code2dView ()
- (void)printBarcode;
- (NSString*)getBuilderData;
- (int)getBuilderType;
- (int)getBuilderLevel;
- (long)getBuilderWidth;
- (long)getBuilderHeight;
- (long)getBuilderMaxSize;
- (void)setLevelList;
- (void)setDefaultSetting;
- (void)setDefaultBarcodeData;
- (NSString*)getDefaultBarcodeData;
- (int)getBuilderLevelRange;
@end

@implementation Code2dView
@synthesize buttonType_;
@synthesize buttonLevel_;
@synthesize textData_;
@synthesize textModuleSizeW_;
@synthesize textModuleSizeH_;
@synthesize textMaxSize_;
@synthesize scrollView_;
@synthesize itemView_;
@synthesize textLevelRange_;

- (id)initWithPrinter:(EposPrint*)printer
          printername:(NSString*)printername
             language:(int)language
{
    self = [super init];
    if (self) {
        printer_ = [printer retain];
        printername_ = [printername retain];
        language_ = language;
        changeBarcodeData_ = NO;
        defaultBarcodeData_ = @"";
    }
    return self;
}

- (void)dealloc {
    [buttonType_ release];
    [buttonLevel_ release];
    [textData_ release];
    [textModuleSizeW_ release];
    [textModuleSizeH_ release];
    [textMaxSize_ release];
    [scrollView_ release];
    [itemView_ release];
    [textLevelRange_ release];

    [printer_ release];
    [printername_ release];
    [typeList_ release];
    [levelList_ release];
    [defaultBarcodeData_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [scrollView_ setContentSize:itemView_.frame.size];
    
    //set multiline edit style
    textData_.layer.borderWidth = 1;
    textData_.layer.borderColor = [[UIColor grayColor] CGColor];
    textData_.layer.cornerRadius = 8;
    
    //set multiline edit keyboard accessory
    UIToolbar *doneToolbar = [[[UIToolbar alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 44)] autorelease];
    doneToolbar.barStyle = UIBarStyleBlackTranslucent;
    [doneToolbar sizeToFit];
    UIBarButtonItem *space = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:self action:nil] autorelease];
    UIBarButtonItem *doneButton = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(eventDidPushDone:)] autorelease];
    
    NSMutableArray *items = [NSMutableArray arrayWithObjects:space, doneButton, nil];
    [doneToolbar setItems:items animated:YES];
    textData_.inputAccessoryView = doneToolbar;
    
    //init type list
    typeList_ = [[PickerTableView alloc] init];
    NSString *typelist[] = {
        @"code2type_pdf417s",
        @"code2type_pdf417t",
        @"code2type_qr1",
        @"code2type_qr2",
        @"code2type_maxcode2",
        @"code2type_maxcode3",
        @"code2type_maxcode4",
        @"code2type_maxcode5",
        @"code2type_maxcode6",
        @"code2type_gr1st",
        @"code2type_gr1so",
        @"code2type_gr1es",
        @"code2type_aztecfull",
        @"code2type_azteccompact",
        @"code2type_dmsquare",
        @"code2type_dmrect_8rows",
        @"code2type_dmrect_12rows",
        @"code2type_dmrect_16rows",
        nil,
    };
    items = [[[NSMutableArray alloc] init] autorelease];
    for(int i = 0; typelist[i] != nil; i++){
        [items addObject:NSLocalizedString(typelist[i], @"")];
    }
    [buttonType_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [typeList_ setItemList:items];
    typeList_.delegate = self;

    //init level list
    [self setLevelList];
    
    //default setting
    [self setDefaultSetting];
    [self setDefaultBarcodeData];
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];

    //auto scroll setting
    scrollBaseView_ = scrollView_;
    defaultActiveViewItem_ = textData_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setButtonType_:nil];
    [self setButtonLevel_:nil];
    [self setTextData_:nil];
    [self setTextModuleSizeW_:nil];
    [self setTextModuleSizeH_:nil];
    [self setTextMaxSize_:nil];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setTextLevelRange_:nil];
    [super viewDidUnload];
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj {
    if(obj == typeList_){
        [buttonType_ setTitle:[typeList_ getItem:position] forState:UIControlStateNormal];
        
        //reset level list
        [self setLevelList];
        
        //set default settings
        [self setDefaultSetting];
        
        //check default change
        if(![defaultBarcodeData_ isEqualToString:textData_.text]){
            changeBarcodeData_ = YES;
        }
        if(changeBarcodeData_){
            //not update
            return ;
        }
        
        //update barcode data
        [self setDefaultBarcodeData];
    }else if(obj == levelList_){
        [buttonLevel_ setTitle:[levelList_ getItem:position] forState:UIControlStateNormal];
    }
}

- (void)eventDidPushDone:(id)sender
{
    [textData_ resignFirstResponder];
}

- (IBAction)eventDidPush:(id)sender {
    switch(((UIView*)sender).tag){
        case 0:
            [self printBarcode];
            break;
        case 1:
            [typeList_ show];
            break;
        case 2:
            [levelList_ show];
            break;
        default:
            break;
    }
}

- (void)printBarcode
{
    if(textData_.text.length == 0){
        [ShowMsg showError:@"errmsg_notext"];
        return ;
    }
    
    //create builder
    EposBuilder *builder = [[[EposBuilder alloc] initWithPrinterModel:printername_ Lang:language_] autorelease];
    if(builder == nil){
        return ;
    }
    
    //add command
    int level = [self getBuilderLevelRange];
    if (EPOS_OC_PARAM_UNSPECIFIED == level) {
        level = [self getBuilderLevel];
    }
    int result = [builder addSymbol:[self getBuilderData]
                               Type:[self getBuilderType]
                              Level:level
                              Width:[self getBuilderWidth]
                             Height:[self getBuilderHeight]
                               Size:[self getBuilderMaxSize]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addBarcode"];
        return ;
    }
    
    //send builder data
    unsigned long status = 0;
    unsigned long battery = 0;
    result = [printer_ sendData:builder Timeout:SEND_TIMEOUT Status:&status Battery:&battery];
    [ShowMsg showStatus:result status:status battery:battery];
    
    //remove builder
    [builder clearCommandBuffer];
}

- (NSString*)getBuilderData
{
    return textData_.text;
}

- (int)getBuilderType
{
    int typelist[] = {
        EPOS_OC_SYMBOL_PDF417_STANDARD,
        EPOS_OC_SYMBOL_PDF417_TRUNCATED,
        EPOS_OC_SYMBOL_QRCODE_MODEL_1,
        EPOS_OC_SYMBOL_QRCODE_MODEL_2,
        EPOS_OC_SYMBOL_MAXICODE_MODE_2,
        EPOS_OC_SYMBOL_MAXICODE_MODE_3,
        EPOS_OC_SYMBOL_MAXICODE_MODE_4,
        EPOS_OC_SYMBOL_MAXICODE_MODE_5,
        EPOS_OC_SYMBOL_MAXICODE_MODE_6,
        EPOS_OC_SYMBOL_GS1_DATABAR_STACKED,
        EPOS_OC_SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL,
        EPOS_OC_SYMBOL_GS1_DATABAR_EXPANDED_STACKED,
        EPOS_OC_SYMBOL_AZTECCODE_FULLRANGE,
        EPOS_OC_SYMBOL_AZTECCODE_COMPACT,
        EPOS_OC_SYMBOL_DATAMATRIX_SQUARE,
        EPOS_OC_SYMBOL_DATAMATRIX_RECTANGLE_8,
        EPOS_OC_SYMBOL_DATAMATRIX_RECTANGLE_12,
        EPOS_OC_SYMBOL_DATAMATRIX_RECTANGLE_16,
    };

    return typelist[typeList_.selectIndex];
}

- (int)getBuilderLevel
{
    int *levellist = NULL;
    int pdf417levellist[] = {
        EPOS_OC_LEVEL_0,
        EPOS_OC_LEVEL_1,
        EPOS_OC_LEVEL_2,
        EPOS_OC_LEVEL_3,
        EPOS_OC_LEVEL_4,
        EPOS_OC_LEVEL_5,
        EPOS_OC_LEVEL_6,
        EPOS_OC_LEVEL_7,
        EPOS_OC_LEVEL_8,
    };
    int qrlevellist[] = {
        EPOS_OC_LEVEL_L,
        EPOS_OC_LEVEL_M,
        EPOS_OC_LEVEL_Q,
        EPOS_OC_LEVEL_H,
    };
    switch(typeList_.selectIndex){
        case SYMBOL_PDF417_STANDARD_INDEX:
        case SYMBOL_PDF417_TRUNCATED_INDEX:
            levellist = pdf417levellist;
            break;
        case SYMBOL_QRCODE_MODEL_1_INDEX:
        case SYMBOL_QRCODE_MODEL_2_INDEX:
            levellist = qrlevellist;
            break;
        default:
            break;
    }
    if(levellist == NULL){
        return EPOS_OC_LEVEL_DEFAULT;
    }else{
        return levellist[levelList_.selectIndex];
    }
}

- (long)getBuilderWidth
{
    if(textModuleSizeW_.enabled){
        return [textModuleSizeW_.text intValue];
    }else{
        return EPOS_OC_PARAM_UNSPECIFIED;
    }
}

- (long)getBuilderHeight
{
    if(textModuleSizeH_.enabled){
        return [textModuleSizeH_.text intValue];
    }else{
        return EPOS_OC_PARAM_UNSPECIFIED;
    }
}

- (long)getBuilderMaxSize
{
    if(textMaxSize_.enabled){
        return [textMaxSize_.text intValue];
    }else{
        return EPOS_OC_PARAM_UNSPECIFIED;
    }
}

- (int)getBuilderLevelRange
{
    if (textLevelRange_.enabled) {
        return [textLevelRange_.text intValue];
    } else {
        return EPOS_OC_PARAM_UNSPECIFIED;
    }
}

- (void)setLevelList
{
    NSString **levellist = nil;
    NSString *pdf417levellist[] = {
        @"level_pdf417_0",
        @"level_pdf417_1",
        @"level_pdf417_2",
        @"level_pdf417_3",
        @"level_pdf417_4",
        @"level_pdf417_5",
        @"level_pdf417_6",
        @"level_pdf417_7",
        @"level_pdf417_8",
        nil,
    };
    NSString *qrlevellist[] = {
        @"level_qr_l",
        @"level_qr_m",
        @"level_qr_q",
        @"level_qr_h",
        nil,
    };
    switch(typeList_.selectIndex){
        case SYMBOL_PDF417_STANDARD_INDEX:
        case SYMBOL_PDF417_TRUNCATED_INDEX:
            levellist = pdf417levellist;
            break;
        case SYMBOL_QRCODE_MODEL_1_INDEX:
        case SYMBOL_QRCODE_MODEL_2_INDEX:
            levellist = qrlevellist;
            break;
        default:
            break;
    }
    
    if(levellist == nil){
        //disable
        buttonLevel_.enabled = NO;
        [buttonLevel_ setTitle:NSLocalizedString(@"empty", @"") forState:UIControlStateDisabled];
        [buttonLevel_ setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
    }else{
        buttonLevel_.enabled = YES;
        [levelList_ release];

        levelList_ = [[PickerTableView alloc] init];
        NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];
        for(int i = 0; levellist[i] != nil; i++){
            [items addObject:NSLocalizedString(levellist[i], @"")];
        }
        [levelList_ setItemList:items];
        levelList_.delegate = self;
        
        [buttonLevel_ setTitle:[levelList_ getItem:levelList_.selectIndex] forState:UIControlStateNormal];
    }
}

- (void)setDefaultSetting
{
    switch(typeList_.selectIndex){
        case SYMBOL_PDF417_STANDARD_INDEX:
        case SYMBOL_PDF417_TRUNCATED_INDEX:
            textModuleSizeW_.text = @"3";
            textModuleSizeH_.text = @"3";
            textMaxSize_.text = @"0";
            textLevelRange_.text = @"";
            textModuleSizeW_.enabled = YES;
            textModuleSizeH_.enabled = YES;
            textMaxSize_.enabled = YES;
            textLevelRange_.enabled = NO;
            break;
        case SYMBOL_QRCODE_MODEL_1_INDEX:
        case SYMBOL_QRCODE_MODEL_2_INDEX:
            textModuleSizeW_.text = @"3";
            textModuleSizeH_.text = @"";
            textMaxSize_.text = @"";
            textLevelRange_.text = @"";
            textModuleSizeW_.enabled = YES;
            textModuleSizeH_.enabled = NO;
            textMaxSize_.enabled = NO;
            textLevelRange_.enabled = NO;
            break;
        case SYMBOL_MAXICODE_MODE_2:
        case SYMBOL_MAXICODE_MODE_3:
        case SYMBOL_MAXICODE_MODE_4:
        case SYMBOL_MAXICODE_MODE_5:
        case SYMBOL_MAXICODE_MODE_6:
            textModuleSizeW_.text = @"";
            textModuleSizeH_.text = @"";
            textMaxSize_.text = @"";
            textLevelRange_.text = @"";
            textModuleSizeW_.enabled = NO;
            textModuleSizeH_.enabled = NO;
            textMaxSize_.enabled = NO;
            textLevelRange_.enabled = NO;
            break;
        case SYMBOL_GS1_DATABAR_STACKED:
        case SYMBOL_GS1_DATABAR_STACKED_OMNIDIRECTIONAL:
        case SYMBOL_GS1_DATABAR_EXPANDED_STACKED:
            textModuleSizeW_.text = @"2";
            textModuleSizeH_.text = @"";
            textMaxSize_.text = @"114";
            textLevelRange_.text = @"";
            textModuleSizeW_.enabled = YES;
            textModuleSizeH_.enabled = NO;
            textMaxSize_.enabled = YES;
            textLevelRange_.enabled = NO;
            break;
        case SYMBOL_AZTECCODE_FULLRANGE:
        case SYMBOL_AZTECCODE_COMPACT:
            textModuleSizeW_.text = @"3";
            textModuleSizeH_.text = @"";
            textMaxSize_.text = @"";
            textLevelRange_.text = @"23";
            textModuleSizeW_.enabled = YES;
            textModuleSizeH_.enabled = NO;
            textMaxSize_.enabled = NO;
            textLevelRange_.enabled = YES;
            break;
        case SYMBOL_DATAMATRIX_SQUARE:
        case SYMBOL_DATAMATRIX_RECTANGLE_8:
        case SYMBOL_DATAMATRIX_RECTANGLE_12:
        case SYMBOL_DATAMATRIX_RECTANGLE_16:
            textModuleSizeW_.text = @"3";
            textModuleSizeH_.text = @"";
            textMaxSize_.text = @"";
            textLevelRange_.text = @"";
            textModuleSizeW_.enabled = YES;
            textModuleSizeH_.enabled = NO;
            textMaxSize_.enabled = NO;
            textLevelRange_.enabled = NO;
            break;
        default:
            break;
    }
}

- (void)setDefaultBarcodeData
{
    //update default data
    [defaultBarcodeData_ release];
    defaultBarcodeData_ = [[self getDefaultBarcodeData] retain];
    
    //update ui
    textData_.text = defaultBarcodeData_;
}

- (NSString*)getDefaultBarcodeData
{
    NSString *dataList[] = {
        @"code2data_pdf417s",
        @"code2data_pdf417t",
        @"code2data_qr1",
        @"code2data_qr2",
        @"code2data_maxcode2",
        @"code2data_maxcode3",
        @"code2data_maxcode4",
        @"code2data_maxcode5",
        @"code2data_maxcode6",
        @"code2data_gr1st",
        @"code2data_gr1so",
        @"code2data_gr1es",
        @"code2data_aztecfull",
        @"code2data_azteccompact",
        @"code2data_dmsquare",
        @"code2data_dmrect_8rows",
        @"code2data_dmrect_12rows",
        @"code2data_dmrect_16rows",
    };
    
    return NSLocalizedString(dataList[typeList_.selectIndex], @"");
}

@end
