#import "CutView.h"
#import "ePOS-Print.h"
#import "ShowMsg.h"

#define SEND_TIMEOUT  10 * 1000

@interface CutView ()
- (void)cut;
- (int)getBuilderType;
@end

@implementation CutView
@synthesize buttonType_;
@synthesize scrollView_;
@synthesize itemView_;

- (id)initWithPrinter:(EposPrint*)printer
          printername:(NSString*)printername
             language:(int)language
{
    self = [super init];
    if (self) {
        printer_ = [printer retain];
        printername_ = [printername retain];
        language_ = language;
    }
    return self;
}

- (void)dealloc {
    [buttonType_ release];
    [scrollView_ release];
    [itemView_ release];
    
    [printer_ release];
    [printername_ release];
    [typeList_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //init type list
    typeList_ = [[PickerTableView alloc] init];
    NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"cuttype_nofeed", @"")];
    [items addObject:NSLocalizedString(@"cuttype_feed", @"")];
    [typeList_ setItemList:items];
    typeList_.delegate = self;
    typeList_.selectIndex = 1;
    [buttonType_ setTitle:[items objectAtIndex:typeList_.selectIndex] forState:UIControlStateNormal];
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    [scrollView_ setContentSize:itemView_.frame.size];
}

- (void)viewDidUnload
{
    [self setButtonType_:nil];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [super viewDidUnload];
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj
{
    [buttonType_ setTitle:[typeList_ getItem:position] forState:UIControlStateNormal];
}

- (IBAction)eventDidPush:(id)sender
{
    switch(((UIView*)sender).tag){
        case 0:
            [self cut];
            break;
        case 1:
            [typeList_ show];
            break;
        default:
            break;
    }
}

- (void)cut
{
    //create builder
    EposBuilder *builder = [[[EposBuilder alloc] initWithPrinterModel:printername_ Lang:language_] autorelease];
    if(builder == nil){
        return ;
    }
    
    //add command
    int result = [builder addCut:[self getBuilderType]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addCut"];
        return ;
    }
    
    //send builder data
    unsigned long status = 0;
    unsigned long battery = 0;
    result = [printer_ sendData:builder Timeout:SEND_TIMEOUT Status:&status Battery:&battery];
    [ShowMsg showStatus:result status:status battery:battery];
    
    //remove builder
    [builder clearCommandBuffer];
}

- (int)getBuilderType
{
    switch(typeList_.selectIndex){
        case 1:
            return EPOS_OC_CUT_FEED;
        case 0:
        default:
            return EPOS_OC_CUT_NO_FEED;
    }
}

@end
