//
//  LogSettingsView.m
//  ePOSPrintSample
//
//  Created by Daisuke Muramatsu on 2013/05/20.
//
//

#import "LogSettingsView.h"
#import "ePOS-Print.h"
#import "ShowMsg.h"

@interface LogSettingsView ()
- (int)getEnabled;
- (NSString *)getIpAddress;
- (int)getPort;
- (int)getLogSize;
- (int)getLogLevel;
- (int)getPeriod;
@end

@implementation LogSettingsView
@synthesize scrollView_;
@synthesize itemView_;
@synthesize buttonEnabled_;
@synthesize textIPAddress_;
@synthesize textPort_;
@synthesize textLogSize_;
@synthesize buttonLogLevel_;
@synthesize switchSaveSettings_;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

-(void)dealloc
{
    [scrollView_ release];
    [itemView_ release];
    [buttonEnabled_ release];
    [textIPAddress_ release];
    [textPort_ release];
    [textLogSize_ release];
    [buttonLogLevel_ release];
    [switchSaveSettings_ release];
    
    [enabledList_ release];
    [logLevelList_ release];
    
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    
    //init enabled list.
    enabledList_ = [[PickerTableView alloc] init];
    NSMutableArray * items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"log_disable", @"")];
    [items addObject:NSLocalizedString(@"log_storage", @"")];
    [items addObject:NSLocalizedString(@"log_tcp", @"")];
    [buttonEnabled_ setTitle:[items objectAtIndex:0]
                    forState:UIControlStateNormal];
    [enabledList_ setItemList:items];
    enabledList_.delegate = self;
    
    //init log level list.
    logLevelList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"log_low", @"")];
    [buttonLogLevel_ setTitle:[items objectAtIndex:0]
                     forState:UIControlStateNormal];
    [logLevelList_ setItemList:items];
    logLevelList_.delegate = self;
    
    //default setting
    textIPAddress_.text = @"192.168.192.168";
    textPort_.text = @"49152";
    textLogSize_.text = @"1";
    switchSaveSettings_.on = NO;
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    scrollBaseView_ = scrollView_;
    defaultActiveViewItem_ = textIPAddress_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

-(void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setButtonEnabled_:nil];
    [self setTextIPAddress_:nil];
    [self setTextPort_:nil];
    [self setTextLogSize_:nil];
    [self setButtonLogLevel_:nil];
    [self setSwitchSaveSettings_:nil];
    [super viewDidUnload];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)eventDidPush:(id)sender
{
    switch(((UIView*)sender).tag){
        case 0:
        {
            int result = [EposLog setLogSettings:[self getPeriod]
                                         Enabled:[self getEnabled]
                                       IpAddress:[self getIpAddress]
                                            Port:[self getPort]
                                         LogSize:[self getLogSize]
                                        LogLevel:[self getLogLevel]];
            if (result != EPOS_OC_SUCCESS) {
                [ShowMsg showExceptionEpos:result method:@"setLogSettings"];
                return;
            }
            break;
        }
        case 1:
            [enabledList_ show];
            break;
        case 2:
            [logLevelList_ show];
            break;
        default:
            break;
    }
}

- (int)getEnabled
{
    switch (enabledList_.selectIndex) {
        case 1:
            return EPOS_OC_LOG_STORAGE;
        case 2:
            return EPOS_OC_LOG_TCP;
        case 0:
        default:
            return EPOS_OC_LOG_DISABLE;
    }
}

- (NSString *)getIpAddress
{
    return textIPAddress_.text;
}

- (int)getPort
{
    return [textPort_.text intValue];
}

- (int)getLogSize
{
    return [textLogSize_.text intValue];
}

- (int)getLogLevel
{
    switch (logLevelList_.selectIndex) {
        case 0:
        default:
            return EPOS_OC_LOG_LOW;
    }
}

- (int)getPeriod
{
    if (switchSaveSettings_.on) {
        return EPOS_OC_LOG_PERMANENT;
    } else {
        return EPOS_OC_LOG_TEMPORARY;
    }
}

#pragma mark - SelectPickerTableDelegate method.
-(void)onSelectPickerItem:(NSInteger)position obj:(id)obj {
    if (obj == enabledList_) {
        [buttonEnabled_ setTitle:[enabledList_ getItem:position]
                        forState:UIControlStateNormal];
    } else if (obj == logLevelList_) {
        [buttonLogLevel_ setTitle:[logLevelList_ getItem:position]
                         forState:UIControlStateNormal];
    }
}

@end
