#import "MainView.h"
#import "DiscoverView.h"
#import "OpenView.h"
#import "TextView.h"
#import "ImageView.h"
#import "BarcodeView.h"
#import "Code2dView.h"
#import "PageModeView.h"
#import "CutView.h"
#import "GetNameView.h"
#import "ShowMsg.h"
#import "LogSettingsView.h"

@interface MainView ()
- (void)updateButtonState;
- (void)showPrinterStatus;
@end

@implementation MainView
@synthesize scrollView_;
@synthesize buttonOpen_;
@synthesize buttonClose_;
@synthesize buttonCloseDisconnect_;
@synthesize buttonText_;
@synthesize buttonImage_;
@synthesize buttonBarcode_;
@synthesize buttonCode2d_;
@synthesize buttonPageMode_;
@synthesize buttonCut_;
@synthesize buttonGetStatus_;
@synthesize buttonGetName_;
@synthesize buttonLogSettings_;
@synthesize itemView_;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        openDeviceName_ = @"192.168.192.168";
        connectionType_ = EPOS_OC_DEVTYPE_TCP;
        language_ = EPOS_OC_MODEL_ANK;
        printerName_ = @"TM-T88V";
    }
    
    return self;
}

- (void)dealloc {
    [scrollView_ release];
    [printer_ release];
    [openDeviceName_ release];
    [printerName_ release];
    
    [buttonOpen_ release];
    [buttonClose_ release];
    [buttonCloseDisconnect_ release];
    [buttonText_ release];
    [buttonImage_ release];
    [buttonBarcode_ release];
    [buttonCode2d_ release];
    [buttonPageMode_ release];
    [buttonCut_ release];
    [buttonGetStatus_ release];
    [buttonGetName_ release];
    [buttonLogSettings_ release];
    [itemView_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self updateButtonState];
    
    //auto scroll setting
    [scrollView_ setContentSize:itemView_.frame.size];
}

- (void)viewDidUnload
{
    [self setScrollView_:nil];
    [self setButtonOpen_:nil];
    [self setButtonClose_:nil];
    [self setButtonCloseDisconnect_:nil];
    [self setButtonText_:nil];
    [self setButtonImage_:nil];
    [self setButtonBarcode_:nil];
    [self setButtonCode2d_:nil];
    [self setButtonPageMode_:nil];
    [self setButtonCut_:nil];
    [self setButtonGetStatus_:nil];
    [self setButtonGetName_:nil];
    [self setButtonLogSettings_:nil];
    [self setItemView_:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    
}

//called DiscoverView
- (void)onSelectPrinter:(int)connectionType address:(NSString*)ipAddress
{
    [openDeviceName_ release];
    openDeviceName_ = [ipAddress retain];
    connectionType_ = connectionType;
}

//called OpenView
- (void)onOpenPrinter:(EposPrint*)prn
              connectionType:(int)connectionType
            ipaddress:(NSString*)ipaddress
          printername:(NSString*)printername
             language:(int)language
{
    if(printer_ != nil){
        [printer_ closePrinter];
        [printer_ release];
    }
    printer_ = [prn retain];
    [openDeviceName_ release];
    openDeviceName_ = [ipaddress retain];
    [printerName_ release];
    printerName_ = [printername retain];
    language_ = language;
    connectionType_ = connectionType;

    [self updateButtonState];
}

- (void)setEventCallback:(EposPrint *)prn
{
    if(prn != nil){
        [prn setStatusChangeEventCallback:@selector(onStatusChange:Status:) Target:self];
        [prn setBatteryStatusChangeEventCallback:@selector(onBatteryStatusChange:Battery:) Target:self];
    }
}

//click button
- (IBAction)eventButtonDidPush:(id)sender
{
    UIViewController *view = nil;
    switch(((UIView*)sender).tag){
        case 1:
            view = [[[DiscoverView alloc] initWithConnectionType:connectionType_] autorelease];
            ((DiscoverView*)view).delegate = self;
            break;
        case 2:
            view = [[[OpenView alloc] initWithConnectionType:connectionType_ address:openDeviceName_] autorelease];
            ((OpenView*)view).delegate = self;
            break;
        case 3:
            if(printer_ != nil){
                int result = [printer_ closePrinter];
                if(result != EPOS_OC_SUCCESS){
                    [ShowMsg showExceptionEpos:result method:@"closePrinter"];
                    return;
                }
                [printer_ release];
                printer_ = nil;
            }
            [self updateButtonState];
            break;
        case 4:
            view = [[[TextView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 5:
            view = [[[ImageView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 6:
            view = [[[BarcodeView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 7:
            view = [[[Code2dView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 8:
            view = [[[PageModeView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 9:
            view = [[[CutView alloc] initWithPrinter:printer_ printername:printerName_ language:language_] autorelease];
            break;
        case 10:
            [self showPrinterStatus];
            break;
        case 11:
            view = [[[GetNameView alloc] initWithConnectionType:connectionType_ address:openDeviceName_] autorelease];
            break;
        case 12:
            view = [[[LogSettingsView alloc] init] autorelease];
            break;
        case 13:
            if(printer_ != nil){
                int result = [printer_ closePrinter];
                if(result != EPOS_OC_SUCCESS){
                    [ShowMsg showExceptionEpos:result method:@"closePrinter"];
                    return;
                }
                [printer_ release];
                printer_ = nil;
            }
            EposBluetoothConnection *btConnection = [[EposBluetoothConnection alloc] init];
            int result = [btConnection disconnectDevice:openDeviceName_];
            if(result != EPOS_BT_SUCCESS){
                [ShowMsg showExceptionEposBt:result method:@"disconnectDevice"];
            }
            [btConnection release];
            [self updateButtonState];
            break;

        default:
            break;
    }
    if(view != nil){
        [self.navigationController pushViewController:view animated:YES];
    }
}

//chage button state(Enable/Disable)
- (void)updateButtonState
{
    UIButton *target1[] = {
        buttonOpen_,
        buttonGetName_,
        nil,
    };
    UIButton *target2[] = {
        buttonClose_,
        buttonText_,
        buttonImage_,
        buttonBarcode_,
        buttonCode2d_,
        buttonPageMode_,
        buttonCut_,
        buttonGetStatus_,
        nil,
    };
    UIButton *target3[] = {
        buttonClose_,
        buttonCloseDisconnect_,
        buttonText_,
        buttonImage_,
        buttonBarcode_,
        buttonCode2d_,
        buttonPageMode_,
        buttonCut_,
        buttonGetStatus_,
        nil,
    };
    UIButton **enableTarget = nil;
    UIButton **disableTarget = nil;
    if(printer_ == nil){
        enableTarget = target1;
        disableTarget = target3;
    }else{
        switch (connectionType_) {
            case EPOS_OC_DEVTYPE_TCP:
                enableTarget = target2;
                disableTarget = target1;
                break;
            default:
                enableTarget = target3;
                disableTarget = target1;
                break;
        }
    }
    
    for(int i = 0; enableTarget[i] != nil; i++){
        enableTarget[i].enabled = YES;
    }
    for(int i = 0; disableTarget[i] != nil; i++){
        disableTarget[i].enabled = NO;
        [disableTarget[i] setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
    }
}

- (void)showPrinterStatus
{
    // Get printer status and battery status
    int errStatus = EPOS_OC_SUCCESS;
    unsigned long status = 0;
    unsigned long battery = 0;
    
    errStatus = [printer_ getStatus:&status Battery:&battery];
    if (errStatus == EPOS_OC_SUCCESS) {
        [ShowMsg showStatus:errStatus status:status battery:battery];
    }
    else {
        [ShowMsg showExceptionEpos:errStatus method:@"getStatus"];
    }
}

- (void)onStatusChange:(NSString *)deviceName Status:(NSNumber *)status
{
    [ShowMsg showStatusChangeEvent:deviceName Status:[status unsignedLongValue]];
}

- (void)onBatteryStatusChange:(NSString *)deviceName Battery:(NSNumber *)battery
{
    [ShowMsg showBatteryStatusChangeEvent:deviceName Battery:[battery unsignedLongValue]];
}

@end
