#import "OpenView.h"
#import "PickerTableView.h"
#import "ShowMsg.h"
#import "ePOS-Print.h"

@interface OpenView ()
- (void)openPrinter;
@end

@implementation OpenView
@synthesize delegate = delegate_;
@synthesize scrollView_;
@synthesize itemView_;
@synthesize deviceType_;
@synthesize textIp_;
@synthesize buttonLangage_;
@synthesize buttonPrinter_;
@synthesize switchStatusMonitor_;
@synthesize textInterval_;

- (id)initWithConnectionType:(int)connectionType address:(NSString*)address {
    self = [super init];
    if (self) {
        initIpAddress_ = [address retain];
        initConnectionType_ = connectionType;
    }
    return self;
    
}

- (void)dealloc {
    [textIp_ release];
    [buttonLangage_ release];
    [buttonPrinter_ release];
    [printerList_ release];
    [languageList_ release];
    [initIpAddress_ release];
    [scrollView_ release];
    [itemView_ release];
    [deviceType_ release];
    [switchStatusMonitor_ release];
    [textInterval_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //init printer list
    printerList_ = [[PickerTableView alloc] init];
    NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];
	[items addObject:NSLocalizedString(@"printername_m10", @"")];
	[items addObject:NSLocalizedString(@"printername_m30", @"")];
    [items addObject:NSLocalizedString(@"printername_p20", @"")];
    [items addObject:NSLocalizedString(@"printername_p60", @"")];
    [items addObject:NSLocalizedString(@"printername_p60ii", @"")];
    [items addObject:NSLocalizedString(@"printername_p80", @"")];
    [items addObject:NSLocalizedString(@"printername_t20", @"")];
    [items addObject:NSLocalizedString(@"printername_t20ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t70", @"")];
    [items addObject:NSLocalizedString(@"printername_t70ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t81ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t82", @"")];
    [items addObject:NSLocalizedString(@"printername_t82ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t88v", @"")];
    [items addObject:NSLocalizedString(@"printername_t90ii", @"")];
    [items addObject:NSLocalizedString(@"printername_u220", @"")];
    [items addObject:NSLocalizedString(@"printername_u330", @"")];
    [buttonPrinter_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [printerList_ setItemList:items];
    printerList_.delegate = self;
    
    //init device type
    switch(initConnectionType_){
        case EPOS_OC_DEVTYPE_TCP:
            deviceType_.selectedSegmentIndex = 0;
            break;
        default:
            deviceType_.selectedSegmentIndex = 1;
            break;
    }

    //init language list
    languageList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"model_ank", @"")];
    [items addObject:NSLocalizedString(@"model_japanese", @"")];
    [items addObject:NSLocalizedString(@"model_chinese", @"")];
    [items addObject:NSLocalizedString(@"model_taiwan", @"")];
    [items addObject:NSLocalizedString(@"model_korean", @"")];
    [items addObject:NSLocalizedString(@"model_thai", @"")];
    [items addObject:NSLocalizedString(@"model_southasia", @"")];
    [buttonLangage_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [languageList_ setItemList:items];
    languageList_.delegate = self;
    
    //init edit
    textIp_.text = initIpAddress_;
    textInterval_.text = @"1000";
    
    //init switch
    switchStatusMonitor_.on = NO;
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    scrollBaseView_ = scrollView_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setTextIp_:nil];
    [self setButtonLangage_:nil];
    [self setButtonPrinter_:nil];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setDeviceType_:nil];
    [self setSwitchStatusMonitor_:nil];
    [self setTextInterval_:nil];
    [super viewDidUnload];
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj
{
    if(obj == printerList_){
        [buttonPrinter_ setTitle:[printerList_ getItem:position] forState:UIControlStateNormal];
    }else if(obj == languageList_){
        [buttonLangage_ setTitle:[languageList_ getItem:position] forState:UIControlStateNormal];
    }
}

- (IBAction)eventDidPush:(id)sender
{
    switch(((UIView*)sender).tag){
        case 0:
            [self openPrinter];
            break;
        case 1:
            [printerList_ show];
            break;
        case 2:
            [languageList_ show];
            break;
        default:
            break;
    }
}

- (int)getStatusMonitorEnabled
{
    if(switchStatusMonitor_.on){
        return EPOS_OC_TRUE;
    }else{
        return EPOS_OC_FALSE;
    }
}

- (long)getInterval
{
    return [textInterval_.text intValue];
}

- (void)openPrinter
{
    //get open parameter
    if(textIp_.text.length == 0){
        [ShowMsg showError:@"errmsg_noaddress"];
        return ;
    }
    
    //open device type
    int connectionType;
    switch(deviceType_.selectedSegmentIndex){
        case 0:
            connectionType = EPOS_OC_DEVTYPE_TCP;
            break;
        default:
            connectionType = EPOS_OC_DEVTYPE_BLUETOOTH;
            break;
    }      
    
    //open
    EposPrint *printer = [[EposPrint alloc] init];
    if(printer == nil){
        return ;
    }
    
    [delegate_ setEventCallback:printer];
    
    int result = [printer openPrinter:connectionType DeviceName:textIp_.text Enabled:[self getStatusMonitorEnabled] Interval:[self getInterval]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"openPrinter"];
        [printer release];
        return;
    }

    //return settings
    if(delegate_ != nil){
        int language = 0;
        switch(languageList_.selectIndex){
            case 1:
                language = EPOS_OC_MODEL_JAPANESE;
                break;
            case 2:
                language = EPOS_OC_MODEL_CHINESE;
                break;
            case 3:
                language = EPOS_OC_MODEL_TAIWAN;
                break;
            case 4:
                language = EPOS_OC_MODEL_KOREAN;
                break;
            case 5:
                language = EPOS_OC_MODEL_THAI;
                break;
            case 6:
                language = EPOS_OC_MODEL_SOUTHASIA;
                break;
            case 0:
            default:
                language = EPOS_OC_MODEL_ANK;
                break;
        }
        int connectionType;
        switch(deviceType_.selectedSegmentIndex){
            case 0:
                connectionType = EPOS_OC_DEVTYPE_TCP;
                break;
            default:
                connectionType = EPOS_OC_DEVTYPE_BLUETOOTH;
                break;
        }
        [delegate_ onOpenPrinter:printer
                  connectionType:connectionType
                       ipaddress:textIp_.text
                     printername:[printerList_ getItem:printerList_.selectIndex]
                        language:language];
    }
    [printer release];
    
    //return main activity
    [self.navigationController popViewControllerAnimated:YES];
}

@end
