#import "PageModeView.h"
#import <QuartzCore/QuartzCore.h>
#import "ePOS-Print.h"
#import "ShowMsg.h"

#define SEND_TIMEOUT     10 * 1000

@interface PageModeView ()
- (void)printText;
- (long)getBuilderAreaX;
- (long)getBuilderAreaY;
- (long)getBuilderAreaWidth;
- (long)getBuilderAreaHeight;
- (NSString *)getBuilderText;
@end

@implementation PageModeView
@synthesize buttonPrint_;
@synthesize textData_;
@synthesize textAreaX_;
@synthesize textAreaY_;
@synthesize textAreaWidth_;
@synthesize textAreaHeight_;
@synthesize scrollView_;
@synthesize itemView_;

- (id)initWithPrinter:(EposPrint*)printer
          printername:(NSString*)printername
             language:(int)language
{
    self = [super init];
    if (self) {
        printer_ = [printer retain];
        printername_ = [printername retain];
        language_ = language;
    }
    return self;
}

- (void)dealloc {
    [buttonPrint_ release];
    [scrollView_ release];
    [itemView_ release];
    [textData_ release];
    [textAreaX_ release];
    [textAreaY_ release];
    [textAreaWidth_ release];
    [textAreaHeight_ release];

    [printer_ release];
    [printername_ release];
    [super dealloc];
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //set multiline edit style
    textData_.layer.borderWidth = 1;
    textData_.layer.borderColor = [[UIColor grayColor] CGColor];
    textData_.layer.cornerRadius = 8;
    
    //set multiline edit keyboard accessory
    UIToolbar *doneToolbar = [[[UIToolbar alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 44)] autorelease];
    doneToolbar.barStyle = UIBarStyleBlackTranslucent;
    [doneToolbar sizeToFit];
    UIBarButtonItem *space = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:self action:nil] autorelease];
    UIBarButtonItem *doneButton = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(eventDidPushDone:)] autorelease];
    
    NSMutableArray *items = [NSMutableArray arrayWithObjects:space, doneButton, nil];
    [doneToolbar setItems:items animated:YES];
    textData_.inputAccessoryView = doneToolbar;
    
    //default setting
    textData_.text = NSLocalizedString(@"pagemode_edit_text", @"");
    textAreaX_.text = @"100";
    textAreaY_.text = @"100";
    textAreaWidth_.text = @"300";
    textAreaHeight_.text = @"300";
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    scrollBaseView_ = scrollView_;
    defaultActiveViewItem_ = textData_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setButtonPrint_:nil];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setTextData_:nil];
    [self setTextAreaX_:nil];
    [self setTextAreaY_:nil];
    [self setTextAreaWidth_:nil];
    [self setTextAreaHeight_:nil];
    [super viewDidUnload];
}

- (void)eventDidPushDone:(id)sender
{
    [textData_ resignFirstResponder];
}

- (IBAction)eventDidPush:(id)sender {
    [self printText];
}

- (void)printText
{
    if(textData_.text.length == 0){
        [ShowMsg showError:@"errmsg_notext"];
        return ;
    }
    
    //create builder
    EposBuilder *builder = [[[EposBuilder alloc] initWithPrinterModel:printername_ Lang:language_] autorelease];
    if(builder == nil){
        return ;
    }
    
    //add command
    int result = [builder addPageBegin];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addPageBegin"];
        return ;
    }
    
    result = [builder addPageArea:[self getBuilderAreaX]
                                Y:[self getBuilderAreaY]
                            Width:[self getBuilderAreaWidth]
                           Height:[self getBuilderAreaHeight]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addPageArea"];
        return ;
    }
    
    result = [builder addText:[self getBuilderText]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addText"];
        return ;
    }
    
    result = [builder addPageEnd];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addPageEnd"];
        return ;
    }
    
    //send builder data
    unsigned long status = 0;
    unsigned long battery = 0;
    result = [printer_ sendData:builder Timeout:SEND_TIMEOUT Status:&status Battery:&battery];
    [ShowMsg showStatus:result status:status battery:battery];
    
    //remove builder
    [builder clearCommandBuffer];
}

- (long)getBuilderAreaX
{
    return [textAreaX_.text intValue];
}

- (long)getBuilderAreaY
{
    return [textAreaY_.text intValue];
}

- (long)getBuilderAreaWidth
{
    return [textAreaWidth_.text intValue];
}

- (long)getBuilderAreaHeight
{
    return [textAreaHeight_.text intValue];
}

- (NSString *)getBuilderText
{
    return textData_.text;
}


@end
