﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.ApplicationModel.Resources;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.Graphics.Imaging;

using LibEposPrint;

namespace ePOSReceiptPrintSample_CS
{
    public sealed partial class MainPage : Page
    {
        private ResourceLoader loader = null;
        
        private static DevType deviceType = DevType.DEVTYPE_TCP;
        private static string deviceName = "192.168.192.168";

        // For PrinterDiscoveryPage
        public static void SetSelectedPrinterDiscovery(DevType selectDevicetype, string selectDeviceName)
        {
            deviceType = selectDevicetype;
            deviceName = selectDeviceName;
        }

        // For PrinterDiscoveryPage
        public static DevType GetDeviceType()
        {
            return deviceType;
        }

        public MainPage()
        {
            this.InitializeComponent();

            loader = new ResourceLoader();
            
            // Init printer name combobox
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_m10"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_m30"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p20"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p60"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p60ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_p80"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t20"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t20ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t70"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t70ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t81ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t82"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t82ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t88v"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_t90ii"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_u220"));
            this.PrinterName_ComboBox.Items.Add(loader.GetString("printername_u330"));

            // Init printer model combobox
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_ank"), ModelLang.MODEL_LANG_ANK));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_japanese"), ModelLang.MODEL_LANG_JAPANESE));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_chinese"), ModelLang.MODEL_LANG_CHINESE));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_taiwan"), ModelLang.MODEL_LANG_TAIWAN));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_korean"), ModelLang.MODEL_LANG_KOREAN));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_thai"), ModelLang.MODEL_LANG_THAI));
            this.PrinterModel_ComboBox.Items.Add(new ComboBoxModelsItem(loader.GetString("model_southasia"), ModelLang.MODEL_LANG_SOUTHASIA));

            // Default setting
            this.PrinterName_ComboBox.SelectedIndex = 0;
            this.PrinterModel_ComboBox.SelectedIndex = 0;
        }

        private async Task RunPrintSequenceAsync()
        {
            Result result = new Result();
            Builder builder = null;

            // Run print sequence of sample receipt
            this.Warnings_TextBox.Text = "";

            builder = await CreateReceiptPrintDataAsync(result);

            if (result.ErrType == Result.ErrorType.RESULT_ERR_NONE)
            {
                await PrintAsync(builder, result);
            }

            await DisplayMsg(result);

            // Clear command buffer
            if (builder != null)
            {
                builder.ClearCommandBuffer();
                builder = null;
            }
            
            result = null;

            return;
        }

        private async Task<Builder> CreateReceiptPrintDataAsync(Result result)
        {
            Builder builder = null;

            // ComboBox Item null check
            if ((this.PrinterName_ComboBox.SelectedItem == null) ||
               (this.PrinterModel_ComboBox.SelectedItem == null))
            {
                return null;
            }

            // Get printer name and model
            string printerName = (String)this.PrinterName_ComboBox.SelectedItem;
            ModelLang lang = ((ComboBoxModelsItem)PrinterModel_ComboBox.SelectedItem).GetModelLang();

            // Argument null check
            if (result == null)
            {
                return null;
            }

            // Init result
            result.Status = new PrinterStatus();
            result.ErrType = Result.ErrorType.RESULT_ERR_NONE;
            result.ErrHResult = 0;

            // Top logo data
            StorageFile logoFile = await StorageFile.GetFileFromApplicationUriAsync(new Uri("ms-appx:///Assets/store.png"));
            if (logoFile == null)
            {
                return null;
            }

            IRandomAccessStreamWithContentType logoStream = await logoFile.OpenReadAsync();
            BitmapDecoder logoData = await BitmapDecoder.CreateAsync(logoStream);
            logoStream.Dispose();
            logoStream = null;
            logoFile = null;

            // Text buffer
            StringBuilder textData = new StringBuilder();
            try
            {
                // addBarcode API settings
                const int barcodeModuleWidth = 2;
                const int barcodeHeight = 100;

                // Create builder
                builder = new Builder(printerName, lang);

                // Set alignment to center
                builder.AddTextAlign(Align.ALIGN_CENTER);

                // Add top logo to command buffer
                await builder.AddImageAsync(
                    logoData, 0, 0,
                    (int)logoData.PixelWidth,
                    (int)logoData.PixelHeight,
                    Color.COLOR_1,
                    Mode.MODE_MONO,
                    Halftone.HALFTONE_DITHER,
                    Builder.PARAM_DEFAULT,
                    GetCompress(deviceType)
                    );

                logoData = null;
                
                // Add receipt text to command buffer
                // Section 1 : Store information
                builder.AddFeedLine(1);

                textData.Append("THE STORE 1234 (555) 555-5555\n");
                textData.Append("STORE DIRECTOR - John Smith\n");
                textData.Append("\n");
                textData.Append("7/01/07 16:58 6153 05 0191 134\n");
                textData.Append("ST# 21 OP# 001 TE# 01 TR# 747\n");
                textData.Append("------------------------------\n");
                builder.AddText(textData.ToString());
                textData.Clear();

                // Section 2 : Purchased Items
                textData.Append("004 OHEIDA 3PK SPRINGF  9.99 R\n");
                textData.Append("104 3 CUP BLK TEAPOT    9.99 R\n");
                textData.Append("451 EMERIL GRIDDLE/PAN 17.99 R\n");
                textData.Append("389 CANDYMAKER ASSORT   4.99 R\n");
                textData.Append("740 TRIPOD              8.99 R\n");
                textData.Append("334 BLK LOGO PRNTED ZO  7.99 R\n");
                textData.Append("581 AQUA MICROTERRY SC  6.99 R\n");
                textData.Append("934 30L BLK FF DRESS   16.99 R\n");
                textData.Append("075 LEVITATING DESKTOP  7.99 R\n");
                textData.Append("412 **Blue Overprint P  2.99 R\n");
                textData.Append("762 REPOSE 4PCPM CHOC   5.49 R\n");
                textData.Append("613 WESTGATE BLACK 25  59.99 R\n");
                textData.Append("------------------------------\n");
                builder.AddText(textData.ToString());
                textData.Clear();

                // Section 3 : Payment information
                textData.Append("SUBTOTAL                160.38\n");
                textData.Append("TAX                      14.43\n");
                builder.AddText(textData.ToString());
                textData.Clear();

                builder.AddTextDouble(DoubleSize.DOUBLE_TRUE, DoubleSize.DOUBLE_TRUE);
                builder.AddText("TOTAL    174.81\n");
                builder.AddTextDouble(DoubleSize.DOUBLE_FALSE, DoubleSize.DOUBLE_FALSE);
                builder.AddFeedLine(1);

                textData.Append("CASH                    200.00\n");
                textData.Append("CHANGE                   25.19\n");
                textData.Append("------------------------------\n");
                builder.AddText(textData.ToString());
                textData.Clear();

                // Section 4 : Advertisement
                textData.Append("TotalNumber of Items Purchased\n");
                textData.Append("Sign Up and Save !\n");
                textData.Append("With Preferred Saving Card\n");
                builder.AddText(textData.ToString());
                textData.Clear();

                textData = null;

                builder.AddFeedLine(2);

                // Add barcode data to command buffer
                builder.AddBarcode("01209457",
                                   BarcodeType.BARCODE_CODE39,
                                   Hri.HRI_BELOW,
                                   Font.FONT_A,
                                   barcodeModuleWidth,
                                   barcodeHeight);

                // Add cutting command to command buffer
                builder.AddCut(Cut.CUT_FEED);

            }
            catch (Exception ex)
            {
                result.SetErrInfo(Result.ErrorType.RESULT_ERR_EPOSPRINT_OTHER, ex.HResult);

                if (builder != null)
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
            }

            return builder;
        }

        private async Task PrintAsync(Builder builder, Result result)
        {
            bool isBeginTransaction = false;
            // sendData API timeout setting (10000(msec))
            const int sendTimeout = 10000;
            Print printer = null;

            // Null check
            if ((builder == null) || (result == null))
            {
                return;
            }

            //init result
            result.Status = new PrinterStatus();
            result.ErrType = Result.ErrorType.RESULT_ERR_NONE;
            result.ErrHResult = 0;

            printer = new Print();

            try
            {
                // Open
               await printer.OpenPrinterAsync(
                    deviceType,
                    deviceName,
                    Monitoring.MONITORING_FALSE,
                    Print.PARAM_DEFAULT,
                    Print.PARAM_DEFAULT);
            }
            catch (Exception ex)
            {
                result.SetErrInfo(Result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN, ex.HResult);
                printer = null;
                return;
            }

            try
            {
                // Print data if printer is printable
                result.Status = await printer.GetStatusAsync();

                if (IsPrintable(result))
                {
                    result.Status = new PrinterStatus();

                    await printer.BeginTransactionAsync();
                    isBeginTransaction = true;

                    result.Status = await printer.SendDataAsync(builder, sendTimeout);
                }
                else
                {
                    result.SetErrInfo(Result.ErrorType.RESULT_ERR_PRINTER_STATUS, EposHResult.HR_S_OK);
                }
            }
            catch (Exception ex)
            {
                result.SetErrInfo(Result.ErrorType.RESULT_ERR_EPOSPRINT_SEND, ex.HResult);
                result.Status = printer.GetPrinterStatus(ex.HResult);
            }

            if (isBeginTransaction)
            {
                try
                {
                    await printer.EndTransactionAsync();
                }
                catch (Exception ex)
                {
                    // Do nothing
                }
            }

            try
            {
                await printer.ClosePrinterAsync();
                printer = null;
            }
            catch (Exception ex)
            {
                // Do nothing
            }

            return;
        }

        private async Task DisplayMsg(Result result)
        {
            if (result == null)
            {
                return;
            }

            String errorMsg = MsgMaker.MakeErrorMessage(result);
            String warningMsg = MsgMaker.MakeWarningMessage(result);

            if (!errorMsg.Equals(String.Empty))
            {
                await ShowMsg.Show(errorMsg);
            }
            if (!warningMsg.Equals(String.Empty))
            {
                this.Warnings_TextBox.Text = warningMsg;
            }

            errorMsg = null;
            warningMsg = null;

            return;
        }

        private bool IsPrintable(Result result)
        {
            if (result == null)
            {
                return false;
            }

            if ((result.Status.printerStatus & Print.ST_OFF_LINE) == Print.ST_OFF_LINE)
            {
                return false;
            }

            if ((result.Status.printerStatus & Print.ST_NO_RESPONSE) == Print.ST_NO_RESPONSE)
            {
                return false;
            }

            return true;
        }

        private Compress GetCompress(DevType connectionType)
        {
            if (connectionType == DevType.DEVTYPE_BLUETOOTH)
            {
                return Compress.COMPRESS_DEFLATE;
            }
            else
            {
                return Compress.COMPRESS_NONE;
            }
        }

        private void PrinterDiscovery_Button_Click(object sender, RoutedEventArgs e)
        {
            // Printer Discovery Page
            this.Frame.Navigate(typeof(PrinterDiscoveryPage), null);
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            // run print sequence
            this.IsEnabled = false;

            await this.RunPrintSequenceAsync();

            this.IsEnabled = true;
        }

    }
}
