﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;


namespace ePOSReceiptPrintSample_CS
{
    class MsgMaker
    {
        private const int BATTERY_NEAR_END = 0x3131;
        private const int BATTERY_REAL_END = 0x3130;
        private static ResourceLoader loader = new ResourceLoader();

        public static String MakeErrorMessage(Result result)
        {
            String msg = "";
            if (result == null)
            {
                return msg;
            }

            switch (result.ErrType)
            {
                case Result.ErrorType.RESULT_ERR_NONE:
                    msg = "";
                    break;

                case Result.ErrorType.RESULT_ERR_PRINTER_STATUS:
                    msg = MakePrinterStatusErrorHandlingText(result);
                    break;

                case Result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN:
                    msg = MakeEposPrintAPIErrorHandlingText(result);
                    break;

                case Result.ErrorType.RESULT_ERR_EPOSPRINT_SEND:
                    msg = MakeEposPrintAPIErrorHandlingText(result);
                    break;

                case Result.ErrorType.RESULT_ERR_EPOSPRINT_OTHER:
                    msg = MakeEposPrintAPIErrorHandlingText(result);
                    break;

                case Result.ErrorType.RESULT_ERR_EPSONIO:
                    msg = MakeEpsonIoAPIErrorHandlingText(result);
                    break;

                default:
                    // Should not be reached
                    msg = loader.GetString("handlingmsg_ex_application_error");
                    break;
            }

            return msg;
        }

        public static String MakeWarningMessage(Result result)
        {
            String msg = "";
            if (result == null)
            {
                return msg;
            }

            PrinterStatus statusResult = result.Status;
            int status = statusResult.printerStatus;
            int battery = statusResult.batteryStatus;

            if ((status & Print.ST_RECEIPT_NEAR_END) == Print.ST_RECEIPT_NEAR_END)
            {
                msg += loader.GetString("handlingmsg_warn_receipt_near_end");
            }
            if (battery == BATTERY_NEAR_END)
            {
                msg += loader.GetString("handlingmsg_warn_battery_near_end");
            }

            return msg;
        }

        private static String MakePrinterStatusErrorHandlingText(Result result)
        {
            String msg = "";
            if (result == null)
            {
                return msg;
            }

            PrinterStatus statusResult = result.Status;
            int status = statusResult.printerStatus;
            int battery = statusResult.batteryStatus;
        
            if ((status & Print.ST_NO_RESPONSE) == Print.ST_NO_RESPONSE)
            {
        	    msg += loader.GetString("handlingmsg_err_no_response");
            }

            if ((status & Print.ST_COVER_OPEN) == Print.ST_COVER_OPEN)
            {
        	    msg += loader.GetString("handlingmsg_err_cover_open");
            }

            if (((status & Print.ST_PAPER_FEED) == Print.ST_PAPER_FEED) || ((status & Print.ST_PANEL_SWITCH) == Print.ST_PANEL_SWITCH))
            {
     	   	    msg += loader.GetString("handlingmsg_err_paper_feed");
     	    }

            if (((status & Print.ST_AUTOCUTTER_ERR) == Print.ST_AUTOCUTTER_ERR) || ((status & Print.ST_MECHANICAL_ERR) == Print.ST_MECHANICAL_ERR))
            {
     	   	    msg += loader.GetString("handlingmsg_err_autocutter");
            }

            if ((status & Print.ST_UNRECOVER_ERR) == Print.ST_UNRECOVER_ERR)
            {
        	    msg += loader.GetString("handlingmsg_err_unrecover");
            }

            if ((status & Print.ST_RECEIPT_END) == Print.ST_RECEIPT_END)
            {
        	    msg += loader.GetString("handlingmsg_err_receipt_end");
            }

            if ((status & Print.ST_HEAD_OVERHEAT) == Print.ST_HEAD_OVERHEAT)
            {
        	    msg += loader.GetString("handlingmsg_err_overheat");
                msg += loader.GetString("handlingmsg_err_head");
            }

            if ((status & Print.ST_MOTOR_OVERHEAT) == Print.ST_MOTOR_OVERHEAT)
            {
        	    msg += loader.GetString("handlingmsg_err_overheat");
                msg += loader.GetString("handlingmsg_err_motor");
            }

            if ((status & Print.ST_BATTERY_OVERHEAT) == Print.ST_BATTERY_OVERHEAT)
            {
                msg += loader.GetString("handlingmsg_err_overheat");
                msg += loader.GetString("handlingmsg_err_battery");
            }

            if ((status & Print.ST_WRONG_PAPER) == Print.ST_WRONG_PAPER)
            {
        	    msg += loader.GetString("handlingmsg_err_wrong_paper");
            }

            if (battery == BATTERY_REAL_END)
            {
        	    msg += loader.GetString("handlingmsg_err_battery_real_end");
            }

            return msg;
        }

        private static String MakeEposPrintAPIErrorHandlingText(Result result)
        {
            String retMsg = "";
            String printerMsg = "";

            if (result == null)
            {
                return retMsg;
            }

            int status = result.Status.printerStatus;

            if (result.ErrHResult == EposHResult.HR_E_INVALIDARG)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == EposHResult.HR_E_ACCESSDENIED)
            {
                retMsg = MakeEposPrintAPIAccessDeniedHandlingText(result);
            }
            else if (result.ErrHResult == EposHResult.HR_E_ABORT)
            {
                retMsg = loader.GetString("handlingmsg_ex_timeout");
            }
            else if (result.ErrHResult == EposHResult.HR_E_OUTOFMEMORY)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == EposHResult.HR_E_PENDING)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == EposHResult.HR_E_FAIL)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else
            {
                // Should not be reached
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }

            retMsg += "\n";
            retMsg += loader.GetString("handlingmsg_notice_printer_problems");

            printerMsg = MakePrinterStatusErrorHandlingText(result);
            if (printerMsg.Equals(String.Empty))
            {
                printerMsg += loader.GetString("handlingmsg_notice_failed");
            }

            retMsg += printerMsg;

            return retMsg;
        }

        private static String MakeEpsonIoAPIErrorHandlingText(Result result)
        {
            String retMsg = "";

            if (result == null)
            {
                return retMsg;
            }

            if (result.ErrHResult == IoHResult.HR_E_INVALIDARG)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == IoHResult.HR_E_ACCESSDENIED)
            {
                retMsg = loader.GetString("handlingmsg_ex_connect");
            }
            else if (result.ErrHResult == IoHResult.HR_E_ABORT)
            {
                retMsg = loader.GetString("handlingmsg_ex_timeout");
            }
            else if (result.ErrHResult == IoHResult.HR_E_OUTOFMEMORY)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == IoHResult.HR_E_PENDING)
            {
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            else if (result.ErrHResult == IoHResult.HR_E_FAIL)
            {
                retMsg = loader.GetString("handlingmsg_ex_failure");
            }
            else
            {
                // Should not be reached
                retMsg = loader.GetString("handlingmsg_ex_application_error");
            }
            return retMsg;
        }

        private static String MakeEposPrintAPIAccessDeniedHandlingText(Result result)
        {
            String retMsg = "";

            if (result == null) {
                return retMsg;
            }

            int status = result.Status.printerStatus;

            switch (result.ErrType)
            {
                case Result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN:
                    retMsg = loader.GetString("handlingmsg_ex_open");
                    break;

                case Result.ErrorType.RESULT_ERR_EPOSPRINT_SEND:
                    if ((status & Print.ST_NO_RESPONSE) == Print.ST_NO_RESPONSE)
                    {
                        retMsg = loader.GetString("handlingmsg_ex_connect");
                    }
                    else if (((status & Print.ST_OFF_LINE) == Print.ST_OFF_LINE))
                    {
                        retMsg = loader.GetString("handlingmsg_ex_off_line");
                    }
                    else
                    {
                        // Should not be reached
                        retMsg = loader.GetString("handlingmsg_ex_application_error");
                    }
                    break;

                default:
                    // Should not be reached
                    retMsg = loader.GetString("handlingmsg_ex_application_error");
                    break;
            }

            return retMsg;
        }

    }
}
