﻿using ePOSReceiptPrintSample_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using Windows.UI.Core;
using System.Threading;
using System.Threading.Tasks;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOSReceiptPrintSample_CS
{
    public sealed partial class PrinterDiscoveryPage : Page
    {
        private ResourceLoader loader = null;
        private Timer updateListThread = null;

        private NavigationHelper navigationHelper;

        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        private async void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            await FindStart();
        }

        public PrinterDiscoveryPage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;

            loader = new ResourceLoader();

            DevType deviceType = MainPage.GetDeviceType();
            switch(deviceType)
            {
                case DevType.DEVTYPE_BLUETOOTH:
                    BT_RadioButton.IsChecked = true;
                    break;

                case DevType.DEVTYPE_TCP:
                    TCP_RadioButton.IsChecked = true;
                    break;

                default:
                    TCP_RadioButton.IsChecked = true;
                    break;
            }
        }

        

        private void Run(object sender)
        {
            UpdateList();
        }

        private async Task FindStart()
        {
            Result errResult = new Result();
            String displayMsg = "";
            String errorMsg = "";
            String warningMsg = "";

            // Stop running finder
            while (true)
            {
                try
                {
                    await Finder.StopAsync();
                    break;
                }
                catch (Exception ex)
                {
                    if (ex.HResult != EposHResult.HR_E_PENDING)
                    {
                        break;
                    }
                }
            }

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }

            PrinterDiscovery_ListView.Items.Clear();

            // Get device type and find
            try
            {
                if (BT_RadioButton.IsChecked == true)
                {
                    await Finder.StartAsync(IoDevType.BLUETOOTH, "");
                }
                else
                {
                    await Finder.StartAsync(IoDevType.TCP, "255.255.255.255");
                }
                
            }
            catch (Exception ex)
            {
                errResult.SetErrInfo(Result.ErrorType.RESULT_ERR_EPSONIO, ex.HResult);
                errorMsg = MsgMaker.MakeErrorMessage(errResult);
                warningMsg = MsgMaker.MakeWarningMessage(errResult);

                if (!errorMsg.Equals(String.Empty))
                {
                    displayMsg += loader.GetString("handlingmsg_notice_error");
                    displayMsg += errorMsg;
                }
                if (!warningMsg.Equals(String.Empty))
                {
                    displayMsg += loader.GetString("handlingmsg_notice_warning");
                    displayMsg += warningMsg;
                }
            }

            if (errResult.ErrType != Result.ErrorType.RESULT_ERR_NONE)
            {
                await ShowMsg.Show(displayMsg);
            }

            updateListThread = new Timer(Run, null, 0, 500);

            errResult = null;
            displayMsg = null;
            errorMsg = null;
            warningMsg = null;

            return;
        }

        private async void UpdateList()
        {
            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                DeviceInfo[] deviceList = null;

                try
                {
                    deviceList = Finder.GetDeviceInfoList(IoFilterOption.DEFAULT);

                    if (deviceList == null)
                    {
                        if (PrinterDiscovery_ListView.Items.Count() > 0)
                        {
                            PrinterDiscovery_ListView.Items.Clear();
                        }
                    }
                    else if (deviceList.Length != PrinterDiscovery_ListView.Items.Count())
                    {
                        PrinterDiscovery_ListView.Items.Clear();
                        for (int i = 0; i < deviceList.Length; i++)
                        {
                            PrinterDiscovery_ListView.Items.Add(new DisplayDeviceList(deviceList[i].PrinterName, deviceList[i].DeviceName));
                        }
                    }

                }
                catch (Exception)
                {
                    return;
                }

            });
        }

        private DevType GetSelectedDevicetype()
        {
            if (BT_RadioButton.IsChecked == true)
            {
                return DevType.DEVTYPE_BLUETOOTH;
            }
            else
            {
                return DevType.DEVTYPE_TCP;
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override async void OnNavigatedFrom(NavigationEventArgs e)
        {
            while (true)
            {
                try
                {
                    await Finder.StopAsync();
                    break;
                }
                catch (Exception ex)
                {
                    if (ex.HResult != EposHResult.HR_E_PENDING)
                    {
                        break;
                    }
                }
            }

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }
            navigationHelper.OnNavigatedFrom(e);
        }

        private void ListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            MainPage.SetSelectedPrinterDiscovery(GetSelectedDevicetype(), (String)((DisplayDeviceList)PrinterDiscovery_ListView.SelectedItem).Device);

            if (updateListThread != null)
            {
                updateListThread.Dispose();
            }
            if (this.Frame.CanGoBack)
            {
                this.Frame.GoBack();
            }
        }

        private async void RadioButton_Clicked(object sender, RoutedEventArgs e)
        {
            await FindStart();
        }
    }

    public class DisplayDeviceList
    {
        public String DeviceName { get; set; }
        public String Device { get; set; }

        public DisplayDeviceList(String devicename, String device)
        {
            DeviceName = devicename;
            Device = device;
        }
    }
}
