﻿Imports System.Threading.Tasks
Imports System.Text
Imports Windows.ApplicationModel.Resources
Imports Windows.Storage
Imports Windows.Storage.Streams
Imports Windows.Graphics.Imaging

Imports LibEposPrint


Public NotInheritable Class MainPage
    Inherits Page
    
    Private _loader As ResourceLoader = Nothing

    Private Shared _deviceType As DevType = DevType.DEVTYPE_TCP
    Private Shared _deviceName As String = "192.168.192.168"

    ' For PrinterDiscoveryPage
    Public Shared Sub SetSelectedPrinterDiscovery(selectDevicetype As DevType, selectDeviceName As String)

        _deviceType = selectDevicetype
        _deviceName = selectDeviceName

    End Sub

    ' For PrinterDiscoveryPage
    Public Shared Function GetDeviceType()

        Return _deviceType

    End Function

    Public Sub New()

        InitializeComponent()

        _loader = New ResourceLoader()

        ' Init printer name combobox
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_m10"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_m30"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p20"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p60"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p60ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_p80"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t20"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t20ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t70"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t70ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t81ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t82"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t82ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t88v"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_t90ii"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_u220"))
        Me.PrinterName_ComboBox.Items.Add(_loader.GetString("printername_u330"))

        ' Init printer model combobox
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_ank"), ModelLang.MODEL_LANG_ANK))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_japanese"), ModelLang.MODEL_LANG_JAPANESE))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_chinese"), ModelLang.MODEL_LANG_CHINESE))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_taiwan"), ModelLang.MODEL_LANG_TAIWAN))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_korean"), ModelLang.MODEL_LANG_KOREAN))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_thai"), ModelLang.MODEL_LANG_THAI))
        Me.PrinterModel_ComboBox.Items.Add(New ComboBoxModelsItem(_loader.GetString("model_southasia"), ModelLang.MODEL_LANG_SOUTHASIA))

        ' Default setting
        Me.PrinterName_ComboBox.SelectedIndex = 0
        Me.PrinterModel_ComboBox.SelectedIndex = 0

    End Sub

    Private Async Function RunPrintSequence() As Task

        Dim result As Result = New Result
        Dim builder As Builder = Nothing

        ' Run print squence of sample receipt
        Me.Warnings_TextBox.Text = ""

        builder = Await CreateReceiptPrintDataAsync(result)

        If result.ErrType = result.ErrorType.RESULT_ERR_NONE Then
            Await PrintAsync(builder, result)
        End If

        Await DisplayMsg(result)

        ' Clear command buffer
        If builder IsNot Nothing Then
            builder.ClearCommandBuffer()
        End If

        builder = Nothing
        result = Nothing

        Return

    End Function

    Private Async Function CreateReceiptPrintDataAsync(result As Result) As Task(Of Builder)

        ' Argument null check
        If result Is Nothing Then
            Return Nothing
        End If

        ' Combobox Item null check
        If Me.PrinterName_ComboBox.SelectedItem Is Nothing Then
            Return Nothing
        End If

        Dim builder As Builder = Nothing

        Dim printerName As String = ""
        Dim lang As ModelLang = ModelLang.MODEL_LANG_ANK

        ' Top logo data
        Dim logoFile As StorageFile = Await StorageFile.GetFileFromApplicationUriAsync(New Uri("ms-appx:///Assets/store.png"))

        If logoFile Is Nothing Then
            Return Nothing
        End If

        Dim logoStream As IRandomAccessStreamWithContentType = Await logoFile.OpenReadAsync()
        Dim logoData As BitmapDecoder = Await BitmapDecoder.CreateAsync(logoStream)
        logoStream.Dispose()

        logoStream = Nothing
        logoFile = Nothing

        ' Text buffer
        Dim textData As StringBuilder = New StringBuilder()

        ' AddBarcode API settings
        Const barcodeModuleWidth As Integer = 2
        Const barcodeHeight As Integer = 100

        If Me.PrinterModel_ComboBox.SelectedItem Is Nothing Then
            logoData = Nothing
            textData = Nothing

            Return Nothing
        End If

        ' Init result
        result.Status = New PrinterStatus()
        result.ErrType = result.ErrorType.RESULT_ERR_NONE
        result.ErrHResult = 0

        ' Get printer name and model
        printerName = CType(Me.PrinterName_ComboBox.SelectedItem(), String)
        lang = (CType(Me.PrinterModel_ComboBox.SelectedItem, ComboBoxModelsItem)).GetModelLang

        Try
            ' Create builder
            builder = New Builder(printerName, lang)

            ' Set alignment to center
            builder.AddTextAlign(Align.ALIGN_CENTER)

            ' Add top logo to command buffer
            Await builder.AddImageAsync(
                logoData, 0, 0,
                CType(logoData.PixelWidth, Integer),
                CType(logoData.PixelHeight, Integer),
                Color.COLOR_1,
                Mode.MODE_MONO,
                Halftone.HALFTONE_DITHER,
                builder.PARAM_DEFAULT,
                GetCompress(_deviceType)
                )

            logoData = Nothing

            ' Add receipt text to command buffer
            ' Section 1 : Store information
            builder.AddFeedLine(1)

            textData.Append("THE STORE 1234 (555) 555-5555" + vbCrLf)
            textData.Append("STORE DIRECTOR - John Smith" + vbCrLf)
            textData.Append(vbCrLf)
            textData.Append("7/01/07 16:58 6153 05 0191 134" + vbCrLf)
            textData.Append("ST# 21 OP# 001 TE# 01 TR# 747" + vbCrLf)
            textData.Append("------------------------------" + vbCrLf)
            builder.AddText(textData.ToString())
            textData.Clear()

            ' Section 2 : Purchased Items
            textData.Append("004 OHEIDA 3PK SPRINGF  9.99 R" + vbCrLf)
            textData.Append("104 3 CUP BLK TEAPOT    9.99 R" + vbCrLf)
            textData.Append("451 EMERIL GRIDDLE/PAN 17.99 R" + vbCrLf)
            textData.Append("389 CANDYMAKER ASSORT   4.99 R" + vbCrLf)
            textData.Append("740 TRIPOD              8.99 R" + vbCrLf)
            textData.Append("334 BLK LOGO PRNTED ZO  7.99 R" + vbCrLf)
            textData.Append("581 AQUA MICROTERRY SC  6.99 R" + vbCrLf)
            textData.Append("934 30L BLK FF DRESS   16.99 R" + vbCrLf)
            textData.Append("075 LEVITATING DESKTOP  7.99 R" + vbCrLf)
            textData.Append("412 **Blue Overprint P  2.99 R" + vbCrLf)
            textData.Append("762 REPOSE 4PCPM CHOC   5.49 R" + vbCrLf)
            textData.Append("613 WESTGATE BLACK 25  59.99 R" + vbCrLf)
            textData.Append("------------------------------" + vbCrLf)
            builder.AddText(textData.ToString())
            textData.Clear()

            ' Section 3 : Payment information
            textData.Append("SUBTOTAL                160.38" + vbCrLf)
            textData.Append("TAX                      14.43" + vbCrLf)
            builder.AddText(textData.ToString())
            textData.Clear()

            builder.AddTextDouble(DoubleSize.DOUBLE_TRUE, DoubleSize.DOUBLE_TRUE)
            builder.AddText("TOTAL    174.81" + vbCrLf)
            builder.AddTextDouble(DoubleSize.DOUBLE_FALSE, DoubleSize.DOUBLE_FALSE)
            builder.AddFeedLine(1)

            textData.Append("CASH                    200.00" + vbCrLf)
            textData.Append("CHANGE                   25.19" + vbCrLf)
            textData.Append("------------------------------" + vbCrLf)
            builder.AddText(textData.ToString())
            textData.Clear()

            ' Section 4 : Advertisement
            textData.Append("TotalNumber of Items Purchased" + vbCrLf)
            textData.Append("Sign Up and Save !" + vbCrLf)
            textData.Append("With Preferred Saving Card" + vbCrLf)
            builder.AddText(textData.ToString())
            textData.Clear()

            textData = Nothing

            builder.AddFeedLine(2)

            ' Add barcode data to command buffer
            builder.AddBarcode("01209457",
                                BarcodeType.BARCODE_CODE39,
                                Hri.HRI_BELOW,
                                Font.FONT_A,
                                barcodeModuleWidth,
                                barcodeHeight)

            ' Add cutting commandt to command buffer
            builder.AddCut(Cut.CUT_FEED)
        Catch ex As Exception
            result.SetErrInfo(result.ErrorType.RESULT_ERR_EPOSPRINT_OTHER, ex.HResult)

            If builder IsNot Nothing Then
                builder.ClearCommandBuffer()
                builder = Nothing
            End If

        End Try

        Return builder

    End Function

    Private Async Function PrintAsync(builder As Builder, result As Result) As Task

        ' Null check
        If builder Is Nothing Then
            Return
        End If

        If result Is Nothing Then
            Return
        End If

        Dim isBeginTransaction As Boolean = False
        Dim printer As Print = Nothing
        ' sendData API settings
        Const sendTimeout As Integer = 10000

        ' Init result
        result.Status = New PrinterStatus()
        result.ErrType = result.ErrorType.RESULT_ERR_NONE
        result.ErrHResult = 0

        printer = New Print()

        Try
            ' Open
            Await printer.OpenPrinterAsync(
                _deviceType,
                _deviceName,
                Monitoring.MONITORING_FALSE,
                Print.PARAM_DEFAULT,
                Print.PARAM_DEFAULT)
        Catch ex As Exception
            result.SetErrInfo(result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN, ex.HResult)
            printer = Nothing

            Return
        End Try

        Try
            ' Print data if printer is printable
            result.Status = Await printer.GetStatusAsync()

            If IsPrintable(result) Then
                result.Status = New PrinterStatus()

                Await printer.BeginTransactionAsync()
                isBeginTransaction = True

                result.Status = Await printer.SendDataAsync(builder, sendTimeout)
            Else
                result.SetErrInfo(result.ErrorType.RESULT_ERR_PRINTER_STATUS, EposHResult.HR_S_OK)
            End If
        Catch ex As Exception
            result.SetErrInfo(result.ErrorType.RESULT_ERR_EPOSPRINT_SEND, ex.HResult)
            result.Status = printer.GetPrinterStatus(ex.HResult)
        End Try

        If isBeginTransaction Then
            Try
                Await printer.EndTransactionAsync()
            Catch ex As Exception
                ' Do nothing
            End Try
        End If

        Try
            Await printer.ClosePrinterAsync()
            printer = Nothing
        Catch ex As Exception
            ' Do nothing
        End Try

        Return

    End Function

    Private Async Function DisplayMsg(result As Result) As Task

        ' Null check
        If result Is Nothing Then
            Return
        End If

        Dim errorMsg As String = MsgMaker.MakeErrorMessage(result)
        Dim warningMsg As String = MsgMaker.MakeWarningMessage(result)

        If Not errorMsg.Equals(String.Empty) Then
            Await ShowMsg.Show(errorMsg)
        End If

        If Not warningMsg.Equals(String.Empty) Then
            Me.Warnings_TextBox.Text = warningMsg
        End If

        errorMsg = Nothing
        warningMsg = Nothing

        Return

    End Function

    Private Function IsPrintable(result As Result) As Boolean

        If result Is Nothing Then
            Return False
        End If

        If CBool(result.Status.printerStatus And Print.ST_OFF_LINE) Then
            Return False
        End If

        If CBool(result.Status.printerStatus And Print.ST_NO_RESPONSE) Then
            Return False
        End If

        Return True

    End Function

    Private Function GetCompress(connectionType As DevType) As Compress

        If connectionType = DevType.DEVTYPE_BLUETOOTH Then
            Return Compress.COMPRESS_DEFLATE
        Else
            Return Compress.COMPRESS_NONE
        End If

    End Function

    Private Sub PrinterDiscovery_Button_Click(sender As Object, e As RoutedEventArgs)

        ' Printer Discovery Page
        Me.Frame.Navigate(GetType(PrinterDiscoveryPage), Me)

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)

        ' Run Print Sequence
        Me.IsEnabled = False

        Await Me.RunPrintSequence()

        Me.IsEnabled = True

    End Sub

End Class

