﻿Imports Windows.ApplicationModel.Resources

Imports LibEposPrint

Public Class MsgMaker

    Private Const BATTERY_NEAR_END As Integer = &H3131
    Private Const BATTERY_REAL_END As Integer = &H3130
    Private Shared _loader As ResourceLoader = New ResourceLoader()

    Public Shared Function MakeErrorMessage(msgResult As Result) As String

        Dim msg As String = ""

        If (msgResult Is Nothing) Then
            Return msg
        End If

        Select Case msgResult.ErrType
            Case Result.ErrorType.RESULT_ERR_NONE
                msg = ""

            Case Result.ErrorType.RESULT_ERR_PRINTER_STATUS
                msg = MakePrinterStatusErrorHandlingText(msgResult)

            Case Result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN
                msg = MakeEposPrintAPIErrorHandlingText(msgResult)

            Case Result.ErrorType.RESULT_ERR_EPOSPRINT_SEND
                msg = MakeEposPrintAPIErrorHandlingText(msgResult)

            Case Result.ErrorType.RESULT_ERR_EPOSPRINT_OTHER
                msg = MakeEposPrintAPIErrorHandlingText(msgResult)

            Case Result.ErrorType.RESULT_ERR_EPSONIO
                msg = MakeEpsonIoAPIErrorHandlingText(msgResult)

            Case Else
                ' Should not be reached
                msg = _loader.GetString("handlingmsg_ex_application_error")
        End Select

        Return msg

    End Function

    Public Shared Function MakeWarningMessage(msgResult As Result) As String

        Dim msg As String = ""

        If (msgResult Is Nothing) Then
            Return msg
        End If

        Dim status As Integer = msgResult.Status.printerStatus
        Dim battery As Integer = msgResult.Status.batteryStatus

        If CBool(status And Print.ST_RECEIPT_NEAR_END) Then
            msg += _loader.GetString("handlingmsg_warn_receipt_near_end")
        End If

        If battery = BATTERY_NEAR_END Then
            msg += _loader.GetString("handlingmsg_warn_battery_near_end")
        End If

        Return msg

    End Function

    Private Shared Function MakePrinterStatusErrorHandlingText(msgResult As Result) As String

        Dim retMsg As String = ""

        If (msgResult Is Nothing) Then
            Return retMsg
        End If

        Dim status As Integer = msgResult.Status.printerStatus
        Dim battery As Integer = msgResult.Status.batteryStatus

        If CBool(status And Print.ST_NO_RESPONSE) Then
            retMsg += _loader.GetString("handlingmsg_err_no_response")
        End If

        If CBool(status And Print.ST_COVER_OPEN) Then
            retMsg += _loader.GetString("handlingmsg_err_cover_open")
        End If

        If CBool(status And Print.ST_PAPER_FEED) Or CBool(status And Print.ST_PANEL_SWITCH) Then
            retMsg += _loader.GetString("handlingmsg_err_paper_feed")
        End If

        If CBool(status And Print.ST_AUTOCUTTER_ERR) Or CBool(status And Print.ST_MECHANICAL_ERR) Then
            retMsg += _loader.GetString("handlingmsg_err_autocutter")
        End If

        If CBool(status And Print.ST_UNRECOVER_ERR) Then
            retMsg += _loader.GetString("handlingmsg_err_unrecover")
        End If

        If CBool(status And Print.ST_RECEIPT_END) Then
            retMsg += _loader.GetString("handlingmsg_err_receipt_end")
        End If

        If CBool(status And Print.ST_HEAD_OVERHEAT) Then
            retMsg += _loader.GetString("handlingmsg_err_overheat")
            retMsg += _loader.GetString("handlingmsg_err_head")
        End If

        If CBool(status And Print.ST_MOTOR_OVERHEAT) Then
            retMsg += _loader.GetString("handlingmsg_err_overheat")
            retMsg += _loader.GetString("handlingmsg_err_motor")
        End If

        If CBool(status And Print.ST_BATTERY_OVERHEAT) Then
            retMsg += _loader.GetString("handlingmsg_err_overheat")
            retMsg += _loader.GetString("handlingmsg_err_battery")
        End If

        If CBool(status And Print.ST_WRONG_PAPER) Then
            retMsg += _loader.GetString("handlingmsg_err_wrong_paper")
        End If

        If battery = BATTERY_REAL_END Then
            retMsg += _loader.GetString("handlingmsg_err_battery_real_end")
        End If

        Return retMsg

    End Function

    Private Shared Function MakeEposPrintAPIErrorHandlingText(msgResult As Result) As String

        Dim retMsg As String = ""

        If (msgResult Is Nothing) Then
            Return retMsg
        End If

        Dim printerMsg As String = ""
        Dim status As Integer = msgResult.Status.printerStatus

        If msgResult.ErrHResult = EposHResult.HR_E_INVALIDARG Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = EposHResult.HR_E_ACCESSDENIED Then
            retMsg = MakeEposPrintAPIAccessDeniedHandlingText(msgResult)

        ElseIf msgResult.ErrHResult = EposHResult.HR_E_ABORT Then
            retMsg = _loader.GetString("handlingmsg_ex_timeout")

        ElseIf msgResult.ErrHResult = EposHResult.HR_E_OUTOFMEMORY Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = EposHResult.HR_E_PENDING Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = EposHResult.HR_E_FAIL Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        Else
            ' Should not be reached
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        End If

        retMsg += vbCrLf

        retMsg += _loader.GetString("handlingmsg_notice_printer_problems")
        printerMsg = MakePrinterStatusErrorHandlingText(msgResult)
        If printerMsg.Equals(String.Empty) Then
            printerMsg = _loader.GetString("handlingmsg_notice_failed")
        End If
        retMsg += printerMsg

        Return retMsg

    End Function

    Private Shared Function MakeEpsonIoAPIErrorHandlingText(msgResult As Result) As String

        Dim retMsg As String = ""
        Dim printerMsg As String = ""

        If (msgResult Is Nothing) Then
            Return retMsg
        End If

        If msgResult.ErrHResult = IoHResult.HR_E_INVALIDARG Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = IoHResult.HR_E_ACCESSDENIED Then
            retMsg = _loader.GetString("handlingmsg_ex_connect")

        ElseIf msgResult.ErrHResult = IoHResult.HR_E_ABORT Then
            retMsg = _loader.GetString("handlingmsg_ex_timeout")

        ElseIf msgResult.ErrHResult = IoHResult.HR_E_OUTOFMEMORY Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = IoHResult.HR_E_PENDING Then
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        ElseIf msgResult.ErrHResult = IoHResult.HR_E_FAIL Then
            retMsg = _loader.GetString("handlingmsg_ex_failure")

        Else
            ' Should not be reached
            retMsg = _loader.GetString("handlingmsg_ex_application_error")

        End If

        Return retMsg

    End Function

    Private Shared Function MakeEposPrintAPIAccessDeniedHandlingText(msgResult As Result) As String

        Dim retMsg As String = ""

        If (msgResult Is Nothing) Then
            Return retMsg
        End If

        Dim status As Integer = msgResult.Status.printerStatus

        Select Case msgResult.ErrType
            Case Result.ErrorType.RESULT_ERR_EPOSPRINT_OPEN
                retMsg = _loader.GetString("handlingmsg_ex_open")

            Case Result.ErrorType.RESULT_ERR_EPOSPRINT_SEND
                If (status And Print.ST_NO_RESPONSE) = Print.ST_NO_RESPONSE Then
                    retMsg = _loader.GetString("handlingmsg_ex_connect")

                ElseIf (status And Print.ST_OFF_LINE) = Print.ST_OFF_LINE Then
                    retMsg = _loader.GetString("handlingmsg_ex_off_line")

                Else
                    ' Should not be reached
                    retMsg = _loader.GetString("handlingmsg_ex_application_error")
                End If

            Case Else
                ' Should not be reached
                retMsg = _loader.GetString("handlingmsg_ex_application_error")
        End Select

        Return retMsg

    End Function

End Class
