﻿Imports Windows.UI.Core
Imports System.Threading
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class PrinterDiscoveryPage
    Inherits Page

    Private _updateListThread As Timer = Nothing
    Private _loader As ResourceLoader = New ResourceLoader
    Private _navigationHelper As Common.NavigationHelper

    Public ReadOnly Property NavigationHelper As Common.NavigationHelper

        Get
            Return Me._navigationHelper
        End Get

    End Property

    Private Async Sub NavigationHelper_LoadState(sender As Object, e As Common.LoadStateEventArgs)

        Await FindStart()

    End Sub

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)
        AddHandler Me._navigationHelper.LoadState, AddressOf NavigationHelper_LoadState

        Dim deviceType As DevType = MainPage.GetDeviceType()

        Select Case (deviceType)
            Case DevType.DEVTYPE_BLUETOOTH
                BT_RadioButton.IsChecked = True
            Case Else
                TCP_RadioButton.IsChecked = True
        End Select

    End Sub

    Private Async Sub Run(sender As Object)

        Await UpdateList()

    End Sub

    Private Async Function FindStart() As Task

        Dim errResult As Result = New Result()
        Dim displayMsg As String = ""
        Dim errorMsg As String = ""
        Dim warningMsg As String = ""

        ' Stop running finder
        While True
            Try
                Await Finder.StopAsync()
                Exit While
            Catch ex As Exception
                If ex.HResult <> EposHResult.HR_E_PENDING Then
                    Exit While
                End If
            End Try
        End While

        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If

        PrinterDiscovery_ListView.Items.Clear()

        ' Get device type and find
        Try
            If BT_RadioButton.IsChecked Then
                Await Finder.StartAsync(IoDevType.BLUETOOTH, "")
            Else
                Await Finder.StartAsync(IoDevType.TCP, "255.255.255.255")
            End If
        Catch ex As Exception
            errResult.SetErrInfo(Result.ErrorType.RESULT_ERR_EPSONIO, ex.HResult)
            errorMsg = MsgMaker.MakeErrorMessage(errResult)
            warningMsg = MsgMaker.MakeWarningMessage(errResult)

            If Not (errorMsg.Equals(String.Empty)) Then
                displayMsg += _loader.GetString("handlingmsg_notice_error")
                displayMsg += errorMsg
            End If

            If Not (warningMsg.Equals(String.Empty)) Then
                displayMsg += _loader.GetString("handlingmsg_notice_warning")
                displayMsg += errorMsg
            End If
        End Try

        If Not (errResult.ErrType = Result.ErrorType.RESULT_ERR_NONE) Then
            Await ShowMsg.Show(displayMsg)
        End If

        _updateListThread = New Timer((AddressOf Run), Nothing, 0, 1000)

        errResult = Nothing
        displayMsg = Nothing
        errorMsg = Nothing
        warningMsg = Nothing

        Return

    End Function

    Private Async Function UpdateList() As Task

        Await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, _
                                 Sub()
                                     Try
                                         Dim deviceList As DeviceInfo() = Nothing
                                         deviceList = Finder.GetDeviceInfoList(IoFilterOption.DEFAULT)
                                         If deviceList Is Nothing Then
                                             If PrinterDiscovery_ListView.Items.Count() > 0 Then
                                                 PrinterDiscovery_ListView.Items.Clear()
                                             End If
                                         ElseIf deviceList.Length <> PrinterDiscovery_ListView.Items.Count() Then
                                             PrinterDiscovery_ListView.Items.Clear()
                                             Dim i As Integer = 0
                                             For i = 0 To deviceList.Length - 1
                                                 PrinterDiscovery_ListView.Items.Add(New DisplayDeviceList(deviceList(i).PrinterName, deviceList(i).DeviceName))
                                             Next
                                         End If
                                     Catch ex As Exception
                                         Return
                                     End Try
                                 End Sub)

    End Function

    Private Function GetSelectedDevicetype()

        If BT_RadioButton.IsChecked Then
            Return DevType.DEVTYPE_BLUETOOTH
        Else
            Return DevType.DEVTYPE_TCP
        End If

    End Function

    Protected Overrides Sub OnNavigatedTo(e As NavigationEventArgs)

        _navigationHelper.OnNavigatedTo(e)

    End Sub


    Protected Overrides Async Sub OnNavigatedFrom(e As NavigationEventArgs)

        While True
            Try
                Await Finder.StopAsync()
                Exit While
            Catch ex As Exception
                If ex.HResult <> EposHResult.HR_E_PENDING Then
                    Exit While
                End If
            End Try
        End While

        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If

        _navigationHelper.OnNavigatedFrom(e)

    End Sub

    Private Sub ListView_SelectionChanged(sender As Object, e As SelectionChangedEventArgs)

        MainPage.SetSelectedPrinterDiscovery(GetSelectedDevicetype(), (PrinterDiscovery_ListView.SelectedItem).Device)

        If _updateListThread IsNot Nothing Then
            _updateListThread.Dispose()
        End If

        If Me.Frame.CanGoBack Then
            Me.Frame.GoBack()
        End If

    End Sub

    Private Async Sub RadioButton_Clicked(sender As Object, e As RoutedEventArgs) Handles TCP_RadioButton.Click, BT_RadioButton.Click

        Await FindStart()

    End Sub

End Class

Public Class DisplayDeviceList

    Public Property DeviceName As String
    Public Property Device As String

    Public Sub New(ByVal deviceName As String, ByVal device As String)

        Me.DeviceName = deviceName
        Me.Device = device

    End Sub

End Class
