#import "DiscoverView.h"
#import "ePOS-Print.h"
#import "ePOSBluetoothConnection.h"
#import "ShowMsg.h"
#import "Result.h"
#import "MsgMaker.h"

#define DISCOVERY_INTERVAL  0.5

@interface DiscoverView()
- (void)reloadPrinterView;
- (void)timerFindPrinter:(NSTimer *)timer;
- (void)findStart;
@end

@implementation DiscoverView
@synthesize printerView_;
@synthesize delegate = delegate_;
@synthesize deviceType_;
@synthesize other_;
@synthesize none_;

- (id)initWithConnectionType:(int)connectionType
{
    self = [super init];
    
    if(self) {
        printerList_ = [[NSArray alloc] init];
        connectionType_ = connectionType;
    }
    
    return self;
}

- (void)dealloc
{
    [printerList_ release];
    [printerView_ release];
    [deviceType_ release];
    [other_ release];
    [none_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.navigationController setNavigationBarHidden:NO
                                             animated:YES];
    
    self.other_ = [NSString stringWithFormat:@"Other..."];
    self.none_ = [NSString stringWithFormat:@""];

    // init device type
    switch(connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:
            deviceType_.selectedSegmentIndex = 0;
            break;
            
        case EPOS_OC_DEVTYPE_BLUETOOTH:
            deviceType_.selectedSegmentIndex = 1;
            break;
            
        default:
            break;
    }

    // find start
    [self findStart];
}

-(void) viewWillDisappear:(BOOL)animated
{
    // stop old finder
    if(timer_ != nil) {
        [timer_ invalidate];
        [timer_ release];
        timer_ = nil;
    }
    
    [EpsonIoFinder stop];

    [self setPrinterView_:nil];
    [self setDeviceType_:nil];
    
    [super viewWillDisappear:animated];
}

// get table section number(NSTableView)
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    NSInteger sectionNumber = 0;

    switch(connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:
            sectionNumber = 1;
            break;
            
        case EPOS_OC_DEVTYPE_BLUETOOTH:
            sectionNumber = 2;
            break;
            
        default:
            break;
    }
    
    return sectionNumber;
}

// get table row number(NSTableView)
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger rowNumber = 0;

    switch(connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:
            rowNumber = [printerList_ count];
            break;

        case EPOS_OC_DEVTYPE_BLUETOOTH:
            if(section == 0) {
                rowNumber = [printerList_ count];
                break;
            }
            rowNumber = 1;
            break;
            
        default:
            break;
    }

    return rowNumber;
}

// get table row(cell) item(NSTableView)
- (UITableViewCell *)tableView:(UITableView *)tableView
    cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    // create TableViewCell
    static NSString *identifier = @"basis-cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:identifier];
    if(nil == cell) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                       reuseIdentifier:identifier] autorelease];
    }

    switch(connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:
            if((indexPath.row >= 0) && (indexPath.row < [printerList_ count])) {
                cell.textLabel.text = [[printerList_ objectAtIndex:indexPath.row] printerName];
                cell.detailTextLabel.text = [[printerList_ objectAtIndex:indexPath.row] deviceName];
            }
            break;
            
        case EPOS_OC_DEVTYPE_BLUETOOTH:
            if(indexPath.section == 0) {
                if((indexPath.row >= 0) && (indexPath.row < [printerList_ count])) {
                    cell.textLabel.text = [[printerList_ objectAtIndex:indexPath.row] printerName];
                    cell.detailTextLabel.text = [[printerList_ objectAtIndex:indexPath.row] deviceName];
                }
            }
            else {
                cell.textLabel.text = self.other_;
                cell.detailTextLabel.text = self.none_;
            }
            break;
            
        default:
            break;
    }
    
    return cell;
}

// select table(row) item(NSTableView)
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch(connectionType_) {
        case EPOS_OC_DEVTYPE_TCP:
            // return settings
            if((indexPath.row >= 0) && (indexPath.row < [printerList_ count])) {
                [tableView deselectRowAtIndexPath:indexPath
                                         animated:NO];
                if(delegate_ != nil) {
                    NSString *ipAddress = [NSString stringWithString:[[printerList_ objectAtIndex:indexPath.row] deviceName]];
                    [delegate_ onSelectPrinter:EPOS_OC_DEVTYPE_TCP
                                       address:ipAddress];
                }
            }
            [self.navigationController popToRootViewControllerAnimated:YES];
            break;
            
        case EPOS_OC_DEVTYPE_BLUETOOTH:
            if(indexPath.section == 0) {
                // return settings
                if((indexPath.row >= 0) && (indexPath.row < [printerList_ count])) {
                    [tableView deselectRowAtIndexPath:indexPath
                                             animated:NO];
                    if(delegate_ != nil) {
                        NSString *macAddress = [NSString stringWithString:[[printerList_ objectAtIndex:indexPath.row] deviceName]];
                        
                        [delegate_ onSelectPrinter:EPOS_OC_DEVTYPE_BLUETOOTH address:macAddress];
                    }
                }
                
                [self.navigationController popToRootViewControllerAnimated:YES];
            }
            else {
                [self performSelectorOnMainThread:@selector(connectDevice:)
                                       withObject:nil
                                    waitUntilDone:NO];
            }
            break;
            
        default:
            break;
    }
}

-(void)connectDevice:(id)userInfo
{
    [EpsonIoFinder stop];
    
    EposBluetoothConnection *btConnection = [[EposBluetoothConnection alloc] init];
    if(btConnection == nil) {
        return;
    }
    
    NSMutableString *macAddress = [[NSMutableString alloc] init];
    int errStatus = [btConnection connectDevice:macAddress];
    
    if(errStatus == EPOS_BT_SUCCESS) {
        [delegate_ onSelectPrinter:EPOS_OC_DEVTYPE_BLUETOOTH address:(NSString *)macAddress];
        [self.navigationController popToRootViewControllerAnimated:YES];
    }
    else {
        Result *errResult = [[Result alloc] init];
        NSMutableString *displayMsg = [[NSMutableString alloc] init];
        NSString *errMsg = nil;
        NSString *warningMsg = nil;

        [self findStart];
        [self reloadPrinterView];

        [errResult setErrInfo:RESULT_ERR_EPOSBT Status:errStatus];
        errMsg = [MsgMaker makeErrorMessage:errResult];
        warningMsg = [MsgMaker makeWarningMessage:errResult];

        if(errMsg.length > 0) {
            [displayMsg appendString:NSLocalizedString(@"handlingmsg_notice_error", @"")];
            [displayMsg appendString:errMsg];
        }
        if(warningMsg.length > 0) {
            [displayMsg appendString:NSLocalizedString(@"handlingmsg_notice_warning", @"")];
            [displayMsg appendString:warningMsg];
        }

        if(displayMsg.length > 0) {
            [ShowMsg show:self Message:displayMsg];
        }

        [errResult release];
        [displayMsg release];
        [errMsg release];
        [warningMsg release];
    }

    [btConnection release];
    [macAddress release];
}


// update printe view(call find thread)
- (void)reloadPrinterView
{
    [printerView_ reloadData];
}

// change connection type
- (IBAction)changeConnectionType:(id)sender
{
    switch(deviceType_.selectedSegmentIndex) {
        case 0:
            connectionType_ = EPOS_OC_DEVTYPE_TCP;
            break;
            
        default :
            connectionType_ = EPOS_OC_DEVTYPE_BLUETOOTH;
            break;
    }
    
    [self reloadPrinterView];
    [self findStart];
}

// find printer timer
- (void)timerFindPrinter:(NSTimer *)timer
{
    int result = 0;
    NSArray *items = [EpsonIoFinder getDeviceInfoList:&result
                                         FilterOption:EPSONIO_OC_PARAM_DEFAULT];
    
    if((items != nil) && (result == EPSONIO_OC_SUCCESS)) {
        if([printerList_ count] != [items count]) {
            [printerList_ release];
            printerList_ = [items retain];
            
            [self performSelectorOnMainThread:@selector(reloadPrinterView)
                                   withObject:nil
                                waitUntilDone:YES];
        }
    }
}

// find start/restart
- (void)findStart
{
    // stop old finder
    [EpsonIoFinder stop];

    // stop find timer
    if(timer_ != nil) {
        [timer_ invalidate];
        [timer_ release];
        timer_ = nil;
    }

    // clear list
    [printerList_ release];
    
    printerList_ = [[NSArray alloc] init];
    if(printerView_ != nil) {
        [printerView_ reloadData];
    }

    // find device type and find option
    int connectionType;
    NSString *option = nil;
    switch(deviceType_.selectedSegmentIndex) {
        case 0:
            connectionType = EPSONIO_OC_DEVTYPE_TCP;
            option = @"255.255.255.255";
            break;
            
        default:
            connectionType = EPSONIO_OC_DEVTYPE_BLUETOOTH;
            break;
    }

    // find start
    int result = [EpsonIoFinder start:connectionType
                           FindOption:option];
    
    if(result != EPSONIO_OC_SUCCESS) {
        Result *errResult = [[Result alloc] init];
        NSMutableString *displayMsg = [[NSMutableString alloc] init];
        NSString *errMsg = nil;
        NSString *warningMsg = nil;

        [errResult setErrInfo:RESULT_ERR_EPSONIO
                       Status:result];
        
        errMsg = [MsgMaker makeErrorMessage:errResult];
        warningMsg = [MsgMaker makeWarningMessage:errResult];

        if(errMsg.length > 0) {
            [displayMsg appendString:NSLocalizedString(@"handlingmsg_notice_error", @"")];
            [displayMsg appendString:errMsg];
        }
        
        if(warningMsg.length > 0) {
            [displayMsg appendString:NSLocalizedString(@"handlingmsg_notice_warning", @"")];
            [displayMsg appendString:warningMsg];
        }

        if(displayMsg.length > 0) {
            [ShowMsg show:self Message:displayMsg];
        }

        [errResult release];
        [displayMsg release];
        [errMsg release];
        [warningMsg release];

        return ;
    }

    // start timer
    timer_ = [[NSTimer scheduledTimerWithTimeInterval:DISCOVERY_INTERVAL
                                               target:self
                                             selector:@selector(timerFindPrinter:)
                                             userInfo:nil
                                              repeats:YES] retain];
}

@end
