#import "MainView.h"
#import "DiscoverView.h"
#import "ShowMsg.h"
#import "MsgMaker.h"
#import "PickerModelsItem.h"

@interface MainView()
- (void)runPrintSequence;
- (EposBuilder *)createReceiptData:(Result *)result;
- (void)print:(EposBuilder *)builder Result:(Result *)result;
- (void)displayMsg:(Result *)result;
- (BOOL)isPrintable:(Result *)result;
- (int)getCompress:(int)connection;
@end

@implementation MainView
@synthesize scrollView_;
@synthesize itemView_;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil
                           bundle:nibBundleOrNil];
    
    if(self) {
        openDeviceName_ = @"192.168.192.168";
        connectionType_ = EPOS_OC_DEVTYPE_TCP;
        printerModel_ = EPOS_OC_MODEL_ANK;
        printerName_ = @"TM-T88V";
    }

    return self;
}

- (void)dealloc
{
    [scrollView_ release];
    [openDeviceName_ release];
    [printerName_ release];
    [itemView_ release];
    [nameList_ release];
    [modelList_ release];
    [_buttonDiscovery release];
    [_buttonName release];
    [_buttonModel release];
    [_buttonPrint release];
    [_textWarnings release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    // init printer name list
    nameList_ = [[PickerTableView alloc] init];
    NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"printername_m10", @"")];
	[items addObject:NSLocalizedString(@"printername_m30", @"")];
    [items addObject:NSLocalizedString(@"printername_p20", @"")];
    [items addObject:NSLocalizedString(@"printername_p60", @"")];
    [items addObject:NSLocalizedString(@"printername_p60ii", @"")];
    [items addObject:NSLocalizedString(@"printername_p80", @"")];
    [items addObject:NSLocalizedString(@"printername_t20", @"")];
    [items addObject:NSLocalizedString(@"printername_t20ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t70", @"")];
    [items addObject:NSLocalizedString(@"printername_t70ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t81ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t82", @"")];
    [items addObject:NSLocalizedString(@"printername_t82ii", @"")];
    [items addObject:NSLocalizedString(@"printername_t88v", @"")];
    [items addObject:NSLocalizedString(@"printername_t90ii", @"")];
    [items addObject:NSLocalizedString(@"printername_u220", @"")];
    [items addObject:NSLocalizedString(@"printername_u330", @"")];

    [nameList_ setItemList:items];
    
    [_buttonName setTitle:[nameList_ getItem:0]
                 forState:UIControlStateNormal];
    
    nameList_.delegate = self;

    // init printer model list
    modelList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];

    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_ank", @"")
                                                    modelConstant:EPOS_OC_MODEL_ANK] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_japanese", @"")
                                                    modelConstant:EPOS_OC_MODEL_JAPANESE] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_chinese", @"")
                                                    modelConstant:EPOS_OC_MODEL_CHINESE] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_taiwan", @"")
                                                    modelConstant:EPOS_OC_MODEL_TAIWAN] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_korean", @"")
                                                    modelConstant:EPOS_OC_MODEL_KOREAN] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_thai", @"")
                                                    modelConstant:EPOS_OC_MODEL_THAI] autorelease]];
    
    [items addObject:[[[PickerModelsItem alloc] initWithModelData:NSLocalizedString(@"model_southasia", @"")
                                                    modelConstant:EPOS_OC_MODEL_SOUTHASIA] autorelease]];

    [modelList_ setItemList:items];
    
    [_buttonModel setTitle:[modelList_ getItem:0]
                  forState:UIControlStateNormal];
    
    modelList_.delegate = self;

    // auto scroll setting
    [scrollView_ setContentSize:itemView_.frame.size];
}

- (void)runPrintSequence
{
    Result *result = nil;
    EposBuilder *builder = nil;

    // Run print sequence of sample receipt
    _textWarnings.text = @"";
    
    result = [[Result alloc] init];
    
    builder = [self createReceiptData:result];

    if(result.errType == RESULT_ERR_NONE) {
        [self print:builder
             Result:result];
    }
    
    if(builder != nil) {
        [builder clearCommandBuffer];
        
        [builder release];
    }

    [self displayMsg:result];

    if(result != nil) {
        [result release];
    }
    
    return;
}

- (EposBuilder *)createReceiptData:(Result *)result
{
    int errStatus = EPOS_OC_SUCCESS;
    EposBuilder *builder = nil;
    
    // nil check
    if(result == nil){
        return nil;
    }
    
    // Top logo data
    NSString *path = [[NSBundle mainBundle] pathForResource:@"store"
                                                     ofType:@"png"];
    // nil check
    if(path == nil) {
        return  nil;
    }
    
    // addBarcode API settings
    const int barcodeWidth = 2;
    const int barcodeHeight = 100;
    
    // init result
    result.printerStatus = 0;
    result.batteryStatus = 0;
    result.errStatus = 0;
    result.errType = RESULT_ERR_NONE;

    // Create builder
    builder = [[EposBuilder alloc] initWithPrinterModel:printerName_
                                                   Lang:printerModel_];
    if(builder == nil) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:EPOS_OC_ERR_FAILURE];
        return nil;
    }

    // Set alignment to center
    errStatus = [builder addTextAlign:EPOS_OC_ALIGN_CENTER];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        return nil;
    }
    
    UIImage *logoData = [[UIImage alloc] initWithContentsOfFile:path];
    if(logoData == nil){
        return nil;
    }

    // Add top logo to command buffer
    errStatus = [builder addImage:logoData
                                X:0
                                Y:0
                            Width:logoData.size.width
                           Height:logoData.size.height
                            Color:EPOS_OC_COLOR_1
                             Mode:EPOS_OC_MODE_MONO
                         Halftone:EPOS_OC_HALFTONE_DITHER
                       Brightness:EPOS_OC_PARAM_DEFAULT
                         Compress:[self getCompress:connectionType_]];
    
    [logoData release];
    
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];

        [builder release];
        
        
        return nil;
    }

    // Add receipt text to command buffer
    // Section 1 : Store infomation
    errStatus = [builder addFeedLine:1];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];

        [builder release];
        return nil;
    }
    
    // Text Buffer
    NSMutableString *textData = [[NSMutableString alloc] init];
    if(textData == nil){
        [builder release];
        return nil;
    }
    
    [textData appendString:@"THE STORE 1234 (555) 555-5555\n"];
    [textData appendString:@"STORE DIRECTOR - John Smith\n"];
    [textData appendString:@"\n"];
    [textData appendString:@"7/01/07 16:58 6153 05 0191 134\n"];
    [textData appendString:@"ST# 21 OP# 001 TE# 01 TR# 747\n"];
    [textData appendString:@"------------------------------\n"];
    
    errStatus = [builder addText:textData];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    // Section 2 : Purchaced items
    [textData setString:@""];
    
    [textData appendString:@"004 OHEIDA 3PK SPRINGF  9.99 R\n"];
    [textData appendString:@"104 3 CUP BLK TEAPOT    9.99 R\n"];
    [textData appendString:@"451 EMERIL GRIDDLE/PAN 17.99 R\n"];
    [textData appendString:@"389 CANDYMAKER ASSORT   4.99 R\n"];
    [textData appendString:@"740 TRIPOD              8.99 R\n"];
    [textData appendString:@"334 BLK LOGO PRNTED ZO  7.99 R\n"];
    [textData appendString:@"581 AQUA MICROTERRY SC  6.99 R\n"];
    [textData appendString:@"934 30L BLK FF DRESS   16.99 R\n"];
    [textData appendString:@"075 LEVITATING DESKTOP  7.99 R\n"];
    [textData appendString:@"412 **Blue Overprint P  2.99 R\n"];
    [textData appendString:@"762 REPOSE 4PCPM CHOC   5.49 R\n"];
    [textData appendString:@"613 WESTGATE BLACK 25  59.99 R\n"];
    [textData appendString:@"------------------------------\n"];
    errStatus = [builder addText:textData];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    // Section 3 : Payment infomation
    [textData setString:@""];
    
    [textData appendString:@"SUBTOTAL                160.38\n"];
    [textData appendString:@"TAX                      14.43\n"];
    errStatus = [builder addText:textData];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    [textData setString:@""];

    errStatus = [builder addTextDouble:EPOS_OC_TRUE
                                    Dh:EPOS_OC_TRUE];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    errStatus = [builder addText:@"TOTAL    174.81\n"];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    errStatus = [builder addTextDouble:EPOS_OC_FALSE
                                    Dh:EPOS_OC_FALSE];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    errStatus = [builder addFeedLine:1];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    [textData setString:@""];
    [textData appendString:@"CASH                    200.00\n"];
    [textData appendString:@"CHANGE                   25.19\n"];
    [textData appendString:@"------------------------------\n"];
    errStatus = [builder addText:textData];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    // Section 4 : Advertisement
    [textData setString:@""];
    
    [textData appendString:@"TotalNumber of Items Purchased\n"];
    [textData appendString:@"Sign Up and Save !\n"];
    [textData appendString:@"With Preferred Saving Card\n"];
    
    errStatus = [builder addText:textData];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        [textData release];
        return nil;
    }
    
    [textData setString:@""];
    
    // release temporary text object
    [textData release];
    
    errStatus = [builder addFeedLine:2];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        return nil;
    }

    // Add barcode data to command buffer
    errStatus = [builder addBarcode:@"01209457"
                               Type:EPOS_OC_BARCODE_CODE39
                                Hri:EPOS_OC_HRI_BELOW
                               Font:EPOS_OC_FONT_A
                              Width:barcodeWidth
                             Height:barcodeHeight];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        return nil;
    }

    // Add command to cut receipt to command buffer
    errStatus = [builder addCut:EPOS_OC_CUT_FEED];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [builder release];
        return nil;
    }

    return builder;
}

- (void)print:(EposBuilder *)builder Result:(Result *)result
{
    EposPrint *printer = nil;
    int errStatus = EPOS_OC_SUCCESS;
    unsigned long status = 0;
    unsigned long battery = 0;
    // sendData API timeout setting(10000 msec)
    const int sendTimeout = 10000;

    // nil check
    if((builder == nil) || (result == nil)) {
        return;
    }

    // init result
    result.printerStatus = 0;
    result.batteryStatus = 0;
    result.errStatus = 0;
    result.errType = RESULT_ERR_NONE;

    // do printing
    printer = [[EposPrint alloc] init];
    if(printer == nil) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:EPOS_OC_ERR_FAILURE];
        return;
    }
    
    errStatus = [printer openPrinter:connectionType_
                          DeviceName:openDeviceName_
                             Enabled:EPOS_OC_FALSE
                            Interval:EPOS_OC_PARAM_DEFAULT
                             Timeout:EPOS_OC_PARAM_DEFAULT];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT Status:errStatus];
        [printer release];
        return;
    }

    errStatus = [printer getStatus:&status
                           Battery:&battery];
    if(errStatus != EPOS_OC_SUCCESS) {
        [result setErrInfo:RESULT_ERR_EPOSPRINT
                    Status:errStatus];
        
        [printer closePrinter];
        [printer release];
        return;
    }

    result.printerStatus = status;
    result.batteryStatus = battery;

    if([self isPrintable:result]) {
        status = 0;
        battery = 0;

        errStatus = [printer beginTransaction];
        if(errStatus != EPOS_OC_SUCCESS) {
            [result setErrInfo:RESULT_ERR_EPOSPRINT
                        Status:errStatus];
            
            [printer closePrinter];
            [printer release];
            return;
        }

        errStatus = [printer sendData:builder
                              Timeout:sendTimeout
                               Status:&status
                              Battery:&battery];
        
        result.printerStatus = status;
        result.batteryStatus = battery;
        
        if(errStatus != EPOS_OC_SUCCESS) {
            [result setErrInfo:RESULT_ERR_EPOSPRINT
                        Status:errStatus];
            
            [printer endTransaction];
            [printer closePrinter];
            [printer release];
            return;
        }

        errStatus = [printer endTransaction];
        if(errStatus != EPOS_OC_SUCCESS) {
            [result setErrInfo:RESULT_ERR_EPOSPRINT
                        Status:errStatus];
            
            [printer closePrinter];
            [printer release];
            return;
        }
    }

    // close and release printer object
    [printer closePrinter];
    
    [printer release];

    return;
}

- (void)displayMsg:(Result *)result
{
    // nil check
    if(result == nil) {
        return;
    }

    NSString *errorMsg = [MsgMaker makeErrorMessage:result];
    if(errorMsg != nil){
        if([errorMsg length] > 0) {
            [ShowMsg show:self Message:errorMsg];
        }
        [errorMsg release];
    }
    
    NSString *warningMsg = [MsgMaker makeWarningMessage:result];
    if(warningMsg != nil){
        if([warningMsg length] > 0) {
            _textWarnings.text = warningMsg;
        }
        [warningMsg release];
    }
    
    return;
}

- (BOOL)isPrintable:(Result *)result
{
    // nil check
    if(result == nil) {
        return NO;
    }

    if(IS_INCLUDE_STATUS(result.printerStatus, EPOS_OC_ST_OFF_LINE)) {
        return NO;
    }
    
    if(IS_INCLUDE_STATUS(result.printerStatus, EPOS_OC_ST_NO_RESPONSE)) {
        return NO;
    }
    
    return YES;
}

- (int)getCompress:(int)connection
{
    if(connection == EPOS_OC_DEVTYPE_BLUETOOTH) {
        return EPOS_OC_COMPRESS_DEFLATE;
    }
    else {
        return EPOS_OC_COMPRESS_NONE;
    }
}

- (void)viewDidUnload
{
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setButtonDiscovery:nil];
    [self setButtonName:nil];
    [self setButtonModel:nil];
    [self setButtonPrint:nil];
    [self setTextWarnings:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:YES
                                             animated:YES];

}

// called DiscoverView
- (void)onSelectPrinter:(int)connectionType address:(NSString *)ipAddress
{
    [openDeviceName_ release];
    openDeviceName_ = [ipAddress retain];
    connectionType_ = connectionType;
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj
{
    NSString *tmpStr = [[[NSString alloc] init] autorelease];

    if(obj == nameList_) {
        tmpStr = [nameList_ getItem:position];
        
        [_buttonName setTitle:tmpStr
                     forState:UIControlStateNormal];
        
        printerName_ = tmpStr;
    }
    else if(obj == modelList_) {
        PickerModelsItem *item = [modelList_ getObject:position];
        [_buttonModel setTitle:item.modelName
                      forState:UIControlStateNormal];
        
        printerModel_ = item.modelConstant;
        
        item = nil;
    }
    else {
        /* Do nothing */
    }
}

// click button
- (IBAction)eventButtonDidPush:(id)sender
{
    UIViewController *view = nil;
    switch(((UIView *)sender).tag) {
        // Printer Discovery/Select Button
        case 1:
            view = [[[DiscoverView alloc] initWithConnectionType:connectionType_] autorelease];
            ((DiscoverView *)view).delegate = self;
            break;

        // Printer Name Picker Activate Button
        case 2:
            [nameList_ show];
            break;

        // Printer Model Picker Activate Button
        case 3:
            [modelList_ show];
            break;

        // Print Button
        case 4:
            [self runPrintSequence];
            break;
    }
    
    if(view != nil) {
        [self.navigationController pushViewController:view
                                             animated:YES];
    }
}

@end
